@@ -49,6 +49,23 @@
 #include "register.h"
 #include "num_stdio.h"
 
+/* HAVE_SSLEEP is defined when Windows Sleep is found
+ * HAVE_SLEEP is defined when POSIX sleep is found
+ * _WIN32 is defined when compiling with MinGW
+ *
+ * When cross-compiling from POSIX to Windows using MinGW, HAVE_SLEEP
+ * will often be defined by configure although it is not supported by
+ * MinGW.  So substitute the sleep definition below in such a case and
+ * when compiling on Windows using MinGW where HAVE_SLEEP will be
+ * undefined.
+ *
+ * FIXME:  Needs better handling for all versions of MinGW.
+ *
+ */
+#if (defined(HAVE_SSLEEP) || defined(_WIN32)) && (!defined(HAVE_SLEEP))
+#include "hl_sleep.h"
+#endif
+
 // ---------------------------------------------------------------------------
 //    ADAT INCLUDES
 // ---------------------------------------------------------------------------
@@ -594,7 +611,7 @@ size_t trimwhitespace( char *out, size_t len, const char *str )
     }
 
     // Trim leading space
-    while(isspace(*str)) str++;
+    while(isspace((int)*str)) str++;
 
     if(*str == 0)  // All spaces?
     {
@@ -605,7 +622,7 @@ size_t trimwhitespace( char *out, size_t len, const char *str )
 
     // Trim trailing space
     end = (char *)(str + strlen(str) - 1);
-    while(end > str && isspace(*end))
+    while(end > str && isspace((int)*end))
     {
       *end = '\0';
       end--;
@@ -732,7 +749,7 @@ int adat_parse_freq( char                    *pcStr,
 
             // Get Freq Value from response string
 
-            while(( isalpha( *pcEnd ) == 0 ) ||
+            while(( isalpha( (int)*pcEnd ) == 0 ) ||
                   ( *pcEnd == '.' ))
             {
                 acValueBuf[ nI++ ] = *pcEnd;
@@ -748,7 +765,7 @@ int adat_parse_freq( char                    *pcStr,
             // Get Freq Unit from response string
 
             nI = 0;
-            while( isalpha( *pcEnd ) != 0 )
+            while( isalpha( (int)*pcEnd ) != 0 )
             {
                 acUnitBuf[ nI++ ] = *pcEnd;
                 pcEnd += sizeof( char );
@@ -3809,4 +3826,3 @@ DECLARE_PROBERIG_BACKEND(adat)
 // ---------------------------------------------------------------------------
 // END OF FILE
 // ---------------------------------------------------------------------------
-
