@@ -33,6 +33,7 @@
 
 #include "hamlib/rig.h"
 #include "serial.h"
+#include "parallel.h"
 #include "misc.h"
 #include "tones.h"
 #include "idx_builtin.h"
@@ -336,7 +337,11 @@ static int dummy_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
   		rig_strvfo(vfo), rig_strrmode(mode), buf);
 
   curr->mode = mode;
-  curr->width = width;
+
+  if (width == RIG_PASSBAND_NORMAL)
+    curr->width = rig_passband_normal(rig, mode);
+  else
+    curr->width = width;
 
   return RIG_OK;
 }
@@ -408,10 +413,22 @@ static int dummy_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
 static int dummy_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
 {
   struct dummy_priv_data *priv = (struct dummy_priv_data *)rig->state.priv;
+  int status = 0;
+  ptt_t par_status = RIG_PTT_OFF;
 
   rig_debug(RIG_DEBUG_VERBOSE, "%s called\n", __FUNCTION__);
   *ptt = priv->ptt;
 
+  // sneak a look at the hardware PTT and OR that in with our result
+  // as if it had keyed us
+  switch (rig->state.pttport.type.ptt)
+    {
+    case RIG_PTT_SERIAL_DTR: ser_get_dtr (&rig->state.pttport, &status); break;
+    case RIG_PTT_SERIAL_RTS: ser_get_rts (&rig->state.pttport, &status); break;
+    case RIG_PTT_PARALLEL: par_ptt_get (&rig->state.pttport, &par_status); break;
+    default: break;
+    }
+  *ptt = *ptt || status || RIG_PTT_ON == par_status;
   return RIG_OK;
 }
 
@@ -1410,12 +1427,16 @@ static int dummy_mW2power(RIG * rig, float *power, unsigned int mwpower,
  * Dummy rig capabilities.
  */
 
-#define DUMMY_FUNC  (((setting_t)-1)&~(0xc1000000U))	/* has it all */
-#define DUMMY_LEVEL (((setting_t)-1)&~(1<<27))
-#define DUMMY_PARM  ((setting_t)-1)
+/*
+ * The following macros set bitmasks for the various funcs, levels, parms,
+ * etc.  This dummy backend claims support for almost all of them.
+ */
+#define DUMMY_FUNC  ((setting_t)-1UL) /* All possible functions */
+#define DUMMY_LEVEL (((setting_t)-1UL)&~(1UL<<27)) /* All levels except SQLSTAT */
+#define DUMMY_PARM  ((setting_t)-1UL) /* All possible parms */
 
-#define DUMMY_VFO_OP  0x7ffffffL
-#define DUMMY_SCAN	0x7ffffffL
+#define DUMMY_VFO_OP  0x7ffffffUL /* All possible VFO OPs */
+#define DUMMY_SCAN    0x7ffffffUL /* All possible scan OPs */
 
 #define DUMMY_VFOS (RIG_VFO_A|RIG_VFO_B|RIG_VFO_MEM)
 
