@@ -132,6 +132,7 @@ static int ic706_r2i_mode(RIG *rig, rmode_t mode, pbwidth_t width,
 static const struct icom_priv_caps ic706_priv_caps = {
 		0x48,	/* default address */
 		0,		/* 731 mode */
+    0,    /* no XCHG */
 		ic706_ts_sc_list,
 		.r2i_mode = ic706_r2i_mode
 };
@@ -155,7 +156,7 @@ const struct rig_caps ic706_caps = {
 .serial_handshake =  RIG_HANDSHAKE_NONE,
 .write_delay =  0,
 .post_write_delay =  0,
-.timeout =  200,
+.timeout =  1000,
 .retry =  3,
 .has_get_func =  RIG_FUNC_NONE,
 .has_set_func =  RIG_FUNC_NONE,
@@ -264,6 +265,7 @@ const struct rig_caps ic706_caps = {
 static const struct icom_priv_caps ic706mkii_priv_caps = {
 		0x4e,	/* default address */
 		0,		/* 731 mode */
+    0,    /* no XCHG */
 		ic706_ts_sc_list,
 		.r2i_mode = ic706_r2i_mode
 };
@@ -287,7 +289,7 @@ const struct rig_caps ic706mkii_caps = {
 .serial_handshake =  RIG_HANDSHAKE_NONE,
 .write_delay =  0,
 .post_write_delay =  0,
-.timeout =  200,
+.timeout =  1000,
 .retry =  3,
 .has_get_func =  RIG_FUNC_NONE,
 .has_set_func =  RIG_FUNC_NONE,
@@ -419,6 +421,7 @@ const struct rig_caps ic706mkii_caps = {
 static const struct icom_priv_caps ic706mkiig_priv_caps = {
 		0x58,	/* default address */
 		0,		/* 731 mode */
+    0,    /* no XCHG */
 		ic706_ts_sc_list,
 		.r2i_mode = ic706_r2i_mode
 };
@@ -442,7 +445,7 @@ const struct rig_caps ic706mkiig_caps = {
 .serial_handshake =  RIG_HANDSHAKE_NONE,
 .write_delay =  0,
 .post_write_delay =  0,
-.timeout =  200,
+.timeout =  1000,
 .retry =  3,
 .has_get_func =  IC706IIG_FUNC_ALL,
 .has_set_func =  IC706IIG_FUNC_ALL,
@@ -569,8 +572,6 @@ const struct rig_caps ic706mkiig_caps = {
 .set_mem =  icom_set_mem,
 .vfo_op =  icom_vfo_op,
 .scan =  icom_scan,
-.set_ptt =  icom_set_ptt,
-.get_ptt =  icom_get_ptt,
 .get_dcd =  icom_get_dcd,
 .set_ts =  icom_set_ts,
 .set_rptr_shift =  icom_set_rptr_shift,
