@@ -46,7 +46,7 @@
 #define IC756_OTHER_TX_MODES (RIG_MODE_CW|RIG_MODE_SSB|RIG_MODE_RTTY|RIG_MODE_FM)
 #define IC756_AM_TX_MODES (RIG_MODE_AM)
 
-#define IC756PRO_FUNC_ALL (RIG_FUNC_FAGC|RIG_FUNC_NB|RIG_FUNC_COMP|RIG_FUNC_VOX|RIG_FUNC_TONE|RIG_FUNC_TSQL|RIG_FUNC_SBKIN|RIG_FUNC_FBKIN|RIG_FUNC_NR|RIG_FUNC_MON|RIG_FUNC_MN|RIG_FUNC_RF)
+#define IC756PRO_FUNC_ALL (RIG_FUNC_FAGC|RIG_FUNC_NB|RIG_FUNC_COMP|RIG_FUNC_VOX|RIG_FUNC_TONE|RIG_FUNC_TSQL|RIG_FUNC_SBKIN|RIG_FUNC_FBKIN|RIG_FUNC_NR|RIG_FUNC_MON|RIG_FUNC_MN|RIG_FUNC_RF|RIG_FUNC_ANF)
 
 #define IC756PRO_LEVEL_ALL (RIG_LEVEL_PREAMP|RIG_LEVEL_ATT|RIG_LEVEL_AGC|RIG_LEVEL_COMP|RIG_LEVEL_BKINDL|RIG_LEVEL_BALANCE|RIG_LEVEL_NR|RIG_LEVEL_PBT_IN|RIG_LEVEL_PBT_OUT|RIG_LEVEL_CWPITCH|RIG_LEVEL_RFPOWER|RIG_LEVEL_MICGAIN|RIG_LEVEL_KEYSPD|RIG_LEVEL_NOTCHF|RIG_LEVEL_RAWSTR|RIG_LEVEL_AF|RIG_LEVEL_RF|RIG_LEVEL_SQL|RIG_LEVEL_COMP|RIG_LEVEL_BALANCE)
 
@@ -54,7 +54,7 @@
  * this model.
  */
 
-#define IC756_VFO_ALL (RIG_VFO_A|RIG_VFO_B|RIG_VFO_MEM)
+#define IC756_VFO_ALL (RIG_VFO_MAIN|RIG_VFO_SUB|RIG_VFO_MEM)
 
 #define IC756_VFO_OPS (RIG_OP_CPY|RIG_OP_XCHG|RIG_OP_FROM_VFO|RIG_OP_TO_VFO|RIG_OP_MCL)
 #define IC756_SCAN_OPS (RIG_SCAN_MEM|RIG_SCAN_VFO|RIG_SCAN_PROG|RIG_SCAN_DELTA|RIG_SCAN_PRIO)
@@ -63,31 +63,59 @@
 
 #define IC756PRO_STR_CAL { 16, \
 	{ \
-		{   0, -18 }, \
-		{  10, -16 }, \
-		{  27, -14 }, \
-		{  45, -12 }, \
-		{  60, -10 }, \
-		{  76, -8 }, \
-		{  89, -6 }, \
-		{ 100, -4 }, \
-		{ 110, -2 }, \
-		{ 120, 0 }, \
-		{ 125, 2 }, \
-		{ 129, 4 }, \
-		{ 133, 6 }, \
-		{ 138, 8 }, \
-		{ 142, 10 }, \
-		{ 146, 12 } \
+		 { 0,   -60 }, \
+		 { 14,  -52 }, \
+		 { 29,  -44 }, \
+		 { 44,  -36 }, \
+		 { 60,  -28 }, \
+		 { 75,  -20 }, \
+		 { 91,  -12 }, \
+		 { 107, -4 }, \
+		 { 124,  4 }, \
+		 { 141, 12 }, \
+		 { 158, 20 }, \
+		 { 175, 28 }, \
+		 { 192, 36 }, \
+		 { 210, 44 }, \
+		 { 228, 52 }, \
+		 { 247 ,60 } \
 	} }
 
+/*
+ *  This function deals with the older type radios with only 2 filter widths
+ *  (0 - normal, 1 - narrow)
+ */
+static int r2i_mode(RIG *rig, rmode_t mode, pbwidth_t width,
+		    unsigned char *md, signed char *pd)
+{
+  int err;
+
+  err = rig2icom_mode(rig, mode, width, md, pd);
+
+  if (err != RIG_OK)
+    return err;
+
+  if (PD_WIDE_3 == *pd)
+    {
+      *pd = PD_MEDIUM_2;
+    }
+  else if (*pd > PD_WIDE_3)
+    {
+      (*pd)--;
+    }
+
+  return RIG_OK;
+}
+
 /*
  * ic756 rig capabilities.
  */
 static const struct icom_priv_caps ic756_priv_caps = {
 		0x50,	/* default address */
 		0,		/* 731 mode */
-		ic756_ts_sc_list
+    0,    /* no XCHG */
+		ic756_ts_sc_list,
+		.r2i_mode = r2i_mode
 };
 
 const struct rig_caps ic756_caps = {
@@ -109,7 +137,7 @@ const struct rig_caps ic756_caps = {
 .serial_handshake =  RIG_HANDSHAKE_NONE,
 .write_delay =  0,
 .post_write_delay =  0,
-.timeout =  200,
+.timeout =  1000,
 .retry =  3,
 .has_get_func =  RIG_FUNC_NONE,
 .has_set_func =  RIG_FUNC_NONE,
@@ -170,11 +198,12 @@ const struct rig_caps ic756_caps = {
 	/* mode/filter list, remember: order matters! */
 .filters = 	{
 		{RIG_MODE_SSB|RIG_MODE_RTTY, kHz(2.4)},
+		{RIG_MODE_CW, kHz(2.4)},
 		{RIG_MODE_CW, Hz(500)},
-		{RIG_MODE_AM, kHz(8)},
+		{RIG_MODE_AM, kHz(9)},
 		{RIG_MODE_AM, kHz(2.4)},
 		{RIG_MODE_FM, kHz(15)},
-		{RIG_MODE_FM, kHz(8)},
+		{RIG_MODE_FM, kHz(9)},
 		RIG_FLT_END,
 	},
 .str_cal = IC756PRO_STR_CAL,
@@ -226,6 +255,7 @@ const struct rig_caps ic756_caps = {
 static const struct icom_priv_caps ic756pro_priv_caps = {
 		0x5c,	/* default address */
 		0,		/* 731 mode */
+    0,    /* no XCHG */
 		ic756pro_ts_sc_list
 };
 
@@ -248,7 +278,7 @@ const struct rig_caps ic756pro_caps = {
 .serial_handshake =  RIG_HANDSHAKE_NONE,
 .write_delay =  0,
 .post_write_delay =  0,
-.timeout =  200,
+.timeout =  1000,
 .retry =  3,
 .has_get_func =  IC756PRO_FUNC_ALL,
 .has_set_func =  IC756PRO_FUNC_ALL,
@@ -373,6 +403,7 @@ const struct rig_caps ic756pro_caps = {
 static const struct icom_priv_caps ic756pro2_priv_caps = {
 		0x64,	/* default address */
 		0,		/* 731 mode */
+    0,    /* no XCHG */
 		ic756pro_ts_sc_list
 };
 
@@ -452,7 +483,7 @@ const struct rig_caps ic756pro2_caps = {
 .serial_handshake =  RIG_HANDSHAKE_NONE,
 .write_delay =  0,
 .post_write_delay =  0,
-.timeout =  200,
+.timeout =  1000,
 .retry =  3,
 .has_get_func =  IC756PRO_FUNC_ALL,
 .has_set_func =  IC756PRO_FUNC_ALL,
@@ -535,8 +566,8 @@ const struct rig_caps ic756pro2_caps = {
 
 .set_freq =  icom_set_freq,
 .get_freq =  icom_get_freq,
-.set_mode =  icom_set_mode,
-.get_mode =  icom_get_mode,
+.set_mode =  icom_set_mode_with_data,
+.get_mode =  icom_get_mode_with_data,
 .set_vfo =  icom_set_vfo,
 .set_ant =  icom_set_ant,
 .get_ant =  icom_get_ant,
@@ -731,6 +762,7 @@ static int ic756pro2_get_ext_parm(RIG *rig, token_t token, value_t *val)
 static const struct icom_priv_caps ic756pro3_priv_caps = {
 		0x6e,	/* default address */
 		0,		/* 731 mode */
+    0,    /* no XCHG */
 		ic756pro_ts_sc_list
 };
 
@@ -740,8 +772,6 @@ static const struct icom_priv_caps ic756pro3_priv_caps = {
 #define IC756PROIII_OTHER_TX_MODES (RIG_MODE_CW|RIG_MODE_CWR|RIG_MODE_SSB|RIG_MODE_RTTY|RIG_MODE_RTTYR|RIG_MODE_FM)
 #define IC756PROIII_AM_TX_MODES (RIG_MODE_AM)
 
-#define IC756PROIII_VFO_ALL (RIG_VFO_MAIN|RIG_VFO_SUB|RIG_VFO_MEM)
-
 #define IC756PROIII_PARMS (RIG_PARM_ANN|RIG_PARM_BACKLIGHT|RIG_PARM_APO|RIG_PARM_TIME|RIG_PARM_BEEP)
 
 
@@ -785,7 +815,7 @@ const struct rig_caps ic756pro3_caps = {
 .serial_handshake =  RIG_HANDSHAKE_NONE,
 .write_delay =  0,
 .post_write_delay =  0,
-.timeout =  200,
+.timeout =  1000,
 .retry =  3,
 .has_get_func =  IC756PRO_FUNC_ALL,
 .has_set_func =  IC756PRO_FUNC_ALL,
@@ -818,22 +848,22 @@ const struct rig_caps ic756pro3_caps = {
 	   RIG_CHAN_END,
 	},
 
-.rx_range_list1 =   { {kHz(30),MHz(60),IC756PROIII_ALL_RX_MODES,-1,-1,IC756PROIII_VFO_ALL,IC756_ANTS},
+.rx_range_list1 =   { {kHz(30),MHz(60),IC756PROIII_ALL_RX_MODES,-1,-1,IC756_VFO_ALL,IC756_ANTS},
 	RIG_FRNG_END, },
 .tx_range_list1 =   {
-	FRQ_RNG_HF(1,IC756PROIII_OTHER_TX_MODES, W(5),W(100),IC756PROIII_VFO_ALL,IC756_ANTS),
-	FRQ_RNG_6m(1,IC756PROIII_OTHER_TX_MODES, W(5),W(100),IC756PROIII_VFO_ALL,IC756_ANTS),
-	FRQ_RNG_HF(1,IC756PROIII_AM_TX_MODES, W(5),W(40),IC756PROIII_VFO_ALL,IC756_ANTS),   /* AM class */
-	FRQ_RNG_6m(1,IC756PROIII_AM_TX_MODES, W(5),W(40),IC756PROIII_VFO_ALL,IC756_ANTS),   /* AM class */
+	FRQ_RNG_HF(1,IC756PROIII_OTHER_TX_MODES, W(5),W(100),IC756_VFO_ALL,IC756_ANTS),
+	FRQ_RNG_6m(1,IC756PROIII_OTHER_TX_MODES, W(5),W(100),IC756_VFO_ALL,IC756_ANTS),
+	FRQ_RNG_HF(1,IC756PROIII_AM_TX_MODES, W(5),W(40),IC756_VFO_ALL,IC756_ANTS),   /* AM class */
+	FRQ_RNG_6m(1,IC756PROIII_AM_TX_MODES, W(5),W(40),IC756_VFO_ALL,IC756_ANTS),   /* AM class */
     	RIG_FRNG_END, },
 
-.rx_range_list2 =   { {kHz(30),MHz(60),IC756PROIII_ALL_RX_MODES,-1,-1,IC756PROIII_VFO_ALL,IC756_ANTS},
+.rx_range_list2 =   { {kHz(30),MHz(60),IC756PROIII_ALL_RX_MODES,-1,-1,IC756_VFO_ALL,IC756_ANTS},
 	RIG_FRNG_END, },
 .tx_range_list2 =  {
-	FRQ_RNG_HF(2,IC756PROIII_OTHER_TX_MODES, W(5),W(100),IC756PROIII_VFO_ALL,IC756_ANTS),
-	FRQ_RNG_6m(2,IC756PROIII_OTHER_TX_MODES, W(5),W(100),IC756PROIII_VFO_ALL,IC756_ANTS),
-	FRQ_RNG_HF(2,IC756PROIII_AM_TX_MODES, W(5),W(40),IC756PROIII_VFO_ALL,IC756_ANTS),   /* AM class */
-	FRQ_RNG_6m(2,IC756PROIII_AM_TX_MODES, W(5),W(40),IC756PROIII_VFO_ALL,IC756_ANTS),   /* AM class */
+	FRQ_RNG_HF(2,IC756PROIII_OTHER_TX_MODES, W(5),W(100),IC756_VFO_ALL,IC756_ANTS),
+	FRQ_RNG_6m(2,IC756PROIII_OTHER_TX_MODES, W(5),W(100),IC756_VFO_ALL,IC756_ANTS),
+	FRQ_RNG_HF(2,IC756PROIII_AM_TX_MODES, W(5),W(40),IC756_VFO_ALL,IC756_ANTS),   /* AM class */
+	FRQ_RNG_6m(2,IC756PROIII_AM_TX_MODES, W(5),W(40),IC756_VFO_ALL,IC756_ANTS),   /* AM class */
     	RIG_FRNG_END, },
 
 .tuning_steps = 	{
@@ -875,8 +905,8 @@ const struct rig_caps ic756pro3_caps = {
 
 .set_freq =  icom_set_freq,
 .get_freq =  icom_get_freq,
-.set_mode =  icom_set_mode,
-.get_mode =  icom_get_mode,
+.set_mode =  icom_set_mode_with_data,
+.get_mode =  icom_get_mode_with_data,
 .set_vfo =  icom_set_vfo,
 .set_ant =  icom_set_ant,
 .get_ant =  icom_get_ant,
