@@ -1,6 +1,6 @@
 /*
  *  Hamlib CI-V backend - main file
- *  Copyright (c) 2000-2012 by Stephane Fillod
+ *  Copyright (c) 2000-2015 by Stephane Fillod
  *
  *
  *   This library is free software; you can redistribute it and/or
@@ -193,6 +193,23 @@ const struct ts_sc_list ic7000_ts_sc_list[] = {
 	{ 0, 0 },
 };
 
+const struct ts_sc_list ic7100_ts_sc_list[] = {
+	{ 10,			0x00 },
+	{ 100,			0x01 },
+	{ kHz(1),		0x02 },
+	{ kHz(5),		0x03 },
+	{ kHz(6.25),		0x04 },
+	{ kHz(9),		0x05 },
+	{ kHz(10),		0x06 },
+	{ kHz(12.5),		0x07 },
+	{ kHz(20),		0x08 },
+	{ kHz(25),		0x09 },
+	{ kHz(50),		0x0A },
+	{ kHz(100),		0x0B },
+	{ MHz(1),		0x0C },
+	{ 0,			0x00 },
+};
+
 const struct ts_sc_list ic7200_ts_sc_list[] = {
 	{ 10, 0x00 },
 	{ 100, 0x01 },
@@ -238,6 +255,7 @@ struct icom_addr {
 
 #define TOK_CIVADDR TOKEN_BACKEND(1)
 #define TOK_MODE731 TOKEN_BACKEND(2)
+#define TOK_NOXCHG TOKEN_BACKEND(3)
 
 const struct confparams icom_cfg_params[] = {
 	{ TOK_CIVADDR, "civaddr", "CI-V address", "Transceiver's CI-V address",
@@ -247,6 +265,9 @@ const struct confparams icom_cfg_params[] = {
 			"data length, needed for IC731 and IC735",
 			"0", RIG_CONF_CHECKBUTTON
 	},
+	{ TOK_NOXCHG, "no_xchg", "No VFO XCHG", "Don't Use VFO XCHG to set other VFO mode and Frequency",
+			"0", RIG_CONF_CHECKBUTTON
+	},
 	{ RIG_CONF_END, NULL, }
 };
 
@@ -315,6 +336,7 @@ static const struct icom_addr icom_addr_list[] = {
 	{ RIG_MODEL_OS535, 0x80 }, /* same address as IC-7410 */
 	{ RIG_MODEL_ICID1, 0x01 },
 	{ RIG_MODEL_IC7000, 0x70 },
+	{ RIG_MODEL_IC7100, 0x88 },
 	{ RIG_MODEL_IC7200, 0x76 },
 	{ RIG_MODEL_IC7700, 0x74 },
 	{ RIG_MODEL_NONE, 0 },
@@ -360,6 +382,7 @@ int icom_init(RIG *rig)
 
 	priv->re_civ_addr = priv_caps->re_civ_addr;
 	priv->civ_731_mode = priv_caps->civ_731_mode;
+	priv->no_xchg = priv_caps->no_xchg;
 
 	return RIG_OK;
 }
@@ -511,6 +534,7 @@ pbwidth_t icom_get_dsp_flt(RIG *rig, rmode_t mode) {
 	int retval, res_len, rfstatus;
 	unsigned char resbuf[MAXFRAMELEN];
 	value_t rfwidth;
+  unsigned char fw_sub_cmd = RIG_MODEL_IC7200 == rig->caps->rig_model ? 0x02 : S_MEM_FILT_WDTH;
 
 	if (rig_has_get_func(rig, RIG_FUNC_RF) && (mode & (RIG_MODE_RTTY | RIG_MODE_RTTYR))) {
         if(!rig_get_func(rig, RIG_VFO_CURR, RIG_FUNC_RF, &rfstatus) && (rfstatus)) {
@@ -521,7 +545,7 @@ pbwidth_t icom_get_dsp_flt(RIG *rig, rmode_t mode) {
             return rtty_fil[rfwidth.i];
 		}
 	}
-	retval = icom_transaction (rig, C_CTL_MEM, S_MEM_FILT_WDTH, 0, 0,
+	retval = icom_transaction (rig, C_CTL_MEM, fw_sub_cmd, 0, 0,
 			resbuf, &res_len);
 	if (retval != RIG_OK) {
 		rig_debug(RIG_DEBUG_ERR,"%s: protocol error (%#.2x), "
@@ -548,6 +572,7 @@ int icom_set_dsp_flt(RIG *rig, rmode_t mode, pbwidth_t width) {
 	unsigned char flt_ext;
 	value_t rfwidth;
 	int ack_len=sizeof(ackbuf), flt_idx;
+  unsigned char fw_sub_cmd = RIG_MODEL_IC7200 == rig->caps->rig_model ? 0x02 : S_MEM_FILT_WDTH;
 
     if (width == RIG_PASSBAND_NORMAL)
         width = rig_passband_normal(rig, mode);
@@ -576,7 +601,7 @@ int icom_set_dsp_flt(RIG *rig, rmode_t mode, pbwidth_t width) {
 
     to_bcd(&flt_ext, flt_idx, 2);
 
-	retval = icom_transaction (rig, C_CTL_MEM, S_MEM_FILT_WDTH, &flt_ext, 1,
+	retval = icom_transaction (rig, C_CTL_MEM, fw_sub_cmd, &flt_ext, 1,
 			ackbuf, &ack_len);
 	if (retval != RIG_OK) {
 		rig_debug(RIG_DEBUG_ERR,"%s: protocol error (%#.2x), "
@@ -592,6 +617,61 @@ int icom_set_dsp_flt(RIG *rig, rmode_t mode, pbwidth_t width) {
 	return RIG_OK;
 }
 
+/*
+ * icom_set_mode_with_data
+ */
+int icom_set_mode_with_data (RIG * rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
+{
+  int retval;
+  unsigned char datamode;
+  unsigned char ackbuf[MAXFRAMELEN];
+  int ack_len=sizeof(ackbuf);
+  rmode_t icom_mode;
+  unsigned char dm_sub_cmd = RIG_MODEL_IC7200 == rig->caps->rig_model ? 0x04 : S_MEM_DATA_MODE;
+
+  switch (mode)
+    {
+    case RIG_MODE_PKTUSB: icom_mode = RIG_MODE_USB; break;
+    case RIG_MODE_PKTLSB: icom_mode = RIG_MODE_LSB; break;
+    case RIG_MODE_PKTFM: icom_mode = RIG_MODE_FM; break;
+    default: icom_mode = mode; break;
+    };
+
+  retval = icom_set_mode (rig, vfo, icom_mode, width);
+
+  if (RIG_OK == retval)
+    {
+      if (RIG_MODE_PKTUSB == mode || RIG_MODE_PKTLSB == mode || RIG_MODE_PKTFM == mode)
+        {
+          datamode = 0x01;      /* some rigs (e.g. IC-7700 & IC-7800)
+                                   have D1/2/3 but we cannot know
+                                   which to set so just set D1 */
+        }
+      else
+        {
+          datamode = 0x00;
+        }
+
+      retval = icom_transaction (rig, C_CTL_MEM, dm_sub_cmd, &datamode, 1,
+                                 ackbuf, &ack_len);
+      if (retval != RIG_OK)
+        {
+          rig_debug(RIG_DEBUG_ERR,"%s: protocol error (%#.2x), "
+                    "len=%d\n", __FUNCTION__,ackbuf[0],ack_len);
+        }
+      else
+        {
+          if (ack_len != 1 || ackbuf[0] != ACK)
+            {
+              rig_debug(RIG_DEBUG_ERR,"%s: command not supported ? (%#.2x), "
+                        "len=%d\n", __FUNCTION__,ackbuf[0],ack_len);
+            }
+        }
+    }
+
+  return retval;
+}
+
 /*
  * icom_set_mode
  * Assumes rig!=NULL, rig->state.priv!=NULL
@@ -612,45 +692,107 @@ int icom_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 	priv_caps = (const struct icom_priv_caps *) rig->caps->priv;
 
 	if (priv_caps->r2i_mode != NULL) {	/* call priv code if defined */
-	    	err = priv_caps->r2i_mode(rig, mode, width,
-				&icmode, &icmode_ext);
+    err = priv_caps->r2i_mode(rig, mode, width,
+                              &icmode, &icmode_ext);
 	}
 	else {					/* else call default */
 		err = rig2icom_mode(rig, mode, width,
-				&icmode, &icmode_ext);
+                        &icmode, &icmode_ext);
 	}
 
 	if (err < 0)
 		return err;
 
 	/* IC-731 and IC-735 don't support passband data */
-	if (priv->civ_731_mode || rig->caps->rig_model == RIG_MODEL_OS456)
+	/* IC-726 & IC-475A/E also limited support - only on CW */
+	/* TODO: G4WJS CW wide/narrow are possible with above two radios */
+	if (priv->civ_731_mode || rig->caps->rig_model == RIG_MODEL_OS456
+	    || rig->caps->rig_model == RIG_MODEL_IC726
+	    || rig->caps->rig_model == RIG_MODEL_IC475)
 		icmode_ext = -1;
 
 	retval = icom_transaction (rig, C_SET_MODE, icmode, (unsigned char *) &icmode_ext,
-                                   (icmode_ext == -1 ? 0 : 1), ackbuf, &ack_len);
+                             (icmode_ext == -1 ? 0 : 1), ackbuf, &ack_len);
 	if (retval != RIG_OK)
 		return retval;
 
 	if (ack_len != 1 || ackbuf[0] != ACK) {
 		rig_debug(RIG_DEBUG_ERR,"icom_set_mode: ack NG (%#.2x), "
-					"len=%d\n", ackbuf[0],ack_len);
+              "len=%d\n", ackbuf[0],ack_len);
 		return -RIG_ERJCTED;
 	}
 
 #if 0
-    /* Tentative DSP filter setting ($1A$03), but not supported by every rig,
-     * and some models like IC910/Omni VI Plus have a different meaning for
-     * this subcommand
-     */
+  /* Tentative DSP filter setting ($1A$03), but not supported by every rig,
+   * and some models like IC910/Omni VI Plus have a different meaning for
+   * this subcommand
+   */
 	if ( (rig->caps->rig_model != RIG_MODEL_IC910) &&
-	    (rig->caps->rig_model != RIG_MODEL_OMNIVIP) )
-        icom_set_dsp_flt(rig, mode, width);
+       (rig->caps->rig_model != RIG_MODEL_OMNIVIP) )
+    icom_set_dsp_flt(rig, mode, width);
 #endif
 
 	return RIG_OK;
 }
 
+/*
+ * icom_get_mode_with_data
+ *
+ * newer Icom rigs support data mode with ACC-1 audio input and MIC muted
+ */
+int icom_get_mode_with_data(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
+{
+  unsigned char databuf[MAXFRAMELEN];
+  int data_len, retval;
+  unsigned char dm_sub_cmd = RIG_MODEL_IC7200 == rig->caps->rig_model ? 0x04 : S_MEM_DATA_MODE;
+
+  retval = icom_get_mode (rig, vfo, mode, width);
+
+  if (RIG_OK == retval && (RIG_MODE_USB == *mode || RIG_MODE_LSB == *mode || RIG_MODE_FM == *mode))
+    {
+      /*
+       * fetch data mode on/off
+       */
+      retval = icom_transaction (rig, C_CTL_MEM, dm_sub_cmd, 0, 0,
+                                 databuf, &data_len);
+      if (retval != RIG_OK)
+        {
+          rig_debug(RIG_DEBUG_ERR,"%s: protocol error (%#.2x), "
+                    "len=%d\n", __FUNCTION__, databuf[0], data_len);
+          return -RIG_ERJCTED;
+        }
+
+      /*
+       * databuf should contain Cn,Sc,D0[,D1]
+       */
+      data_len -= 2;
+      if (1 > data_len || data_len > 2)	/* manual says 1 byte answer
+                                           but at least IC756 ProIII
+                                           sends 2 - second byte
+                                           appears to be same as
+                                           second byte from 04 command
+                                           which is filter preset
+                                           number, whatever it is we
+                                           ignore it */
+        {
+          rig_debug(RIG_DEBUG_ERR,"%s: wrong frame len=%d\n",
+                    __FUNCTION__, data_len);
+          return -RIG_ERJCTED;
+        }
+      if (databuf[2])	/* 0x01/0x02/0x03 -> data mode, 0x00 -> not data mode */
+        {
+          switch (*mode)
+            {
+            case RIG_MODE_USB: *mode = RIG_MODE_PKTUSB; break;
+            case RIG_MODE_LSB: *mode = RIG_MODE_PKTLSB; break;
+            case RIG_MODE_FM: *mode = RIG_MODE_PKTFM; break;
+            default: break;
+            }
+        }
+    }
+  return retval;
+}
+
 /*
  * icom_get_mode
  * Assumes rig!=NULL, rig->state.priv!=NULL, mode!=NULL, width!=NULL
@@ -1228,6 +1370,9 @@ int icom_set_conf(RIG *rig, token_t token, const char *val)
 	case TOK_MODE731:
 		priv->civ_731_mode = atoi(val) ? 1:0;
 		break;
+	case TOK_NOXCHG:
+		priv->no_xchg = atoi(val) ? 1:0;
+		break;
 	default:
 		return -RIG_EINVAL;
 	}
@@ -1254,6 +1399,9 @@ int icom_get_conf(RIG *rig, token_t token, char *val)
 	case TOK_MODE731:
 		sprintf(val, "%d", priv->civ_731_mode);
 		break;
+	case TOK_NOXCHG:
+		sprintf(val, "%d", priv->no_xchg);
+		break;
 	default:
 		return -RIG_EINVAL;
 	}
@@ -1297,6 +1445,7 @@ int icom_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
 
 	retval = icom_transaction (rig, C_CTL_PTT, S_PTT, NULL, 0,
 					pttbuf, &ptt_len);
+
 	if (retval != RIG_OK)
 		return retval;
 
@@ -1522,9 +1671,14 @@ int icom_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
 {
 	int status;
 	vfo_t rx_vfo, tx_vfo;
+	struct icom_priv_data *priv;
+	struct rig_state *rs;
+
+	rs = &rig->state;
+	priv = (struct icom_priv_data*)rs->priv;
 
 	/* This method works also in memory mode(RIG_VFO_MEM) */
-	if (rig_has_vfo_op(rig, RIG_OP_XCHG)) {
+	if (!priv->no_xchg && rig_has_vfo_op(rig, RIG_OP_XCHG)) {
 		status = icom_vfo_op(rig, vfo, RIG_OP_XCHG);
 		if (status != RIG_OK)
 			return status;
@@ -1569,9 +1723,14 @@ int icom_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
 {
 	int status;
 	vfo_t rx_vfo, tx_vfo;
+	struct icom_priv_data *priv;
+	struct rig_state *rs;
+
+	rs = &rig->state;
+	priv = (struct icom_priv_data*)rs->priv;
 
 	/* This method works also in memory mode(RIG_VFO_MEM) */
-	if (rig_has_vfo_op(rig, RIG_OP_XCHG)) {
+	if (!priv->no_xchg && rig_has_vfo_op(rig, RIG_OP_XCHG)) {
 		status = icom_vfo_op(rig, vfo, RIG_OP_XCHG);
 		if (status != RIG_OK)
 			return status;
@@ -1616,14 +1775,19 @@ int icom_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width
 {
 	int status;
 	vfo_t rx_vfo, tx_vfo;
+	struct icom_priv_data *priv;
+	struct rig_state *rs;
+
+	rs = &rig->state;
+	priv = (struct icom_priv_data*)rs->priv;
 
 	/* This method works also in memory mode(RIG_VFO_MEM) */
-	if (rig_has_vfo_op(rig, RIG_OP_XCHG)) {
+	if (!priv->no_xchg && rig_has_vfo_op(rig, RIG_OP_XCHG)) {
 		status = icom_vfo_op(rig, vfo, RIG_OP_XCHG);
 		if (status != RIG_OK)
 			return status;
 
-		status = icom_set_mode(rig, RIG_VFO_CURR, tx_mode, tx_width);
+		status = rig->caps->set_mode(rig, RIG_VFO_CURR, tx_mode, tx_width);
 		if (status != RIG_OK)
 			return status;
 
@@ -1642,7 +1806,7 @@ int icom_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width
 	if (status != RIG_OK)
 		return status;
 
-	status = icom_set_mode(rig, RIG_VFO_CURR, tx_mode, tx_width);
+	status = rig->caps->set_mode(rig, RIG_VFO_CURR, tx_mode, tx_width);
 	if (status != RIG_OK)
 		return status;
 
@@ -1664,14 +1828,19 @@ int icom_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *tx_mode, pbwidth_t *tx_wid
 {
 	int status;
 	vfo_t rx_vfo, tx_vfo;
+	struct icom_priv_data *priv;
+	struct rig_state *rs;
+
+	rs = &rig->state;
+	priv = (struct icom_priv_data*)rs->priv;
 
 	/* This method works also in memory mode(RIG_VFO_MEM) */
-	if (rig_has_vfo_op(rig, RIG_OP_XCHG)) {
+	if (!priv->no_xchg && rig_has_vfo_op(rig, RIG_OP_XCHG)) {
 		status = icom_vfo_op(rig, vfo, RIG_OP_XCHG);
 		if (status != RIG_OK)
 			return status;
 
-		status = icom_get_mode(rig, RIG_VFO_CURR, tx_mode, tx_width);
+		status = rig->caps->get_mode(rig, RIG_VFO_CURR, tx_mode, tx_width);
 		if (status != RIG_OK)
 			return status;
 
@@ -1690,7 +1859,7 @@ int icom_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *tx_mode, pbwidth_t *tx_wid
 	if (status != RIG_OK)
 		return status;
 
-	status = icom_get_mode(rig, RIG_VFO_CURR, tx_mode, tx_width);
+	status = rig->caps->get_mode(rig, RIG_VFO_CURR, tx_mode, tx_width);
 	if (status != RIG_OK)
 		return status;
 
@@ -1897,17 +2066,20 @@ int icom_set_func(RIG *rig, vfo_t vfo, setting_t func, int status)
 	unsigned char fctbuf[MAXFRAMELEN], ackbuf[MAXFRAMELEN];
 	int fct_len, acklen, retval;
 	int fct_cn, fct_sc;		/* Command Number, Subcommand */
+	/* r8500, the problem rig */
+	int r8500 = (rig->caps->rig_model == RIG_MODEL_ICR8500)? 1 : 0;
 
 	/*
 	 * except for IC-R8500
 	 */
 	fctbuf[0] = status? 0x01:0x00;
-	fct_len = rig->caps->rig_model == RIG_MODEL_ICR8500 ? 0 : 1;
+	fct_len = r8500 ? 0 : 1;
 
 	switch (func) {
 	case RIG_FUNC_FAGC:
 		fct_cn = C_CTL_FUNC;
-		fct_sc = S_FUNC_AGC;
+		fct_sc = (r8500)?(status)?S_FUNC_AGCON:S_FUNC_AGCOFF:S_FUNC_AGC;
+		/* fct_sc = S_FUNC_AGC; */
 		/* note: should it be a LEVEL only, and no func? --SF */
 		if (status != 0)
 			fctbuf[0] = 0x03;	/* default to 0x03 in IC746 pro super-fast */
@@ -1916,7 +2088,8 @@ int icom_set_func(RIG *rig, vfo_t vfo, setting_t func, int status)
 		break;
 	case RIG_FUNC_NB:
 		fct_cn = C_CTL_FUNC;
-		fct_sc = S_FUNC_NB;
+		fct_sc = (r8500)?(status)?S_FUNC_NBON:S_FUNC_NBOFF:S_FUNC_NB;
+		/* fct_sc = S_FUNC_NB; */
 		break;
 	case RIG_FUNC_COMP:
 		fct_cn = C_CTL_FUNC;
@@ -1960,7 +2133,8 @@ int icom_set_func(RIG *rig, vfo_t vfo, setting_t func, int status)
 		break;
 	case RIG_FUNC_APF:
 		fct_cn = C_CTL_FUNC;
-		fct_sc = S_FUNC_APF;
+		fct_sc = (r8500)?(status)?S_FUNC_APFON:S_FUNC_APFOFF:S_FUNC_APF;
+		/* fct_sc = S_FUNC_APF; */
 		break;
 	case RIG_FUNC_MON:
 		fct_cn = C_CTL_FUNC;
@@ -3155,6 +3329,7 @@ DECLARE_INITRIG_BACKEND(icom)
 	rig_register(&ic78_caps);
 	rig_register(&ic7800_caps);
 	rig_register(&ic7000_caps);
+	rig_register(&ic7100_caps);
 	rig_register(&ic7200_caps);
 	rig_register(&ic781_caps);
 	rig_register(&ic707_caps);
@@ -3192,6 +3367,7 @@ DECLARE_INITRIG_BACKEND(icom)
 
 	rig_register(&ic92d_caps);
 	rig_register(&id1_caps);
+	rig_register(&id5100_caps);
 
 	return RIG_OK;
 }
