@@ -1,22 +1,22 @@
 /*
  *  Hamlib Interface - list of known rigs
  *  Copyright (c) 2000-2003 by Frank Singleton
- *  Copyright (c) 2000-2011 by Stephane Fillod
+ *  Copyright (c) 2000-2015 by Stephane Fillod
  *
  *
- *   This library is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2 of
- *   the License, or (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License as published by the Free Software Foundation; either
+ *   version 2.1 of the License, or (at your option) any later version.
  *
- *   This program is distributed in the hope that it will be useful,
+ *   This library is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU Library General Public License for more details.
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *   Lesser General Public License for more details.
  *
- *   You should have received a copy of the GNU Library General Public
+ *   You should have received a copy of the GNU Lesser General Public
  *   License along with this library; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -95,6 +95,8 @@
 #define RIG_MODEL_FT980 RIG_MAKE_MODEL(RIG_YAESU, 31)
 #define RIG_MODEL_FTDX5000 RIG_MAKE_MODEL(RIG_YAESU, 32)
 #define RIG_MODEL_VX1700 RIG_MAKE_MODEL(RIG_YAESU, 33)
+#define RIG_MODEL_FT1200 RIG_MAKE_MODEL(RIG_YAESU, 34)
+#define RIG_MODEL_FT991 RIG_MAKE_MODEL(RIG_YAESU, 35)
 
 	/*
 	 * Kenwood
@@ -134,6 +136,12 @@
 #define RIG_MODEL_TS590S RIG_MAKE_MODEL(RIG_KENWOOD, 31)
 #define RIG_MODEL_TRANSFOX RIG_MAKE_MODEL(RIG_KENWOOD, 32)  /* SigFox Transfox */
 #define RIG_MODEL_THD72A RIG_MAKE_MODEL(RIG_KENWOOD, 33)
+#define RIG_MODEL_TMD710 RIG_MAKE_MODEL(RIG_KENWOOD, 34)
+#define RIG_MODEL_TMV71 RIG_MAKE_MODEL(RIG_KENWOOD, 35)
+#define RIG_MODEL_F6K RIG_MAKE_MODEL(RIG_KENWOOD, 36) /* Flex 6000 Series */
+#define RIG_MODEL_TS590SG RIG_MAKE_MODEL(RIG_KENWOOD, 37)
+#define RIG_MODEL_XG3 RIG_MAKE_MODEL(RIG_KENWOOD, 38)
+#define RIG_MODEL_TS990S RIG_MAKE_MODEL(RIG_KENWOOD, 39)
 
 	/*
 	 * Icom
@@ -200,7 +208,10 @@
 #define RIG_MODEL_ICR9500 RIG_MAKE_MODEL(RIG_ICOM, 66)
 #define RIG_MODEL_IC7410 RIG_MAKE_MODEL(RIG_ICOM, 67)
 #define RIG_MODEL_IC9100 RIG_MAKE_MODEL(RIG_ICOM, 68)
-/* next one is 69 */
+#define RIG_MODEL_ICRX7 RIG_MAKE_MODEL(RIG_ICOM, 69)
+#define RIG_MODEL_IC7100 RIG_MAKE_MODEL(RIG_ICOM, 70)
+#define RIG_MODEL_ID5100 RIG_MAKE_MODEL(RIG_ICOM, 71)
+/* next one is 72 */
 
 	/*
 	 * Optoelectronics (CI-V)
@@ -388,6 +399,7 @@
 #define RIG_MODEL_TT585 RIG_MAKE_MODEL(RIG_TENTEC, 9)	/* Paragon */
 #define RIG_MODEL_TT588 RIG_MAKE_MODEL(RIG_TENTEC, 11)	/* Omni-VII */
 #define RIG_MODEL_RX331 RIG_MAKE_MODEL(RIG_TENTEC, 12)
+#define RIG_MODEL_TT599 RIG_MAKE_MODEL(RIG_TENTEC, 13)  /* Eagle */
 
 	/*
 	 * Alinco
@@ -404,17 +416,6 @@
 #define RIG_MODEL_505DSP RIG_MAKE_MODEL(RIG_KACHINA, 1)
 
 
-/*! \def RIG_MODEL_RPC
- *  \brief A macro that returns the model number of the RPC Network pseudo-backend.
- *
- *  The RPC backend can be used to connect and send commands to a rig server,
- *  \c rpc.rigd, running on a remote machine. Using this client/server scheme,
- *  several clients can control and monitor the same rig hardware.
- */
-#define RIG_RPC 19
-#define RIG_BACKEND_RPC "rpcrig"
-#define RIG_MODEL_RPC RIG_MAKE_MODEL(RIG_RPC, 1)
-
 	/*
 	 * Gnuradio backend
 	 */
@@ -450,6 +451,7 @@
 #define RIG_MODEL_SDR1000 RIG_MAKE_MODEL(RIG_FLEXRADIO, 1)
 #define RIG_MODEL_SDR1000RFE RIG_MAKE_MODEL(RIG_FLEXRADIO, 2)
 #define RIG_MODEL_DTTSP RIG_MAKE_MODEL(RIG_FLEXRADIO, 3)
+#define RIG_MODEL_DTTSP_UDP RIG_MAKE_MODEL(RIG_FLEXRADIO, 4)
 
 
 	/*
@@ -477,6 +479,11 @@
 #define RIG_MODEL_SI570PICUSB RIG_MAKE_MODEL(RIG_KIT, 11) /* SoftRock Si570 PIC */
 #define RIG_MODEL_FIFISDR RIG_MAKE_MODEL(RIG_KIT, 12) /* FiFi-SDR USB */
 #define RIG_MODEL_FUNCUBEDONGLE RIG_MAKE_MODEL(RIG_KIT, 13)	/* FunCUBE Dongle */
+#define RIG_MODEL_HIQSDR RIG_MAKE_MODEL(RIG_KIT, 14)	/* HiQSDR */
+#define RIG_MODEL_FASDR RIG_MAKE_MODEL(RIG_KIT,15) /* Funkamateur Sdr */
+#define RIG_MODEL_SI570PEABERRY1 RIG_MAKE_MODEL(RIG_KIT, 16) /* Peaberry V1 */
+#define RIG_MODEL_SI570PEABERRY2 RIG_MAKE_MODEL(RIG_KIT, 17) /* Peaberry V2 */
+#define RIG_MODEL_FUNCUBEDONGLEPLUS RIG_MAKE_MODEL(RIG_KIT, 18)	/* FunCUBE Dongle Pro+ */
 
 	/*
 	 * SW/FM/TV tuner cards supported by Video4Linux,*BSD, ..
@@ -502,6 +509,23 @@
 #define RIG_MODEL_PRM8060 RIG_MAKE_MODEL(RIG_PRM80, 1)
 #define RIG_MODEL_PRM8070 RIG_MAKE_MODEL(RIG_PRM80, 2)
 
+        /*
+         * ADAT by HB9CBU
+         *
+         * ADDED: frgo (DG1SBG), 2012-01-01
+         */
+#define RIG_ADAT 29
+#define RIG_BACKEND_ADAT "adat"
+#define RIG_MODEL_ADT_200A RIG_MAKE_MODEL(RIG_ADAT, 1)
+
+        /*
+         * ICOM Marine
+         */
+#define RIG_ICMARINE 30
+#define RIG_BACKEND_ICMARINE "icmarine"
+#define RIG_MODEL_IC_M700PRO RIG_MAKE_MODEL(RIG_ICMARINE, 1)
+#define RIG_MODEL_IC_M802 RIG_MAKE_MODEL(RIG_ICMARINE, 2)
+#define RIG_MODEL_IC_M710 RIG_MAKE_MODEL(RIG_ICMARINE, 3)
 
 	/*
 	 * TODO:
@@ -515,43 +539,6 @@
 */
 typedef int rig_model_t;
 
-
-/*! \def RIG_BACKEND_LIST
- *  \brief Static list of rig models.
- *
- *  This is a NULL terminated list of available rig backends. Each entry
- *  in the list consists of two fields: The branch number, which is an integer,
- *  and the branch name, which is a character string.
- */
-#define RIG_BACKEND_LIST {		\
-		{ RIG_DUMMY, RIG_BACKEND_DUMMY }, \
-		{ RIG_YAESU, RIG_BACKEND_YAESU }, \
-		{ RIG_KENWOOD, RIG_BACKEND_KENWOOD }, \
-		{ RIG_ICOM, RIG_BACKEND_ICOM }, \
-		{ RIG_PCR, RIG_BACKEND_PCR }, \
-		{ RIG_AOR, RIG_BACKEND_AOR }, \
-		{ RIG_JRC, RIG_BACKEND_JRC }, \
-		{ RIG_UNIDEN, RIG_BACKEND_UNIDEN }, \
-		{ RIG_DRAKE, RIG_BACKEND_DRAKE }, \
-		{ RIG_LOWE, RIG_BACKEND_LOWE }, \
-		{ RIG_RACAL, RIG_BACKEND_RACAL }, \
-		{ RIG_WJ, RIG_BACKEND_WJ }, \
-		{ RIG_SKANTI, RIG_BACKEND_SKANTI }, \
-		{ RIG_WINRADIO, RIG_BACKEND_WINRADIO }, \
-		{ RIG_TENTEC, RIG_BACKEND_TENTEC }, \
-		{ RIG_ALINCO, RIG_BACKEND_ALINCO }, \
-		{ RIG_KACHINA, RIG_BACKEND_KACHINA }, \
-		{ RIG_RPC, RIG_BACKEND_RPC }, \
-		{ RIG_TAPR, RIG_BACKEND_TAPR }, \
-		{ RIG_FLEXRADIO, RIG_BACKEND_FLEXRADIO }, \
-		{ RIG_RFT, RIG_BACKEND_RFT }, \
-		{ RIG_KIT, RIG_BACKEND_KIT }, \
-		{ RIG_TUNER, RIG_BACKEND_TUNER }, \
-		{ RIG_RS, RIG_BACKEND_RS }, \
-		{ RIG_PRM80, RIG_BACKEND_PRM80 }, \
-		{ 0, NULL }, /* end */  \
-}
-
 /*
  * struct rig_backend_list {
  *		rig_model_t model;
