@@ -4,19 +4,19 @@
  *  Copyright (c) 2000-2002 by Frank Singleton
  *
  *
- *   This library is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2 of
- *   the License, or (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License as published by the Free Software Foundation; either
+ *   version 2.1 of the License, or (at your option) any later version.
  *
- *   This program is distributed in the hope that it will be useful,
+ *   This library is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU Library General Public License for more details.
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *   Lesser General Public License for more details.
  *
- *   You should have received a copy of the GNU Library General Public
+ *   You should have received a copy of the GNU Lesser General Public
  *   License along with this library; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -67,20 +67,6 @@
 #define ROT_MODEL_DUMMY ROT_MAKE_MODEL(ROT_DUMMY, 1)
 #define ROT_MODEL_NETROTCTL ROT_MAKE_MODEL(ROT_DUMMY, 2)
 
-	/*
-	 * RPC Network pseudo-backend
-	 */
-/*! \def ROT_MODEL_RPC
- *  \brief A macro that returns the model number of the RPC Network pseudo-backend.
- *
- *  The RPC backend can be used to connect and send commands to a rotator server,
- *  \c rpc.rotd, running on a remote machine. Using this client/server scheme,
- *  several clients can control and monitor the same rotator hardware.
- */
-#define ROT_RPC 1
-#define ROT_BACKEND_RPC "rpcrot"
-#define ROT_MODEL_RPC ROT_MAKE_MODEL(ROT_RPC, 1)
-
 	/*
 	 * Easycomm
 	 */
@@ -100,6 +86,8 @@
 #define ROT_BACKEND_EASYCOMM "easycomm"
 #define ROT_MODEL_EASYCOMM1 ROT_MAKE_MODEL(ROT_EASYCOMM, 1)
 #define ROT_MODEL_EASYCOMM2 ROT_MAKE_MODEL(ROT_EASYCOMM, 2)
+#define ROT_MODEL_TRAKBOX ROT_MAKE_MODEL(ROT_EASYCOMM, 3)
+#define ROT_MODEL_EASYCOMM3 ROT_MAKE_MODEL(ROT_EASYCOMM, 4)
 
 /*! \def ROT_MODEL_FODTRACK
  *  \brief A macro that returns the model number of the Fodtrack backend.
@@ -135,12 +123,19 @@
  *  The Rotor-EZ backend can be used with rotators that support the
  *  DCU command set by DF9GR (currently the ERC).
  */
+/*! \def ROT_MODEL_RT21
+ *  \brief A macro that returns the model number of the RT21 backend.
+ *
+ *  The Rotor-EZ backend can be used with rotators that support the
+ *  DCU command set by Green Heron (currently the RT-21).
+ */
 #define ROT_ROTOREZ 4
 #define ROT_BACKEND_ROTOREZ "rotorez"
 #define ROT_MODEL_ROTOREZ ROT_MAKE_MODEL(ROT_ROTOREZ, 1)
 #define ROT_MODEL_ROTORCARD ROT_MAKE_MODEL(ROT_ROTOREZ, 2)
 #define ROT_MODEL_DCU ROT_MAKE_MODEL(ROT_ROTOREZ, 3)
 #define ROT_MODEL_ERC ROT_MAKE_MODEL(ROT_ROTOREZ, 4)
+#define ROT_MODEL_RT21 ROT_MAKE_MODEL(ROT_ROTOREZ, 5)
 
 /*! \def ROT_MODEL_SARTEK1
  *  \brief A macro that returns the model number of the SARtek-1 backend.
@@ -264,45 +259,43 @@
 #define ROT_BACKEND_TS7400 "ts7400"
 #define ROT_MODEL_TS7400 ROT_MAKE_MODEL(ROT_TS7400, 1)
 
+/*! \def ROT_MODEL_NEXSTAR
+ *  \brief A macro that returns the model number of the NEXSTAR backend.
+ *
+ *  The CELESTRON backend can be used with rotators that support the Celestron
+ *  protocol and alike.
+ */
+#define ROT_CELESTRON 14
+#define ROT_BACKEND_CELESTRON "celestron"
+#define ROT_MODEL_NEXSTAR ROT_MAKE_MODEL(ROT_CELESTRON, 1)
+
+/*! \def ROT_MODEL_ETHER6
+ *  \brief A macro that returns the model number of the Ether6 backend.
+ *
+ *  The Ether6 backend can be used with rotators that support the Ether6
+ *  protocol and alike.
+ */
+#define ROT_ETHER6 15
+#define ROT_BACKEND_ETHER6 "ether6"
+#define ROT_MODEL_ETHER6 ROT_MAKE_MODEL(ROT_ETHER6, 1)
+
+
+/*! \def ROT_MODEL_CNCTRK
+ *  \brief A macro that returns the model number of the CNCTRK backend.
+ *
+ *  The CNCTRK backend can be used with rotators that support,
+ *  the LinuxCNC running Axis GUI interface.
+ */
+#define ROT_CNCTRK 16
+#define ROT_BACKEND_CNCTRK "cnctrk"
+#define ROT_MODEL_CNCTRK ROT_MAKE_MODEL(ROT_CNCTRK, 1)
+
 
 /*! \typedef typedef int rot_model_t
     \brief Convenience type definition for rotator model.
 */
 typedef int rot_model_t;
 
-/*! \def ROT_BACKEND_LIST
- *  \brief Static list of rotator models.
- *
- *  This is a NULL terminated list of available rotator backends. Each entry
- *  in the list consists of two fields: The branch number, which is an integer,
- *  and the branch name, which is a character string.
- */
-#define ROT_BACKEND_LIST {		\
-        { ROT_DUMMY, ROT_BACKEND_DUMMY }, \
-        { ROT_RPC, ROT_BACKEND_RPC }, \
-        { ROT_EASYCOMM, ROT_BACKEND_EASYCOMM }, \
-        { ROT_FODTRACK, ROT_BACKEND_FODTRACK }, \
-        { ROT_ROTOREZ, ROT_BACKEND_ROTOREZ }, \
-        { ROT_SARTEK, ROT_BACKEND_SARTEK }, \
-        { ROT_GS232A, ROT_BACKEND_GS232A }, \
-        { ROT_KIT, ROT_BACKEND_KIT }, \
-        { ROT_HEATHKIT, ROT_BACKEND_HEATHKIT }, \
-        { ROT_SPID, ROT_BACKEND_SPID }, \
-        { ROT_M2, ROT_BACKEND_M2 }, \
-        { ROT_ARS, ROT_BACKEND_ARS }, \
-        { ROT_AMSAT, ROT_BACKEND_AMSAT }, \
-        { ROT_TS7400, ROT_BACKEND_TS7400 }, \
-        { 0, NULL }, /* end */  \
-}
-
-/*
- * struct rot_backend_list {
- *		rot_model_t model;
- *		const char *backend;
- * } rot_backend_list[] = ROT_LIST;
- *
- */
-
 #endif /* _ROTLIST_H */
 
 /** @} */
