@@ -3,10 +3,11 @@
  *  Copyright (C) 2010,2011 by Nate Bargmann, n0nb@n0nb.us
  *  Copyright (C) 2011 by Alexander Sack, Alexander Sack, pisymbol@gmail.com
  *
+ *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
  *   License as published by the Free Software Foundation; either
- *   version 2 of the License, or (at your option) any later version.
+ *   version 2.1 of the License, or (at your option) any later version.
  *
  *   This library is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,16 +16,17 @@
  *
  *   You should have received a copy of the GNU Lesser General Public
  *   License along with this library; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  *  See the file 'COPYING.LIB' in the main Hamlib distribution directory for
- *  the complete text of the GNU Lesser Public License version 2.
+ *  the complete text of the GNU Lesser Public License version 2.1.
  *
  */
 
 #include <string.h>
 #include <stdlib.h>
 
+#include "serial.h"
 #include "elecraft.h"
 #include "kenwood.h"
 
@@ -40,6 +42,15 @@ static const struct elec_ext_id_str elec_ext_id_str_lst[] = {
 };
 
 
+/* K3 firmware revision level, will be assigned to the fw_rev pointer in
+ * kenwood_priv_data structure at runtime in electraft_open().  The array is
+ * declared here so that the sizeof operator can be used in the call to
+ * elecraft_get_firmware_revision_level() to calculate the exact size of the
+ * array for the call to strncpy().
+ */
+static char k3_fw_rev[KENWOOD_MAX_BUF_LEN];
+
+
 /* Private Elecraft extra levels definitions
  *
  * Token definitions for .cfgparams in rig_caps
@@ -62,7 +73,7 @@ const struct confparams elecraft_ext_levels[] = {
 /* Private function declarations */
 int verify_kenwood_id(RIG *rig, char *id);
 int elecraft_get_extension_level(RIG *rig, const char *cmd, int *ext_level);
-int elecraft_get_firmware_revision_level(RIG *rig, const char *cmd, char *fw_rev);
+int elecraft_get_firmware_revision_level(RIG *rig, const char *cmd, char *fw_rev, size_t fw_rev_sz);
 
 /* Shared backend function definitions */
 
@@ -93,11 +104,45 @@ int elecraft_open(RIG *rig)
 	 */
 	struct kenwood_priv_data *priv = rig->state.priv;
 
-	/* Use check for "ID017;" to verify rig is reachable */
-	err = verify_kenwood_id(rig, id);
-	if (err != RIG_OK)
-		return err;
+	/* As k3_fw_rev is declared static, it is persistent so the structure
+	 * can point to it.  This way was chosen to allow the compiler to
+	 * calculate the size of the array to resolve a bug found by gcc 4.8.x
+	 */
+	priv->fw_rev = k3_fw_rev;
 
+	/* Use check for "ID017;" to verify rig is reachable */
+	rig_debug(RIG_DEBUG_TRACE, "%s: rig_model=%d,%d\n", __func__,rig->caps->rig_model,RIG_MODEL_XG3);
+	if (rig->caps->rig_model == RIG_MODEL_XG3) { // XG3 doesn't have ID
+		struct rig_state *rs = &rig->state;
+		char *cmd = "V;";
+		char data[32];
+
+		strcpy(data,"EMPTY");
+		// Not going to get carried away with retries and such
+		err = write_block(&rs->rigport, cmd, strlen(cmd));
+		if (err != RIG_OK) {
+			rig_debug(RIG_DEBUG_TRACE, "%s: XG3 cannot request identification\n", __func__);
+			return err;
+		}
+		err = read_string(&rs->rigport, id, sizeof(id), ";", 1);
+		if (err < 0) {
+			rig_debug(RIG_DEBUG_TRACE, "%s: XG3 cannot get identification\n", __func__);
+			return err;
+		}
+
+		rig_debug(RIG_DEBUG_VERBOSE, "%s: id=%s\n", __func__,id);
+#if 0
+		if (err != RIG_OK) {
+			rig_debug(RIG_DEBUG_TRACE, "%s: cannot get identification\n", __func__);
+			return err;
+		}
+#endif
+	}
+	else { // Standard Kenwood
+		err = verify_kenwood_id(rig, id);
+		if (err != RIG_OK)
+			return err;
+	}
 	switch(rig->caps->rig_model) {
 	case RIG_MODEL_K2:
 		err = elecraft_get_extension_level(rig, "K2", &priv->k2_ext_lvl);
@@ -123,17 +168,31 @@ int elecraft_open(RIG *rig)
 		rig_debug(RIG_DEBUG_VERBOSE, "%s: K3 level is %d, %s\n", __func__,
 			priv->k3_ext_lvl, elec_ext_id_str_lst[priv->k3_ext_lvl].id);
 
-		err = elecraft_get_firmware_revision_level(rig, "RVM", priv->k3_fw_rev);
+		err = elecraft_get_firmware_revision_level(rig, "RVM", priv->fw_rev,
+				(sizeof(k3_fw_rev) / sizeof(k3_fw_rev[0])));
+
 		if (err != RIG_OK)
 			return err;
 
 		break;
+	case RIG_MODEL_XG3:
+		rig_debug(RIG_DEBUG_VERBOSE, "%s: XG3 level is %d, %s\n", __func__,
+			priv->k3_ext_lvl, elec_ext_id_str_lst[priv->k3_ext_lvl].id);
+		break;
 	default:
 		rig_debug(RIG_DEBUG_WARN, "%s: unrecognized rig model %d\n",
 			__func__, rig->caps->rig_model);
 		return -RIG_EINVAL;
 	}
 
+	if (RIG_MODEL_XG3 != rig->caps->rig_model)
+		{
+			/* currently we cannot cope with AI mode so turn it off in case
+				 last client left it on */
+			kenwood_set_trn(rig, RIG_TRN_OFF); /* ignore status in case it's
+																						not supported */
+		}
+
 	return RIG_OK;
 }
 
@@ -194,7 +253,7 @@ int elecraft_get_extension_level(RIG *rig, const char *cmd, int *ext_level)
 	char buf[KENWOOD_MAX_BUF_LEN];
 	char *bufptr;
 
-	err = kenwood_safe_transaction(rig, cmd, buf, KENWOOD_MAX_BUF_LEN, 4);
+	err = kenwood_safe_transaction(rig, cmd, buf, KENWOOD_MAX_BUF_LEN, 3);
 	if (err != RIG_OK) {
 		rig_debug(RIG_DEBUG_VERBOSE, "%s: Cannot get K2|K3 ID\n", __func__);
 		return err;
@@ -219,7 +278,8 @@ int elecraft_get_extension_level(RIG *rig, const char *cmd, int *ext_level)
 
 /* Determine firmware revision level */
 
-int elecraft_get_firmware_revision_level(RIG *rig, const char *cmd, char *fw_rev)
+int elecraft_get_firmware_revision_level(RIG *rig, const char *cmd,
+                                         char *fw_rev, size_t fw_rev_sz)
 {
 	rig_debug(RIG_DEBUG_VERBOSE, "%s called\n", __func__);
 
@@ -229,25 +289,29 @@ int elecraft_get_firmware_revision_level(RIG *rig, const char *cmd, char *fw_rev
 	int err;
 	char *bufptr;
 	char buf[KENWOOD_MAX_BUF_LEN];
-	size_t size = KENWOOD_MAX_BUF_LEN;
 
-	err = kenwood_transaction(rig, cmd, sizeof(cmd), buf, &size);
+	/* Get the actual firmware revision number. */
+	err = kenwood_transaction(rig, cmd, buf, sizeof (buf));
+
 	if (err != RIG_OK) {
-		rig_debug(RIG_DEBUG_VERBOSE, "%s: Cannot get firmeware revision level\n", __func__);
+		rig_debug(RIG_DEBUG_VERBOSE, "%s: Cannot get firmware revision level\n", __func__);
 		return err;
 	}
 
-	/* Get the actual firmware revision number. */
+	/* Now buf[] contains the string from the K3 which includes the command
+	 * and the firmware revision number as:  "RVM04.67".
+	 */
+
 	bufptr = &buf[0];
 
 	/* Skip the command string */
-	bufptr += sizeof(cmd);
+	bufptr += strlen(cmd);
 
-	/* Ignore leading zero's */
+	/* Skip leading zero(s) as the revision number has the format of: "04.67" */
 	while (bufptr && *bufptr == '0') { bufptr++; }
 
 	/* Copy out */
-	strncpy(fw_rev, bufptr, sizeof(bufptr));
+	strncpy(fw_rev, bufptr, fw_rev_sz - 1);
 
 	rig_debug(RIG_DEBUG_VERBOSE, "%s: Elecraft firmware revision is %s\n", __func__, fw_rev);
 
