@@ -1,6 +1,8 @@
 /*
  *  Hamlib Kenwood backend - main header
  *  Copyright (c) 2000-2011 by Stephane Fillod
+ *  Copyright (C) 2009,2010 Alessandro Zummo <a.zummo@towertech.it>
+ *  Copyright (C) 2009,2010,2011,2012,2013 by Nate Bargmann, n0nb@n0nb.us
  *
  *
  *   This library is free software; you can redistribute it and/or
@@ -25,23 +27,23 @@
 #include <string.h>
 #include "token.h"
 
-#define BACKEND_VER	"0.8"
+#define BACKEND_VER "0.9"
 
 #define EOM_KEN ';'
 #define EOM_TH '\r'
 
-#define KENWOOD_MODE_TABLE_MAX	10
-#define KENWOOD_MAX_BUF_LEN		50 /* max answer len, arbitrary */
+#define KENWOOD_MODE_TABLE_MAX  24
+#define KENWOOD_MAX_BUF_LEN   50 /* max answer len, arbitrary */
 
 
 /* Tokens for Parameters common to multiple rigs.
  * Use token # >= 1 or <= 100.  Defined here so they will be
  * available in Kenwood name space.
  */
-#define TOK_VOICE	TOKEN_BACKEND(1)
-#define TOK_FINE	TOKEN_BACKEND(2)
-#define TOK_XIT		TOKEN_BACKEND(3)
-#define TOK_RIT		TOKEN_BACKEND(4)
+#define TOK_VOICE TOKEN_BACKEND(1)
+#define TOK_FINE  TOKEN_BACKEND(2)
+#define TOK_XIT   TOKEN_BACKEND(3)
+#define TOK_RIT   TOKEN_BACKEND(4)
 
 /* Token structure assigned to .cfgparams in rig_caps */
 extern const struct confparams kenwood_cfg_params[];
@@ -61,20 +63,23 @@ extern const struct confparams kenwood_cfg_params[];
 #define MD_FSKR '9'
 
 struct kenwood_priv_caps {
-    char cmdtrm;		/* Command termination chars (ken=';' or th='\r') */
-    int if_len;			/* length of IF; anwser */
+    char cmdtrm;    /* Command termination chars (ken=';' or th='\r') */
+    int if_len;     /* length of IF; answer excluding ';' terminator */
     rmode_t *mode_table;
 };
 
 struct kenwood_priv_data {
     char info[KENWOOD_MAX_BUF_LEN];
-    split_t split;		/* current split state */
-    int k2_ext_lvl;		/* Initial K2 extension level */
-    int k3_ext_lvl;		/* Initial K3 extension level */
-    int k2_md_rtty;		/* K2 RTTY mode available flag, 1 = RTTY, 0 = N/A */
-    char k3_fw_rev[KENWOOD_MAX_BUF_LEN]; /* K3 firmware revision level */
+    split_t split;    /* current split state */
+    int k2_ext_lvl;   /* Initial K2 extension level */
+    int k3_ext_lvl;   /* Initial K3 extension level */
+    int k2_md_rtty;   /* K2 RTTY mode available flag, 1 = RTTY, 0 = N/A */
+    char *fw_rev;   /* firmware revision level */
+    unsigned fw_rev_uint; /* firmware revison as a number 1.07 -> 107 */
+    void * data;          /* model specific data */
 };
 
+
 #define kenwood_caps(rig) ((struct kenwood_priv_caps *)(rig)->caps->priv)
 
 
@@ -83,10 +88,9 @@ extern rmode_t kenwood_mode_table[KENWOOD_MODE_TABLE_MAX];
 extern const tone_t kenwood38_ctcss_list[];
 extern const tone_t kenwood42_ctcss_list[];
 
-int kenwood_transaction(RIG *rig, const char *cmd, int cmd_len, char *data,
-				size_t *data_len);
+int kenwood_transaction(RIG *rig, const char *cmd, char *data, size_t data_len);
 int kenwood_safe_transaction(RIG *rig, const char *cmd, char *buf,
-				size_t buf_size, size_t expected);
+        size_t buf_size, size_t expected);
 
 rmode_t kenwood2rmode(unsigned char mode, const rmode_t mode_table[]);
 char rmode2kenwood(rmode_t mode, const rmode_t mode_table[]);
@@ -96,7 +100,10 @@ int kenwood_cleanup(RIG *rig);
 int kenwood_open(RIG *rig);
 
 int kenwood_set_vfo(RIG *rig, vfo_t vfo);
+int kenwood_set_vfo_main_sub(RIG *rig, vfo_t vfo);
 int kenwood_get_vfo_if(RIG *rig, vfo_t *vfo);
+int kenwood_get_vfo_main_sub(RIG *rig, vfo_t *vfo);
+int kenwood_set_split(RIG *rig, vfo_t vfo , split_t split, vfo_t txvfo);
 int kenwood_set_split_vfo(RIG *rig, vfo_t vfo , split_t split, vfo_t txvfo);
 int kenwood_get_split_vfo_if(RIG *rig, vfo_t rxvfo, split_t *split, vfo_t *txvfo);
 
@@ -146,7 +153,8 @@ int kenwood_set_trn(RIG *rig, int trn);
 int kenwood_get_trn(RIG *rig, int *trn);
 
 /* only use if returned string has length 6, e.g. 'SQ011;' */
-int get_kenwood_level(RIG *rig, const char *cmd, int cmd_len, float *f);
+int get_kenwood_level(RIG *rig, const char *cmd, float *f);
+int get_kenwood_func(RIG *rig, const char *cmd, int *status);
 
 extern const struct rig_caps ts950sdx_caps;
 extern const struct rig_caps ts50s_caps;
@@ -163,6 +171,7 @@ extern const struct rig_caps ts930_caps;
 extern const struct rig_caps ts2000_caps;
 extern const struct rig_caps k2_caps;
 extern const struct rig_caps k3_caps;
+extern const struct rig_caps xg3_caps;
 extern const struct rig_caps trc80_caps;
 
 extern const struct rig_caps thd7a_caps;
@@ -182,34 +191,18 @@ extern const struct rig_caps r5000_caps;
 
 extern const struct rig_caps ts480_caps;
 extern const struct rig_caps ts590_caps;
+extern const struct rig_caps ts590sg_caps;
 extern const struct rig_caps thf6a_caps;
 
 extern const struct rig_caps transfox_caps;
 
+extern const struct rig_caps f6k_caps;
+
 /* use when not interested in the answer, but want to check its len */
 static int inline kenwood_simple_transaction(RIG *rig, const char *cmd, size_t expected)
 {
-	char buf[20];
-	return kenwood_safe_transaction(rig, cmd, buf, sizeof(buf), expected);
-}
-
-/* no answer needed at all */
-static int inline kenwood_simple_cmd(RIG *rig, const char *cmd)
-{
-	char buf[20];
-	return kenwood_safe_transaction(rig, cmd, buf, sizeof(buf), 0);
-}
-
-/* answer is the same as the command */
-static int inline kenwood_cmd(RIG *rig, const char *cmd)
-{
-	char buf[20];
-	int lenz = strlen(cmd)+1;
-
-	if (lenz > sizeof(buf))
-	  return -RIG_ENOMEM;
-	else
-	  return kenwood_safe_transaction(rig, cmd, buf, sizeof(buf), lenz);
+  struct kenwood_priv_data *priv = rig->state.priv;
+  return kenwood_safe_transaction(rig, cmd, priv->info, KENWOOD_MAX_BUF_LEN, expected);
 }
 
 #endif /* _KENWOOD_H */
