@@ -140,7 +140,7 @@ const struct rig_caps ts2000_caps = {
 .write_delay =  0,
 .post_write_delay =  50,	/* ms */
 .timeout =  200,
-.retry =  3,
+.retry =  10,
 
 .has_get_func =  TS2000_FUNC_ALL,
 .has_set_func =  TS2000_FUNC_ALL,
@@ -277,6 +277,7 @@ const struct rig_caps ts2000_caps = {
 .priv =  (void *)&ts2000_priv_caps,
 
 .rig_init = kenwood_init,
+.rig_open = kenwood_open,
 .rig_cleanup = kenwood_cleanup,
 .set_freq =  kenwood_set_freq,
 .get_freq =  kenwood_get_freq,
@@ -288,6 +289,8 @@ const struct rig_caps ts2000_caps = {
 .get_mode =  kenwood_get_mode,
 .set_vfo =  kenwood_set_vfo,
 .get_vfo =  kenwood_get_vfo_if,
+.set_split_vfo = kenwood_set_split_vfo,
+.get_split_vfo = kenwood_get_split_vfo_if,
 .set_ctcss_tone =  kenwood_set_ctcss_tone_tn,
 .get_ctcss_tone =  kenwood_get_ctcss_tone,
 .set_ctcss_sql =  kenwood_set_ctcss_sql,
@@ -369,16 +372,16 @@ int ts2000_get_channel(RIG *rig, channel_t *chan)
 
 	int err;
 	char buf[52];
-	size_t buf_size = 52;
 	char cmd[8];
 	struct kenwood_priv_caps *caps = kenwood_caps(rig);
 
 	/* put channel num in the command string */
 	sprintf(cmd, "MR0%03d;", chan->channel_num);
 
-	err = kenwood_transaction(rig, cmd, strlen(cmd), buf, &buf_size );
+	err = kenwood_transaction(rig, cmd, buf, sizeof (buf));
 	if (err != RIG_OK)
 		return err;
+	size_t length = strlen (buf);
 	memset(chan, 0x00, sizeof(channel_t));
 
 	chan->vfo = RIG_VFO_MEM;
@@ -389,11 +392,9 @@ int ts2000_get_channel(RIG *rig, channel_t *chan)
 	 * to read out the filters of a given memory channel. The rig, however,
 	 * stores this information.
 	 */
-	/* Replace ';' with '\0' */
-	buf[ buf_size-1 ]  = '\0';
 	/* First check if a name is assigned.
 	   Name is returned at positions 41-48 (counting from 0) */
-	if( buf_size > 41 ){
+	if( length > 41 ){
 // 	  rig_debug(RIG_DEBUG_VERBOSE, "Copying channel description: %s\n", &buf[ 41 ] );
 	  strcpy( chan->channel_desc, &buf[ 41 ] );
 	}
@@ -505,8 +506,7 @@ int ts2000_get_channel(RIG *rig, channel_t *chan)
 
 	/* Check split freq */
 	cmd[2] = '1';
-	buf_size = 52;
-	err = kenwood_transaction(rig, cmd, strlen(cmd), buf, &buf_size );
+	err = kenwood_transaction(rig, cmd, buf, sizeof (buf));
 	if (err != RIG_OK)
 		return err;
 
@@ -642,8 +642,7 @@ int ts2000_set_channel(RIG *rig, const channel_t *chan)
 		);
 	rig_debug( RIG_DEBUG_VERBOSE, "The command will be: %s\n", buf );
 
-    size_t tmp=0;
-	err = kenwood_transaction(rig, buf, strlen(buf), NULL, &tmp );
+	err = kenwood_transaction(rig, buf, NULL, 0);
 	if (err != RIG_OK)
 		return err;
 	
@@ -666,8 +665,7 @@ int ts2000_set_channel(RIG *rig, const channel_t *chan)
 		);
 	  rig_debug( RIG_DEBUG_VERBOSE, "Split, the command will be: %s\n", buf );
 	
-      tmp = 0;
-	  err = kenwood_transaction(rig, buf, strlen(buf), NULL, &tmp );
+	  err = kenwood_transaction(rig, buf, NULL, 0);
 	}
 	return err;
 }
@@ -687,10 +685,11 @@ int ts2000_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		switch (level) {
 
 		case RIG_LEVEL_PREAMP:
-			retval = kenwood_transaction (rig, "PA", 2, lvlbuf, &lvl_len);
+			retval = kenwood_transaction (rig, "PA", lvlbuf, sizeof (lvlbuf));
 			if (retval != RIG_OK)
 				return retval;
-			if ((lvl_len != 5)){ /*TS-2000 returns 5 chars for PA; */
+			lvl_len = strlen (lvlbuf);
+			if ((lvl_len != 4)){ /*TS-2000 returns 4 chars for PA; */
 				rig_debug(RIG_DEBUG_ERR,"%s: unexpected answer len=%d\n",
 					       __func__, lvl_len);
 				return -RIG_ERJCTED;
@@ -704,11 +703,11 @@ int ts2000_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 			break;
 
 		case RIG_LEVEL_ATT:
-			retval = kenwood_transaction (rig, "RA", 2, lvlbuf, &lvl_len);
+			retval = kenwood_transaction (rig, "RA", lvlbuf, sizeof (lvlbuf));
 			if (retval != RIG_OK)
 				return retval;
-
-			if ((lvl_len != 7)){ /* TS-2000 returns 7 chars for RA; */
+			lvl_len = strlen (lvlbuf);
+			if ((lvl_len != 6)){ /* TS-2000 returns 6 chars for RA; */
 				rig_debug(RIG_DEBUG_ERR,"ts2000_get_level: "
 								"unexpected answer len=%d\n", lvl_len);
 				return -RIG_ERJCTED;
@@ -723,10 +722,11 @@ int ts2000_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 			break;
 
 		case RIG_LEVEL_VOX:
-			retval = kenwood_transaction (rig, "VD", 2, lvlbuf, &lvl_len);
+			retval = kenwood_transaction (rig, "VD", lvlbuf, sizeof (lvlbuf));
 			if (retval != RIG_OK)
 				return retval;
-			if (lvl_len != 7) {
+			lvl_len = strlen (lvlbuf);
+			if (lvl_len != 6) {
 				rig_debug(RIG_DEBUG_ERR,"ts2000_get_level: "
 					"unexpected answer len=%d\n", lvl_len);
 				return -RIG_ERJCTED;
@@ -736,10 +736,11 @@ int ts2000_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 			break;
 
 	        case RIG_LEVEL_AF:
-			retval = kenwood_transaction (rig, "AG0", 3, lvlbuf, &lvl_len);
+						retval = kenwood_transaction (rig, "AG0", lvlbuf, sizeof (lvlbuf));
 			if (retval != RIG_OK)
 				return retval;
-			if (lvl_len != 7) {
+			lvl_len = strlen (lvlbuf);
+			if (lvl_len != 6) {
 				rig_debug(RIG_DEBUG_ERR,"ts2000_get_level: "
 					"unexpected answer len=%d\n", lvl_len);
 				return -RIG_ERJCTED;
@@ -749,10 +750,11 @@ int ts2000_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 			break;
 
 		case RIG_LEVEL_RF:
-			retval = kenwood_transaction (rig, "RG", 2, lvlbuf, &lvl_len);
+			retval = kenwood_transaction (rig, "RG", lvlbuf, sizeof (lvlbuf));
 			if (retval != RIG_OK)
 				return retval;
-			if (lvl_len != 6) {
+			lvl_len = strlen (lvlbuf);
+			if (lvl_len != 5) {
 				rig_debug(RIG_DEBUG_ERR,"ts2000_get_level: "
 					"unexpected answer len=%d\n", lvl_len);
 				return -RIG_ERJCTED;
@@ -762,10 +764,11 @@ int ts2000_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 			break;
 
 		case RIG_LEVEL_SQL:
-			retval = kenwood_transaction (rig, "SQ0", 3, lvlbuf, &lvl_len);
+			retval = kenwood_transaction (rig, "SQ0", lvlbuf, sizeof (lvlbuf));
 			if (retval != RIG_OK)
 				return retval;
-			if (lvl_len != 7) {
+			lvl_len = strlen (lvlbuf);
+			if (lvl_len != 6) {
 				rig_debug(RIG_DEBUG_ERR,"ts2000_get_level: "
 					"unexpected answer len=%d\n", lvl_len);
 				return -RIG_ERJCTED;
@@ -775,10 +778,11 @@ int ts2000_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 			break;
 
 		case RIG_LEVEL_CWPITCH:
-			retval = kenwood_transaction (rig, "EX0310000", 9, lvlbuf, &lvl_len);
+			retval = kenwood_transaction (rig, "EX0310000", lvlbuf, sizeof (lvlbuf));
 			if (retval != RIG_OK)
 				return retval;
-			if (lvl_len != 12) {
+			lvl_len = strlen (lvlbuf);
+			if (lvl_len != 15) {
 				rig_debug(RIG_DEBUG_ERR,"ts2000_get_level: "
 					"unexpected answer len=%d answer=%s\n", lvl_len, lvlbuf);
 				return -RIG_ERJCTED;
@@ -788,10 +792,11 @@ int ts2000_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 			break;
 
 		case RIG_LEVEL_RFPOWER:
-			retval = kenwood_transaction (rig, "PC", 2, lvlbuf, &lvl_len);
+			retval = kenwood_transaction (rig, "PC", lvlbuf, sizeof (lvlbuf));
 			if (retval != RIG_OK)
 				return retval;
-			if (lvl_len != 6) {
+			lvl_len = strlen (lvlbuf);
+			if (lvl_len != 5) {
 				rig_debug(RIG_DEBUG_ERR,"ts2000_get_level: "
 					"unexpected answer len=%d\n", lvl_len);
 				return -RIG_ERJCTED;
@@ -801,10 +806,11 @@ int ts2000_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
         	  	break;
 
 		case RIG_LEVEL_MICGAIN:
-			retval = kenwood_transaction (rig, "MG", 2, lvlbuf, &lvl_len);
+			retval = kenwood_transaction (rig, "MG", lvlbuf, sizeof (lvlbuf));
 			if (retval != RIG_OK)
 				return retval;
-			if (lvl_len != 6) {
+			lvl_len = strlen (lvlbuf);
+			if (lvl_len != 5) {
 				rig_debug(RIG_DEBUG_ERR,"ts2000_get_level: "
 					"unexpected answer len=%d\n", lvl_len);
 				return -RIG_ERJCTED;
@@ -814,10 +820,11 @@ int ts2000_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 			break;
 
 		case RIG_LEVEL_KEYSPD:
-			retval = kenwood_transaction (rig, "KS", 2, lvlbuf, &lvl_len);
+			retval = kenwood_transaction (rig, "KS", lvlbuf, sizeof (lvlbuf));
 			if (retval != RIG_OK)
 				return retval;
-			if (lvl_len != 6) {
+			lvl_len = strlen (lvlbuf);
+			if (lvl_len != 5) {
 				rig_debug(RIG_DEBUG_ERR,"ts2000_get_level: "
 					"unexpected answer len=%d\n", lvl_len);
 				return -RIG_ERJCTED;
@@ -831,10 +838,11 @@ int ts2000_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 			break;
 
 		case RIG_LEVEL_COMP:
-			retval = kenwood_transaction (rig, "PL", 2, lvlbuf, &lvl_len);
+			retval = kenwood_transaction (rig, "PL", lvlbuf, sizeof (lvlbuf));
 			if (retval != RIG_OK)
 				return retval;
-			if (lvl_len != 9) {
+			lvl_len = strlen (lvlbuf);
+			if (lvl_len != 8) {
 				rig_debug(RIG_DEBUG_ERR,"ts2000_get_level: "
 					"unexpected answer len=%d\n", lvl_len);
 				return -RIG_ERJCTED;
@@ -846,7 +854,7 @@ int ts2000_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 
 
 		case RIG_LEVEL_AGC: /* FIX ME: ts2000 returns 0 -20 for AGC */
-			ret = get_kenwood_level(rig, "GT", 2, &val->f);
+			ret = get_kenwood_level(rig, "GT", &val->f);
 			agclevel = 255.0 * val->f;
 			if (agclevel == 0) val->i = 0;
 			else if (agclevel < 85) val->i = 1;
@@ -856,10 +864,11 @@ int ts2000_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 			break;
 
 		case RIG_LEVEL_BKINDL:
-			retval = kenwood_transaction (rig, "SD", 2, lvlbuf, &lvl_len);
+			retval = kenwood_transaction (rig, "SD", lvlbuf, sizeof (lvlbuf));
 			if (retval != RIG_OK)
 				return retval;
-			if (lvl_len != 7) {
+			lvl_len = strlen (lvlbuf);
+			if (lvl_len != 6) {
 				rig_debug(RIG_DEBUG_ERR,"ts2000_get_level: "
 					"unexpected answer len=%d\n", lvl_len);
 				return -RIG_ERJCTED;
@@ -873,10 +882,11 @@ int ts2000_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		    break;
 
 		case RIG_LEVEL_METER:
-			retval = kenwood_transaction (rig, "RM", 2, lvlbuf, &lvl_len);
+			retval = kenwood_transaction (rig, "RM", lvlbuf, sizeof (lvlbuf));
 			if (retval != RIG_OK)
 				return retval;
-			if (lvl_len != 8) {
+			lvl_len = strlen (lvlbuf);
+			if (lvl_len != 7) {
 				rig_debug(RIG_DEBUG_ERR,"ts2000_get_level: "
 					"unexpected answer len=%d\n", lvl_len);
 				return -RIG_ERJCTED;
@@ -886,10 +896,11 @@ int ts2000_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 			break;
 
 		case RIG_LEVEL_VOXGAIN:
-			retval = kenwood_transaction (rig, "VG", 2, lvlbuf, &lvl_len);
+			retval = kenwood_transaction (rig, "VG", lvlbuf, sizeof (lvlbuf));
 			if (retval != RIG_OK)
 				return retval;
-			if (lvl_len != 6) {
+			lvl_len = strlen (lvlbuf);
+			if (lvl_len != 5) {
 				rig_debug(RIG_DEBUG_ERR,"ts2000_get_level: "
 					"unexpected answer len=%d\n", lvl_len);
 				return -RIG_ERJCTED;
@@ -904,11 +915,11 @@ int ts2000_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 
 		case RIG_LEVEL_RAWSTR:
 		case RIG_LEVEL_STRENGTH:
-			retval = kenwood_transaction (rig, "SM0", 3, lvlbuf, &lvl_len);
+			retval = kenwood_transaction (rig, "SM0", lvlbuf, sizeof (lvlbuf));
 			if (retval != RIG_OK)
 				return retval;
-
-			if (( (lvl_len !=8)) || lvlbuf[1] != 'M') {
+			lvl_len = strlen (lvlbuf);
+			if (( (lvl_len !=7)) || lvlbuf[1] != 'M') {
 				/* TS-2000 returns 8 bytes for S meter level */
 				rig_debug(RIG_DEBUG_ERR,"%s: wrong answer len=%d\n",
 					       __func__, lvl_len);
