@@ -402,12 +402,14 @@ static struct {
 	{ RIG_FUNC_ABM, "ABM" },
 	{ RIG_FUNC_BC, "BC" },
 	{ RIG_FUNC_MBC, "MBC" },
+	{ RIG_FUNC_RIT, "RIT" },
 	{ RIG_FUNC_AFC, "AFC" },
 	{ RIG_FUNC_SATMODE, "SATMODE" },
 	{ RIG_FUNC_SCOPE, "SCOPE" },
 	{ RIG_FUNC_RESUME, "RESUME" },
 	{ RIG_FUNC_TBURST, "TBURST" },
 	{ RIG_FUNC_TUNER, "TUNER" },
+	{ RIG_FUNC_XIT, "XIT" },
 	{ RIG_FUNC_NONE, "" },
 };
 
@@ -839,4 +841,5 @@ void HAMLIB_API rig_force_cache_timeout(struct timeval *tv)
     tv->tv_usec = 0;
 }
 
+
 /** @} */
