@@ -118,9 +118,11 @@ const char hamlib_copyright[231] = /* hamlib 1.2 ABI specifies 231 bytes */
 #endif
 
 #if defined(WIN32) && !defined(__CYGWIN__)
-#define DEFAULT_CM108_PTT_BITNUM "fixme"
+/* FIXME: Determine correct GPIO bit number for W32 using MinGW. */
+#define DEFAULT_CM108_PTT_BITNUM 2
 #elif BSD
-#define DEFAULT_CM108_PTT_BITNUM "fixme"
+/* FIXME: Determine correct GPIO bit number for *BSD. */
+#define DEFAULT_CM108_PTT_BITNUM 2
 #else
 #define DEFAULT_CM108_PTT_BITNUM 2
 #endif
@@ -454,7 +456,7 @@ int HAMLIB_API rig_open(RIG *rig)
 {
 	const struct rig_caps *caps;
 	struct rig_state *rs;
-	int status;
+	int status = RIG_OK;
 
 	rig_debug(RIG_DEBUG_VERBOSE,"rig:rig_open called \n");
 
@@ -469,33 +471,40 @@ int HAMLIB_API rig_open(RIG *rig)
 
 	rs->rigport.fd = -1;
 
-	if (rs->rigport.type.rig == RIG_PORT_SERIAL) {
-
-		if (rs->rigport.parm.serial.rts_state != RIG_SIGNAL_UNSET &&
-				(rs->pttport.type.ptt == RIG_PTT_SERIAL_RTS ||
-				rs->rigport.parm.serial.handshake == RIG_HANDSHAKE_HARDWARE)) {
-
-			rig_debug(RIG_DEBUG_ERR, "Cannot set RTS with PTT by RTS or hardware handshare \"%s\"\n",
-						rs->rigport.pathname);
-            return -RIG_ECONF;
-		}
-		if (rs->rigport.parm.serial.dtr_state != RIG_SIGNAL_UNSET &&
-				rs->pttport.type.ptt == RIG_PTT_SERIAL_DTR) {
-			rig_debug(RIG_DEBUG_ERR, "Cannot set DTR with PTT by DTR\"%s\"\n",
-						rs->rigport.pathname);
-            return -RIG_ECONF;
-		}
+	if (rs->rigport.type.rig == RIG_PORT_SERIAL)
+    {
+      if (rs->rigport.parm.serial.rts_state != RIG_SIGNAL_UNSET &&
+          rs->rigport.parm.serial.handshake == RIG_HANDSHAKE_HARDWARE)
+        {
+          rig_debug(RIG_DEBUG_ERR, "Cannot set RTS with hardware handshake \"%s\"\n",
+                    rs->rigport.pathname);
+          return -RIG_ECONF;
+        }
+      if ('\0' == rs->pttport.pathname[0]
+          || !strcmp(rs->pttport.pathname, rs->rigport.pathname))
+        {
+          /* check for control line conflicts */
+          if (rs->rigport.parm.serial.rts_state != RIG_SIGNAL_UNSET &&
+              rs->pttport.type.ptt == RIG_PTT_SERIAL_RTS)
+            {
+              rig_debug(RIG_DEBUG_ERR, "Cannot set RTS with PTT by RTS \"%s\"\n",
+                        rs->rigport.pathname);
+              return -RIG_ECONF;
+            }
+          if (rs->rigport.parm.serial.dtr_state != RIG_SIGNAL_UNSET &&
+              rs->pttport.type.ptt == RIG_PTT_SERIAL_DTR)
+            {
+              rig_debug(RIG_DEBUG_ERR, "Cannot set DTR with PTT by DTR \"%s\"\n",
+                        rs->rigport.pathname);
+              return -RIG_ECONF;
+            }
+        }
     }
 
-    status = port_open(&rs->rigport);
-    if (status < 0)
-        return status;
+  status = port_open(&rs->rigport);
+  if (status < 0)
+    return status;
 
-
-	/*
-	 * FIXME: what to do if PTT open fails or PTT unsupported?
-	 * 			fail rig_open?  remember unallocating..
-	 */
 	switch(rs->pttport.type.ptt) {
 	case RIG_PTT_NONE:
 	case RIG_PTT_RIG:
@@ -503,40 +512,73 @@ int HAMLIB_API rig_open(RIG *rig)
 		break;
 	case RIG_PTT_SERIAL_RTS:
 	case RIG_PTT_SERIAL_DTR:
-		if (rs->pttport.pathname[0] == '\0' &&
-				rs->rigport.type.rig == RIG_PORT_SERIAL)
-			strcpy(rs->pttport.pathname, rs->rigport.pathname);
-		rs->pttport.fd = ser_open(&rs->pttport);
-		if (rs->pttport.fd < 0)
-			rig_debug(RIG_DEBUG_ERR, "Cannot open PTT device \"%s\"\n",
-						rs->pttport.pathname);
-        else {
-			/* Needed on Linux because the kernel forces RTS/DTR at open */
-			if (rs->pttport.type.ptt == RIG_PTT_SERIAL_DTR)
-				ser_set_dtr(&rs->pttport, RIG_PTT_OFF);
-			else if (rs->pttport.type.ptt == RIG_PTT_SERIAL_RTS)
-				ser_set_rts(&rs->pttport, RIG_PTT_OFF);
-		}
-		break;
+	  if (rs->pttport.pathname[0] == '\0' &&
+	      rs->rigport.type.rig == RIG_PORT_SERIAL)
+	    strcpy(rs->pttport.pathname, rs->rigport.pathname);
+	  if (!strcmp(rs->pttport.pathname, rs->rigport.pathname))
+	    {
+	      rs->pttport.fd = rs->rigport.fd;
+
+				/* Needed on Linux because the serial port driver sets RTS/DTR
+					 on open - only need to address the PTT line as we offer
+					 config parameters to control the other (dtr_state &
+					 rts_state) */
+				if (rs->pttport.type.ptt == RIG_PTT_SERIAL_DTR)
+						status = ser_set_dtr(&rs->pttport, 0);
+				if (rs->pttport.type.ptt == RIG_PTT_SERIAL_RTS) 
+						status = ser_set_rts(&rs->pttport, 0);
+	    }
+	  else
+	    {
+	      rs->pttport.fd = ser_open(&rs->pttport);
+	      if (rs->pttport.fd < 0)
+          {
+            rig_debug(RIG_DEBUG_ERR, "Cannot open PTT device \"%s\"\n",
+                      rs->pttport.pathname);
+            status = -RIG_EIO;
+          }
+				if (RIG_OK == status
+						&& (rs->pttport.type.ptt == RIG_PTT_SERIAL_DTR
+								|| rs->pttport.type.ptt == RIG_PTT_SERIAL_RTS))
+					{
+						/* Needed on Linux because the serial port driver sets
+							 RTS/DTR high on open - set both low since we offer no
+							 control of the non-PTT line and low is better than
+							 high */
+						status = ser_set_dtr(&rs->pttport, 0);
+						if (RIG_OK == status)
+							{
+								status = ser_set_rts(&rs->pttport, 0);
+							}
+					}
+	    }
+	  break;
 	case RIG_PTT_PARALLEL:
 		rs->pttport.fd = par_open(&rs->pttport);
 		if (rs->pttport.fd < 0)
-			rig_debug(RIG_DEBUG_ERR, "Cannot open PTT device \"%s\"\n",
-						rs->pttport.pathname);
-        else
+		  {
+		    rig_debug(RIG_DEBUG_ERR, "Cannot open PTT device \"%s\"\n",
+                  rs->pttport.pathname);
+		    status = -RIG_EIO;
+		  }
+    else
 			par_ptt_set(&rs->pttport, RIG_PTT_OFF);
 		break;
 	case RIG_PTT_CM108:
 		rs->pttport.fd = cm108_open(&rs->pttport);
 		if (rs->pttport.fd < 0)
-			rig_debug(RIG_DEBUG_ERR, "Cannot open PTT device \"%s\"\n",
-				  rs->pttport.pathname);
+		  {
+		    rig_debug(RIG_DEBUG_ERR, "Cannot open PTT device \"%s\"\n",
+                  rs->pttport.pathname);
+		    status = -RIG_EIO;
+		  }
 		else
 			cm108_ptt_set(&rs->pttport, RIG_PTT_OFF);
 		break;
 	default:
 		rig_debug(RIG_DEBUG_ERR, "Unsupported PTT type %d\n",
-						rs->pttport.type.ptt);
+              rs->pttport.type.ptt);
+		status = -RIG_ECONF;
 	}
 
 	switch(rs->dcdport.type.dcd) {
@@ -551,20 +593,33 @@ int HAMLIB_API rig_open(RIG *rig)
 			strcpy(rs->dcdport.pathname, rs->rigport.pathname);
 		rs->dcdport.fd = ser_open(&rs->dcdport);
 		if (rs->dcdport.fd < 0)
-			rig_debug(RIG_DEBUG_ERR, "Cannot open DCD device \"%s\"\n",
-						rs->dcdport.pathname);
+		  {
+		    rig_debug(RIG_DEBUG_ERR, "Cannot open DCD device \"%s\"\n",
+                  rs->dcdport.pathname);
+		    status = -RIG_EIO;
+		  }
 		break;
 	case RIG_DCD_PARALLEL:
 		rs->dcdport.fd = par_open(&rs->dcdport);
 		if (rs->dcdport.fd < 0)
-			rig_debug(RIG_DEBUG_ERR, "Cannot open DCD device \"%s\"\n",
-						rs->dcdport.pathname);
+		  {
+		    rig_debug(RIG_DEBUG_ERR, "Cannot open DCD device \"%s\"\n",
+                  rs->dcdport.pathname);
+		    status = -RIG_EIO;
+		  }
 		break;
 	default:
 		rig_debug(RIG_DEBUG_ERR, "Unsupported DCD type %d\n",
-						rs->dcdport.type.dcd);
+              rs->dcdport.type.dcd);
+		status = -RIG_ECONF;
 	}
 
+	if (status < 0)
+	  {
+	    port_close (&rs->rigport, rs->rigport.type.rig);
+	    return status;
+	  }
+
 	add_opened_rig(rig);
 
 	rs->comm_state = 1;
@@ -583,19 +638,19 @@ int HAMLIB_API rig_open(RIG *rig)
 	/*
 	 * trigger state->current_vfo first retrieval
 	 */
-        if (rig_get_vfo(rig, &rs->current_vfo) == RIG_OK)
+  if (rig_get_vfo(rig, &rs->current_vfo) == RIG_OK)
 		rs->tx_vfo = rs->current_vfo;
 
 #if 0
-        /*
-         * Check the current tranceive state of the rig
-         */
-        if (rs->transceive == RIG_TRN_RIG) {
-            int retval, trn;
-            retval = rig_get_trn(rig, &trn);
-            if (retval == RIG_OK && trn == RIG_TRN_RIG)
-                add_trn_rig(rig);
-        }
+  /*
+   * Check the current tranceive state of the rig
+   */
+  if (rs->transceive == RIG_TRN_RIG) {
+    int retval, trn;
+    retval = rig_get_trn(rig, &trn);
+    if (retval == RIG_OK && trn == RIG_TRN_RIG)
+      add_trn_rig(rig);
+  }
 #endif
 	return RIG_OK;
 }
@@ -651,12 +706,18 @@ int HAMLIB_API rig_close(RIG *rig)
 	case RIG_PTT_RIG_MICDATA:
 		break;
 	case RIG_PTT_SERIAL_RTS:
-		ser_set_rts(&rs->pttport, RIG_PTT_OFF);
-		port_close(&rs->pttport, RIG_PORT_SERIAL);
+		ser_set_rts(&rs->pttport, 0);
+		if (rs->pttport.fd != rs->rigport.fd)
+		  {
+		    port_close(&rs->pttport, RIG_PORT_SERIAL);
+		  }
 		break;
 	case RIG_PTT_SERIAL_DTR:
-		ser_set_dtr(&rs->pttport, RIG_PTT_OFF);
-		port_close(&rs->pttport, RIG_PORT_SERIAL);
+		ser_set_dtr(&rs->pttport, 0);
+		if (rs->pttport.fd != rs->rigport.fd)
+		  {
+		    port_close(&rs->pttport, RIG_PORT_SERIAL);
+		  }
 		break;
 	case RIG_PTT_PARALLEL:
         par_ptt_set(&rs->pttport, RIG_PTT_OFF);
@@ -756,7 +817,7 @@ int HAMLIB_API rig_cleanup(RIG *rig)
 int HAMLIB_API rig_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig))
@@ -782,7 +843,12 @@ int HAMLIB_API rig_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 			return retcode;
 
 		retcode = caps->set_freq(rig, vfo, freq);
-		caps->set_vfo(rig, curr_vfo);
+		rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we
+                                           had an error above */
+    if (RIG_OK == retcode)
+      {
+        retcode = rc2;          /* return the first error code */
+      }
 	}
 	if (retcode == RIG_OK &&
 			(vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo))
@@ -811,7 +877,7 @@ int HAMLIB_API rig_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 int HAMLIB_API rig_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig) || !freq)
@@ -833,7 +899,12 @@ int HAMLIB_API rig_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 		if (retcode != RIG_OK)
 			return retcode;
 		retcode = caps->get_freq(rig, vfo, freq);
-		caps->set_vfo(rig, curr_vfo);
+		rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we
+                                           had an error above */
+    if (RIG_OK == retcode)
+      {
+        retcode = rc2;          /* return the first error code */
+      }
 	}
 	/* VFO compensation */
 	if (rig->state.vfo_comp != 0.0)
@@ -867,7 +938,7 @@ int HAMLIB_API rig_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 int HAMLIB_API rig_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig))
@@ -891,7 +962,12 @@ int HAMLIB_API rig_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 			return retcode;
 
 		retcode = caps->set_mode(rig, vfo, mode, width);
-		caps->set_vfo(rig, curr_vfo);
+		rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we
+                                           had an error above */
+    if (RIG_OK == retcode)
+      {
+        retcode = rc2;          /* return the first error code */
+      }
 	}
 
 	if (retcode == RIG_OK &&
@@ -926,7 +1002,7 @@ int HAMLIB_API rig_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 int HAMLIB_API rig_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig) || !mode || !width)
@@ -950,7 +1026,12 @@ int HAMLIB_API rig_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width
 			return retcode;
 
 		retcode = caps->get_mode(rig, vfo, mode, width);
-		caps->set_vfo(rig, curr_vfo);
+		rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we
+                                           had an error above */
+    if (RIG_OK == retcode)
+      {
+        retcode = rc2;          /* return the first error code */
+      }
 	}
 
 	if (retcode == RIG_OK &&
@@ -1168,14 +1249,13 @@ int HAMLIB_API rig_get_vfo(RIG *rig, vfo_t *vfo)
 int HAMLIB_API rig_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig))
 		return -RIG_EINVAL;
 
 	caps = rig->caps;
-
 	switch (rig->state.pttport.type.ptt) {
 	case RIG_PTT_RIG:
 		if (ptt == RIG_PTT_ON_MIC || ptt == RIG_PTT_ON_DATA)
@@ -1197,7 +1277,12 @@ int HAMLIB_API rig_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
 			return retcode;
 
 		retcode = caps->set_ptt(rig, vfo, ptt);
-		caps->set_vfo(rig, curr_vfo);
+		rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we
+                                           had an error above */
+    if (RIG_OK == retcode)
+      {
+        retcode = rc2;          /* return the first error code */
+      }
 		return retcode;
 
 		break;
@@ -1240,7 +1325,7 @@ int HAMLIB_API rig_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
 int HAMLIB_API rig_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
 {
 	const struct rig_caps *caps;
-	int retcode, status;
+	int retcode, rc2, status;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig) || !ptt)
@@ -1266,7 +1351,13 @@ int HAMLIB_API rig_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
 			return retcode;
 
 		retcode = caps->get_ptt(rig, vfo, ptt);
-		caps->set_vfo(rig, curr_vfo);
+		rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we
+                                           had an error above */
+    if (RIG_OK == retcode)
+      {
+        retcode = rc2;          /* return the first error code */
+      }
+
 		return retcode;
 
 		break;
@@ -1325,7 +1416,7 @@ int HAMLIB_API rig_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
 int HAMLIB_API rig_get_dcd(RIG *rig, vfo_t vfo, dcd_t *dcd)
 {
 	const struct rig_caps *caps;
-	int retcode, status;
+	int retcode, rc2, status;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig) || !dcd)
@@ -1350,7 +1441,13 @@ int HAMLIB_API rig_get_dcd(RIG *rig, vfo_t vfo, dcd_t *dcd)
 			return retcode;
 
 		retcode = caps->get_dcd(rig, vfo, dcd);
-		caps->set_vfo(rig, curr_vfo);
+		rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we
+                                           had an error above */
+    if (RIG_OK == retcode)
+      {
+        retcode = rc2;          /* return the first error code */
+      }
+
 		return retcode;
 
 		break;
@@ -1402,7 +1499,7 @@ int HAMLIB_API rig_get_dcd(RIG *rig, vfo_t vfo, dcd_t *dcd)
 int HAMLIB_API rig_set_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t rptr_shift)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig))
@@ -1425,7 +1522,13 @@ int HAMLIB_API rig_set_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t rptr_shift)
 		return retcode;
 
 	retcode = caps->set_rptr_shift(rig, vfo, rptr_shift);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                         an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
+
 	return retcode;
 }
 
@@ -1446,7 +1549,7 @@ int HAMLIB_API rig_set_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t rptr_shift)
 int HAMLIB_API rig_get_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t *rptr_shift)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig) || !rptr_shift)
@@ -1469,7 +1572,13 @@ int HAMLIB_API rig_get_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t *rptr_shift)
 		return retcode;
 
 	retcode = caps->get_rptr_shift(rig, vfo, rptr_shift);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                         an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;          /* return the first error code */
+    }
+
 	return retcode;
 }
 
@@ -1491,7 +1600,7 @@ int HAMLIB_API rig_get_rptr_shift(RIG *rig, vfo_t vfo, rptr_shift_t *rptr_shift)
 int HAMLIB_API rig_set_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t rptr_offs)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig))
@@ -1514,7 +1623,13 @@ int HAMLIB_API rig_set_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t rptr_offs)
 		return retcode;
 
 	retcode = caps->set_rptr_offs(rig, vfo, rptr_offs);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                         an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;          /* return the first error code */
+    }
+
 	return retcode;
 }
 
@@ -1536,7 +1651,7 @@ int HAMLIB_API rig_set_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t rptr_offs)
 int HAMLIB_API rig_get_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t *rptr_offs)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig) || !rptr_offs)
@@ -1559,7 +1674,13 @@ int HAMLIB_API rig_get_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t *rptr_offs)
 		return retcode;
 
 	retcode = caps->get_rptr_offs(rig, vfo, rptr_offs);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                         an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;          /* return the first error code */
+    }
+
 	return retcode;
 }
 
@@ -1582,7 +1703,7 @@ int HAMLIB_API rig_get_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t *rptr_offs)
 int HAMLIB_API rig_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo, tx_vfo;
 
 	if (CHECK_RIG_ARG(rig))
@@ -1625,11 +1746,16 @@ int HAMLIB_API rig_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
 	else
 		retcode = caps->set_freq(rig, RIG_VFO_CURR, tx_freq);
 
+  /* try and revert even if we had an error above */
 	if (caps->set_vfo) {
-		caps->set_vfo(rig, curr_vfo);
+		rc2 = caps->set_vfo(rig, curr_vfo);
 	} else {
-		caps->vfo_op(rig, vfo, RIG_OP_TOGGLE);
+		rc2 = caps->vfo_op(rig, vfo, RIG_OP_TOGGLE);
 	}
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;          /* return the first error code */
+    }
 	return retcode;
 }
 
@@ -1650,7 +1776,7 @@ int HAMLIB_API rig_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
 int HAMLIB_API rig_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo, tx_vfo;
 
 	if (CHECK_RIG_ARG(rig) || !tx_freq)
@@ -1693,11 +1819,16 @@ int HAMLIB_API rig_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
 	else
 		retcode = caps->get_freq(rig, RIG_VFO_CURR, tx_freq);
 
+ /* try and revert even if we had an error above */
 	if (caps->set_vfo) {
-		caps->set_vfo(rig, curr_vfo);
+		rc2 = caps->set_vfo(rig, curr_vfo);
 	} else {
-		caps->vfo_op(rig, vfo, RIG_OP_TOGGLE);
+		rc2 = caps->vfo_op(rig, vfo, RIG_OP_TOGGLE);
 	}
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
 
 	return retcode;
 }
@@ -1721,7 +1852,7 @@ int HAMLIB_API rig_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
 int HAMLIB_API rig_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo, tx_vfo;
 
 	if (CHECK_RIG_ARG(rig))
@@ -1764,11 +1895,16 @@ int HAMLIB_API rig_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_
 	else
 		retcode = caps->set_mode(rig, RIG_VFO_CURR, tx_mode, tx_width);
 
+ /* try and revert even if we had an error above */
 	if (caps->set_vfo) {
-		caps->set_vfo(rig, curr_vfo);
+		rc2 = caps->set_vfo(rig, curr_vfo);
 	} else {
-		caps->vfo_op(rig, vfo, RIG_OP_TOGGLE);
+		rc2 = caps->vfo_op(rig, vfo, RIG_OP_TOGGLE);
 	}
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
 
 	return retcode;
 }
@@ -1795,7 +1931,7 @@ int HAMLIB_API rig_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_
 int HAMLIB_API rig_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *tx_mode, pbwidth_t *tx_width)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo, tx_vfo;
 
 	if (CHECK_RIG_ARG(rig) || !tx_mode || !tx_width)
@@ -1838,11 +1974,16 @@ int HAMLIB_API rig_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *tx_mode, pbwidth
 	else
 		retcode = caps->get_mode(rig, RIG_VFO_CURR, tx_mode, tx_width);
 
+  /* try and revert even if we had an error above */
 	if (caps->set_vfo) {
-		caps->set_vfo(rig, curr_vfo);
+		rc2 = caps->set_vfo(rig, curr_vfo);
 	} else {
-		caps->vfo_op(rig, vfo, RIG_OP_TOGGLE);
+		rc2 = caps->vfo_op(rig, vfo, RIG_OP_TOGGLE);
 	}
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
 
 	if (*tx_width == RIG_PASSBAND_NORMAL && *tx_mode != RIG_MODE_NONE)
 		*tx_width = rig_passband_normal (rig, *tx_mode);
@@ -1869,7 +2010,7 @@ int HAMLIB_API rig_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *tx_mode, pbwidth
 int HAMLIB_API rig_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t tx_vfo)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig))
@@ -1897,7 +2038,12 @@ int HAMLIB_API rig_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t tx_vf
 		return retcode;
 
 	retcode = caps->set_split_vfo(rig, vfo, split, tx_vfo);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                           an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
 
 	if (retcode == RIG_OK)
 		rig->state.tx_vfo = tx_vfo;
@@ -1923,7 +2069,7 @@ int HAMLIB_API rig_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t tx_vf
 int HAMLIB_API rig_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_vfo)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig) || !split || !tx_vfo)
@@ -1949,7 +2095,13 @@ int HAMLIB_API rig_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_
 		return retcode;
 
 	retcode = caps->get_split_vfo(rig, vfo, split, tx_vfo);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                         an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
+
 	return retcode;
 }
 
@@ -1971,7 +2123,7 @@ int HAMLIB_API rig_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_
 int HAMLIB_API rig_set_rit(RIG *rig, vfo_t vfo, shortfreq_t rit)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig))
@@ -1994,7 +2146,13 @@ int HAMLIB_API rig_set_rit(RIG *rig, vfo_t vfo, shortfreq_t rit)
 		return retcode;
 
 	retcode = caps->set_rit(rig, vfo, rit);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                         an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
+
 	return retcode;
 }
 
@@ -2016,7 +2174,7 @@ int HAMLIB_API rig_set_rit(RIG *rig, vfo_t vfo, shortfreq_t rit)
 int HAMLIB_API rig_get_rit(RIG *rig, vfo_t vfo, shortfreq_t *rit)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig) || !rit)
@@ -2039,7 +2197,13 @@ int HAMLIB_API rig_get_rit(RIG *rig, vfo_t vfo, shortfreq_t *rit)
 		return retcode;
 
 	retcode = caps->get_rit(rig, vfo, rit);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                         an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
+
 	return retcode;
 }
 
@@ -2061,7 +2225,7 @@ int HAMLIB_API rig_get_rit(RIG *rig, vfo_t vfo, shortfreq_t *rit)
 int HAMLIB_API rig_set_xit(RIG *rig, vfo_t vfo, shortfreq_t xit)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig))
@@ -2084,7 +2248,13 @@ int HAMLIB_API rig_set_xit(RIG *rig, vfo_t vfo, shortfreq_t xit)
 		return retcode;
 
 	retcode = caps->set_xit(rig, vfo, xit);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                         an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
+
 	return retcode;
 }
 
@@ -2106,7 +2276,7 @@ int HAMLIB_API rig_set_xit(RIG *rig, vfo_t vfo, shortfreq_t xit)
 int HAMLIB_API rig_get_xit(RIG *rig, vfo_t vfo, shortfreq_t *xit)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig) || !xit)
@@ -2129,7 +2299,13 @@ int HAMLIB_API rig_get_xit(RIG *rig, vfo_t vfo, shortfreq_t *xit)
 		return retcode;
 
 	retcode = caps->get_xit(rig, vfo, xit);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                         an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
+
 	return retcode;
 }
 
@@ -2153,7 +2329,7 @@ int HAMLIB_API rig_get_xit(RIG *rig, vfo_t vfo, shortfreq_t *xit)
 int HAMLIB_API rig_set_ts(RIG *rig, vfo_t vfo, shortfreq_t ts)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig))
@@ -2176,7 +2352,13 @@ int HAMLIB_API rig_set_ts(RIG *rig, vfo_t vfo, shortfreq_t ts)
 		return retcode;
 
 	retcode = caps->set_ts(rig, vfo, ts);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                         an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
+
 	return retcode;
 }
 
@@ -2198,7 +2380,7 @@ int HAMLIB_API rig_set_ts(RIG *rig, vfo_t vfo, shortfreq_t ts)
 int HAMLIB_API rig_get_ts(RIG *rig, vfo_t vfo, shortfreq_t *ts)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig) || !ts)
@@ -2221,7 +2403,13 @@ int HAMLIB_API rig_get_ts(RIG *rig, vfo_t vfo, shortfreq_t *ts)
 		return retcode;
 
 	retcode = caps->get_ts(rig, vfo, ts);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                         an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
+
 	return retcode;
 }
 
@@ -2247,7 +2435,7 @@ int HAMLIB_API rig_get_ts(RIG *rig, vfo_t vfo, shortfreq_t *ts)
 int HAMLIB_API rig_set_ant(RIG *rig, vfo_t vfo, ant_t ant)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig))
@@ -2270,7 +2458,13 @@ int HAMLIB_API rig_set_ant(RIG *rig, vfo_t vfo, ant_t ant)
 		return retcode;
 
 	retcode = caps->set_ant(rig, vfo, ant);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                         an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
+
 	return retcode;
 }
 
@@ -2292,7 +2486,7 @@ int HAMLIB_API rig_set_ant(RIG *rig, vfo_t vfo, ant_t ant)
 int HAMLIB_API rig_get_ant(RIG *rig, vfo_t vfo, ant_t *ant)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig) || !ant)
@@ -2315,7 +2509,13 @@ int HAMLIB_API rig_get_ant(RIG *rig, vfo_t vfo, ant_t *ant)
 		return retcode;
 
 	retcode = caps->get_ant(rig, vfo, ant);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                         an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
+
 	return retcode;
 }
 
@@ -2610,7 +2810,7 @@ vfo_op_t HAMLIB_API rig_has_vfo_op(RIG *rig, vfo_op_t op)
 int HAMLIB_API rig_vfo_op(RIG *rig, vfo_t vfo, vfo_op_t op)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig))
@@ -2633,7 +2833,13 @@ int HAMLIB_API rig_vfo_op(RIG *rig, vfo_t vfo, vfo_op_t op)
 		return retcode;
 
 	retcode = caps->vfo_op(rig, vfo, op);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                         an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
+
 	return retcode;
 }
 
@@ -2681,7 +2887,7 @@ scan_t HAMLIB_API rig_has_scan(RIG *rig, scan_t scan)
 int HAMLIB_API rig_scan(RIG *rig, vfo_t vfo, scan_t scan, int ch)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig))
@@ -2705,7 +2911,13 @@ int HAMLIB_API rig_scan(RIG *rig, vfo_t vfo, scan_t scan, int ch)
 		return retcode;
 
 	retcode = caps->scan(rig, vfo, scan, ch);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                         an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
+
 	return retcode;
 }
 
@@ -2727,7 +2939,7 @@ int HAMLIB_API rig_scan(RIG *rig, vfo_t vfo, scan_t scan, int ch)
 int HAMLIB_API rig_send_dtmf(RIG *rig, vfo_t vfo, const char *digits)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig) || !digits)
@@ -2750,7 +2962,13 @@ int HAMLIB_API rig_send_dtmf(RIG *rig, vfo_t vfo, const char *digits)
 		return retcode;
 
 	retcode = caps->send_dtmf(rig, vfo, digits);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                         an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
+
 	return retcode;
 }
 
@@ -2773,7 +2991,7 @@ int HAMLIB_API rig_send_dtmf(RIG *rig, vfo_t vfo, const char *digits)
 int HAMLIB_API rig_recv_dtmf(RIG *rig, vfo_t vfo, char *digits, int *length)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig) || !digits || !length)
@@ -2796,7 +3014,13 @@ int HAMLIB_API rig_recv_dtmf(RIG *rig, vfo_t vfo, char *digits, int *length)
 		return retcode;
 
 	retcode = caps->recv_dtmf(rig, vfo, digits, length);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                         an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
+
 	return retcode;
 }
 
@@ -2818,7 +3042,7 @@ int HAMLIB_API rig_recv_dtmf(RIG *rig, vfo_t vfo, char *digits, int *length)
 int HAMLIB_API rig_send_morse(RIG *rig, vfo_t vfo, const char *msg)
 {
 	const struct rig_caps *caps;
-	int retcode;
+	int retcode, rc2;
 	vfo_t curr_vfo;
 
 	if (CHECK_RIG_ARG(rig) || !msg)
@@ -2841,7 +3065,13 @@ int HAMLIB_API rig_send_morse(RIG *rig, vfo_t vfo, const char *msg)
 		return retcode;
 
 	retcode = caps->send_morse(rig, vfo, msg);
-	caps->set_vfo(rig, curr_vfo);
+  rc2 = caps->set_vfo(rig, curr_vfo); /* try and revert even if we had
+                                         an error above */
+  if (RIG_OK == retcode)
+    {
+      retcode = rc2;            /* return the first error code */
+    }
+
 	return retcode;
 }
 
