@@ -1,6 +1,6 @@
 /*
  *  Hamlib Interface - serial communication low-level support
- *  Copyright (c) 2000-2010 by Stephane Fillod
+ *  Copyright (c) 2000-2013 by Stephane Fillod
  *  Copyright (c) 2000-2003 by Frank Singleton
  *  Parts of the PTT handling are derived from soundmodem, an excellent
  *  ham packet softmodem written by Thomas Sailer, HB9JNX.
@@ -346,6 +346,11 @@ int HAMLIB_API serial_setup(hamlib_port_t *rp)
   sg.sg_flags = RAW;
 #endif
 
+  /*
+   * VTIME in deciseconds, rp->timeout in miliseconds
+   */
+  options.c_cc[VTIME] = (rp->timeout + 99) / 100;
+  options.c_cc[VMIN] = 1;
 
   /*
    * Flush serial port
@@ -425,20 +430,29 @@ int ser_close(hamlib_port_t *p)
 int HAMLIB_API ser_set_rts(hamlib_port_t *p, int state)
 {
 	unsigned int y = TIOCM_RTS;
+  int rc;
+
+  rig_debug(RIG_DEBUG_VERBOSE, "%s: RTS=%d\n", __func__, state);
 
 #if defined(TIOCMBIS) && defined(TIOCMBIC)
-	return IOCTL(p->fd, state ? TIOCMBIS : TIOCMBIC, &y) < 0 ?
-			-RIG_EIO : RIG_OK;
+	rc = IOCTL(p->fd, state ? TIOCMBIS : TIOCMBIC, &y);
 #else
-	if (IOCTL(p->fd, TIOCMGET, &y) < 0) {
-		return -RIG_EIO;
-	}
-	if (state)
-		y |= TIOCM_RTS;
-	else
-		y &= ~TIOCM_RTS;
-	return IOCTL(p->fd, TIOCMSET, &y) < 0 ? -RIG_EIO : RIG_OK;
+	rc = IOCTL(p->fd, TIOCMGET, &y);
+  if (rc >= 0)
+    {
+      if (state)
+        y |= TIOCM_RTS;
+      else
+        y &= ~TIOCM_RTS;
+      rc = IOCTL(p->fd, TIOCMSET, &y);
+    }
 #endif
+  if (rc < 0)
+    {
+      rig_debug(RIG_DEBUG_ERR, "%s: Cannot change RTS - %s\n", __func__, strerror(errno));
+      return -RIG_EIO;
+    }
+  return RIG_OK;
 }
 
 /**
@@ -466,20 +480,29 @@ int HAMLIB_API ser_get_rts(hamlib_port_t *p, int *state)
 int HAMLIB_API ser_set_dtr(hamlib_port_t *p, int state)
 {
 	unsigned int y = TIOCM_DTR;
+  int rc;
+
+  rig_debug(RIG_DEBUG_VERBOSE, "%s: DTR=%d\n", __func__, state);
 
 #if defined(TIOCMBIS) && defined(TIOCMBIC)
-	return IOCTL(p->fd, state ? TIOCMBIS : TIOCMBIC, &y) < 0 ?
-			-RIG_EIO : RIG_OK;
+	rc = IOCTL(p->fd, state ? TIOCMBIS : TIOCMBIC, &y);
 #else
-	if (IOCTL(p->fd, TIOCMGET, &y) < 0) {
-		return -RIG_EIO;
-	}
-	if (state)
-		y |= TIOCM_DTR;
-	else
-		y &= ~TIOCM_DTR;
-	return IOCTL(p->fd, TIOCMSET, &y) < 0 ? -RIG_EIO : RIG_OK;
+	rc = IOCTL(p->fd, TIOCMGET, &y);
+  if (rc >= 0)
+    {
+      if (state)
+        y |= TIOCM_DTR;
+      else
+        y &= ~TIOCM_DTR;
+      rc = IOCTL(p->fd, TIOCMSET, &y);
+    }
 #endif
+  if (rc < 0)
+    {
+      rig_debug(RIG_DEBUG_ERR, "%s: Cannot change DTR - %s\n", __func__, strerror(errno));
+      return -RIG_EIO;
+    }
+  return RIG_OK;
 }
 
 /**
