@@ -33,12 +33,29 @@
 #include <math.h>
 
 #include <hamlib/rig.h>
-#include <serial.h>
-#include <misc.h>
-#include <cal.h>
+#include "serial.h"
+#include "misc.h"
+#include "cal.h"
 
 #include "tt550.h"
 
+/* HAVE_SSLEEP is defined when Windows Sleep is found
+ * HAVE_SLEEP is defined when POSIX sleep is found
+ * _WIN32 is defined when compiling with MinGW
+ *
+ * When cross-compiling from POSIX to Windows using MinGW, HAVE_SLEEP
+ * will often be defined by configure although it is not supported by
+ * MinGW.  So substitute the sleep definition below in such a case and
+ * when compiling on Windows using MinGW where HAVE_SLEEP will be
+ * undefined.
+ *
+ * FIXME:  Needs better handling for all versions of MinGW.
+ *
+ */
+#if (defined(HAVE_SSLEEP) || defined(_WIN32)) && (!defined(HAVE_SLEEP))
+#include "hl_sleep.h"
+#endif
+
 
 /*
  * Filter table for 550 reciver support
@@ -100,7 +117,7 @@ tt550_transaction (RIG * rig, const char *cmd, int cmd_len, char *data,
     return 0;
   }
 
-  retval = read_string (&rs->rigport, data, *data_len, "", 0);
+  retval = read_string (&rs->rigport, data, *data_len, NULL, 0);
   if (retval == -RIG_ETIMEOUT)
 	retval = 0;
   if (retval < 0)
