@@ -1,5 +1,7 @@
 /*
- * rigctl.c - (C) Stephane Fillod 2000-2009
+ * rigctl.c - (C) Stephane Fillod 2000-2010
+ *            (C) Nate Bargmann 2003,2006,2008,2010,2011,2012,2013
+ *            (C) The Hamlib Group 2002,2006,2007,2012
  *
  * This program test/control a radio using Hamlib.
  * It takes commands in interactive mode as well as
@@ -32,9 +34,38 @@
 #include <unistd.h>
 #include <ctype.h>
 #include <errno.h>
-
 #include <getopt.h>
 
+#ifdef HAVE_LIBREADLINE
+#  if defined(HAVE_READLINE_READLINE_H)
+#    include <readline/readline.h>
+#  elif defined(HAVE_READLINE_H)    /* !defined(HAVE_READLINE_READLINE_H) */
+#    include <readline.h>
+#  else                             /* !defined(HAVE_READLINE_H) */
+extern char *readline ();
+#  endif                            /* HAVE_READLINE_H */
+#else
+/* no readline */
+#endif                              /* HAVE_LIBREADLINE */
+
+#ifdef HAVE_READLINE_HISTORY
+#  include <sys/stat.h>
+#  define HST_SHRT_OPTS "iI"
+#  if defined(HAVE_READLINE_HISTORY_H)
+#    include <readline/history.h>
+#  elif defined(HAVE_HISTORY_H)
+#    include <history.h>
+#  else                             /* !defined(HAVE_HISTORY_H) */
+extern void add_history ();
+extern int write_history ();
+extern int read_history ();
+#  endif                            /* defined(HAVE_READLINE_HISTORY_H) */
+#else
+/* no history */
+#define HST_SHRT_OPTS ""
+#endif                              /* HAVE_READLINE_HISTORY */
+
+
 #include <hamlib/rig.h>
 #include "misc.h"
 #include "iofunc.h"
@@ -73,6 +104,10 @@ static struct option long_options[] =
 	{"show-conf",       0, 0, 'L'},
 	{"dump-caps",       0, 0, 'u'},
 	{"vfo",             0, 0, 'o'},
+#ifdef HAVE_READLINE_HISTORY
+	{"read-history",    0, 0, 'i'},
+	{"save-history",    0, 0, 'I'},
+#endif
 	{"verbose",         0, 0, 'v'},
 	{"help",            0, 0, 'h'},
 	{"version",         0, 0, 'V'},
@@ -81,9 +116,16 @@ static struct option long_options[] =
 
 #define MAXCONFLEN 128
 
+/* variable for readline support */
+#ifdef HAVE_LIBREADLINE
+static const int have_rl = 1;
+#else                               /* no readline */
+static const int have_rl = 0;
+#endif
+
+
 int interactive = 1;    /* if no cmd on command line, switch to interactive */
 int prompt = 1;         /* Print prompt in rigctl */
-int opt_end = 0;        /* only used by rigctld */
 int vfo_mode = 0;       /* vfo_mode = 0 means target VFO is 'currVFO' */
 
 char send_cmd_term = '\r';  /* send_cmd termination char */
@@ -99,6 +141,14 @@ int main (int argc, char *argv[])
 	int verbose = 0;
 	int show_conf = 0;
 	int dump_caps_opt = 0;
+#ifdef HAVE_READLINE_HISTORY
+	int rd_hist = 0;
+	int sv_hist = 0;
+	const char *hist_dir = NULL;
+	const char hist_file[] = "/.rigctl_history";
+	char *hist_path = NULL;
+	struct stat hist_dir_stat;
+#endif
 	const char *rig_file=NULL, *ptt_file=NULL, *dcd_file=NULL;
 	ptt_type_t ptt_type = RIG_PTT_NONE;
 	dcd_type_t dcd_type = RIG_DCD_NONE;
@@ -110,7 +160,7 @@ int main (int argc, char *argv[])
 		int c;
 		int option_index = 0;
 
-		c = getopt_long (argc, argv, SHORT_OPTIONS,
+		c = getopt_long (argc, argv, SHORT_OPTIONS HST_SHRT_OPTS,
 			long_options, &option_index);
 		if (c == -1)
 			break;
@@ -228,6 +278,14 @@ int main (int argc, char *argv[])
 			case 'o':
 				vfo_mode++;
 				break;
+#ifdef HAVE_READLINE_HISTORY
+			case 'i':
+				rd_hist++;
+				break;
+			case 'I':
+				sv_hist++;
+				break;
+#endif
 			case 'v':
 				verbose++;
 				break;
@@ -326,6 +384,34 @@ int main (int argc, char *argv[])
 
 	exitcode = 0;
 
+#ifdef HAVE_LIBREADLINE
+	if (interactive && prompt && have_rl) {
+		rl_readline_name = "rigctl";
+#ifdef HAVE_READLINE_HISTORY
+		using_history();	/* Initialize Readline History */
+
+		if (rd_hist || sv_hist) {
+			if (!(hist_dir = getenv("RIGCTL_HIST_DIR")))
+				hist_dir = getenv("HOME");
+
+			if (((stat(hist_dir, &hist_dir_stat) == -1) && (errno == ENOENT))
+				|| !(S_ISDIR(hist_dir_stat.st_mode))) {
+				fprintf(stderr, "Warning: %s is not a directory!\n", hist_dir);
+			}
+
+			hist_path = (char *)calloc((sizeof(char) * (strlen(hist_dir) + strlen(hist_file) + 1)), sizeof(char));
+
+			strncpy(hist_path, hist_dir, strlen(hist_dir));
+			strncat(hist_path, hist_file, strlen(hist_file));
+		}
+
+		if (rd_hist && hist_path)
+			if (read_history(hist_path) == ENOENT)
+				fprintf(stderr, "Warning: Could not read history from %s\n", hist_path);
+#endif
+	}
+#endif	/* HAVE_LIBREADLINE */
+
 	do {
 		retcode = rigctl_parse(my_rig, stdin, stdout, argv, argc);
 		if (retcode == 2)
@@ -333,6 +419,20 @@ int main (int argc, char *argv[])
 	}
 	while (retcode == 0 || retcode == 2);
 
+#ifdef HAVE_LIBREADLINE
+	if (interactive && prompt && have_rl) {
+#ifdef HAVE_READLINE_HISTORY
+		if (sv_hist && hist_path)
+			if (write_history(hist_path) == ENOENT)
+				fprintf(stderr, "\nWarning: Could not write history to %s\n", hist_path);
+
+		if ((rd_hist || sv_hist) && hist_path) {
+			free(hist_path);
+			hist_path = (char *)NULL;
+		}
+#endif
+	}
+#endif
 	rig_close(my_rig); /* close port */
 	rig_cleanup(my_rig); /* if you care about memory */
 
@@ -361,6 +461,10 @@ void usage(void)
 	"  -l, --list                 list all model numbers and exit\n"
 	"  -u, --dump-caps            dump capabilities and exit\n"
 	"  -o, --vfo                  do not default to VFO_CURR, require extra vfo arg\n"
+#ifdef HAVE_READLINE_HISTORY
+	"  -i, --read-history         read prior interactive session history\n"
+	"  -I, --save-history         save current interactive session history\n"
+#endif
 	"  -v, --verbose              set verbose mode, cumulative\n"
 	"  -h, --help                 display this help and exit\n"
 	"  -V, --version              output version information and exit\n\n"
@@ -371,4 +475,3 @@ void usage(void)
 	printf("\nReport bugs to <hamlib-developer@lists.sourceforge.net>.\n");
 
 }
-
