@@ -1,7 +1,8 @@
 /*
  * rigctl_parse.c - (C) Stephane Fillod 2000-2011
+ *                  (C) Nate Bargmann 2003,2006,2008,2010,2011,2012,2013
  *                  (C) Terry Embry 2008-2009
- *                  (C) The Hamlib Group 2010
+ *                  (C) The Hamlib Group 2002,2006,2007,2008,2009,2010,2011
  *
  * This program tests/controls a radio using Hamlib.
  * It takes commands in interactive mode as well as
@@ -35,7 +36,31 @@
 #include <ctype.h>
 #include <errno.h>
 
-#include <getopt.h>
+#ifdef HAVE_LIBREADLINE
+#  if defined(HAVE_READLINE_READLINE_H)
+#    include <readline/readline.h>
+#  elif defined(HAVE_READLINE_H)    /* !defined(HAVE_READLINE_READLINE_H) */
+#    include <readline.h>
+#  else                             /* !defined(HAVE_READLINE_H) */
+extern char *readline ();
+#  endif                            /* HAVE_READLINE_H */
+#else
+/* no readline */
+#endif                              /* HAVE_LIBREADLINE */
+
+#ifdef HAVE_READLINE_HISTORY
+#  if defined(HAVE_READLINE_HISTORY_H)
+#    include <readline/history.h>
+#  elif defined(HAVE_HISTORY_H)
+#    include <history.h>
+#  else                             /* !defined(HAVE_HISTORY_H) */
+extern void add_history ();
+extern int write_history ();
+extern int read_history ();
+#  endif                            /* defined(HAVE_READLINE_HISTORY_H) */
+                                    /* no history */
+#endif                              /* HAVE_READLINE_HISTORY */
+
 
 #include <hamlib/rig.h>
 #include "misc.h"
@@ -43,6 +68,23 @@
 #include "serial.h"
 #include "sprintflst.h"
 
+/* HAVE_SSLEEP is defined when Windows Sleep is found
+ * HAVE_SLEEP is defined when POSIX sleep is found
+ * _WIN32 is defined when compiling with MinGW
+ *
+ * When cross-compiling from POSIX to Windows using MinGW, HAVE_SLEEP
+ * will often be defined by configure although it is not supported by
+ * MinGW.  So substitute the sleep definition below in such a case and
+ * when compiling on Windows using MinGW where HAVE_SLEEP will be
+ * undefined.
+ *
+ * FIXME:  Needs better handling for all versions of MinGW.
+ *
+ */
+#if (defined(HAVE_SSLEEP) || defined(_WIN32)) && (!defined(HAVE_SLEEP))
+#include "hl_sleep.h"
+#endif
+
 #include "rigctl_parse.h"
 
 /* Hash table implementation See:  http://uthash.sourceforge.net/ */
@@ -54,9 +96,12 @@
 static pthread_mutex_t rig_mutex = PTHREAD_MUTEX_INITIALIZER;
 #endif
 
+#define STR1(S) #S
+#define STR(S) STR1(S)
+
 #define MAXNAMSIZ 32
 #define MAXNBOPT 100	/* max number of different options */
-
+#define MAXARGSZ 127
 
 #define ARG_IN1  0x01
 #define ARG_OUT1 0x02
@@ -72,6 +117,21 @@ static pthread_mutex_t rig_mutex = PTHREAD_MUTEX_INITIALIZER;
 #define ARG_IN  (ARG_IN1|ARG_IN2|ARG_IN3|ARG_IN4)
 #define ARG_OUT  (ARG_OUT1|ARG_OUT2|ARG_OUT3|ARG_OUT4)
 
+/* variables for readline support */
+#ifdef HAVE_LIBREADLINE
+static char *input_line = (char *)NULL;
+static char *result = (char *)NULL;
+static char *parsed_input[sizeof(char) * 5];
+static const int have_rl = 1;
+
+#ifdef HAVE_READLINE_HISTORY
+static char *rp_hist_buf = (char *)NULL;
+#endif
+
+#else                               /* no readline */
+static const int have_rl = 0;
+#endif
+
 struct test_table {
 	unsigned char cmd;
 	const char *name;
@@ -87,7 +147,8 @@ struct test_table {
 
 #define CHKSCN1ARG(a) if ((a) != 1) return -RIG_EINVAL; else do {} while(0)
 
-#define declare_proto_rig(f) static int (f)(RIG *rig, FILE *fout, FILE *fin, int interactive, \
+#define ACTION(f) rigctl_##f
+#define declare_proto_rig(f) static int (ACTION(f))(RIG *rig, FILE *fout, FILE *fin, int interactive, \
 			const struct test_table *cmd, vfo_t vfo, const char *arg1, \
 			const char *arg2, const char *arg3)
 
@@ -157,6 +218,7 @@ declare_proto_rig(send_dtmf);
 declare_proto_rig(recv_dtmf);
 declare_proto_rig(chk_vfo);
 declare_proto_rig(halt);
+declare_proto_rig(pause);
 
 
 /*
@@ -169,71 +231,72 @@ declare_proto_rig(halt);
  *	Available alphabetic letters: -.--------K-----*-----W-Y-
  */
 static struct test_table test_list[] = {
-	{ 'F', "set_freq",          set_freq,       ARG_IN, "Frequency" },
-	{ 'f', "get_freq",          get_freq,       ARG_OUT, "Frequency" },
-	{ 'M', "set_mode",          set_mode,       ARG_IN, "Mode", "Passband" },
-	{ 'm', "get_mode",          get_mode,       ARG_OUT, "Mode", "Passband" },
-	{ 'I', "set_split_freq",    set_split_freq, ARG_IN, "TX Frequency" },
-	{ 'i', "get_split_freq",    get_split_freq, ARG_OUT, "TX Frequency" },
-	{ 'X', "set_split_mode",    set_split_mode, ARG_IN, "TX Mode", "TX Passband" },
-	{ 'x', "get_split_mode",    get_split_mode, ARG_OUT, "TX Mode", "TX Passband" },
-	{ 'S', "set_split_vfo",     set_split_vfo,  ARG_IN, "Split", "TX VFO" },
-	{ 's', "get_split_vfo",     get_split_vfo,  ARG_OUT, "Split", "TX VFO" },
-	{ 'N', "set_ts",            set_ts,         ARG_IN, "Tuning Step" },
-	{ 'n', "get_ts",            get_ts,         ARG_OUT, "Tuning Step" },
-	{ 'L', "set_level",         set_level,      ARG_IN, "Level", "Level Value" },
-	{ 'l', "get_level",         get_level,      ARG_IN1|ARG_OUT2, "Level", "Level Value" },
-	{ 'U', "set_func",          set_func,       ARG_IN, "Func", "Func Status" },
-	{ 'u', "get_func",          get_func,       ARG_IN1|ARG_OUT2, "Func", "Func Status" },
-	{ 'P', "set_parm",          set_parm,       ARG_IN|ARG_NOVFO, "Parm", "Parm Value" },
-	{ 'p', "get_parm",          get_parm,       ARG_IN1|ARG_OUT2|ARG_NOVFO, "Parm", "Parm Value" },
-	{ 'G', "vfo_op",            vfo_op,         ARG_IN, "Mem/VFO Op" },
-	{ 'g', "scan",              scan,           ARG_IN, "Scan Fct", "Scan Channel" },
-	{ 'A', "set_trn",           set_trn,        ARG_IN|ARG_NOVFO, "Transceive" },
-	{ 'a', "get_trn",           get_trn,        ARG_OUT|ARG_NOVFO, "Transceive" },
-	{ 'R', "set_rptr_shift",    set_rptr_shift, ARG_IN, "Rptr Shift" },
-	{ 'r', "get_rptr_shift",    get_rptr_shift, ARG_OUT, "Rptr Shift" },
-	{ 'O', "set_rptr_offs",     set_rptr_offs,  ARG_IN, "Rptr Offset" },
-	{ 'o', "get_rptr_offs",     get_rptr_offs,  ARG_OUT, "Rptr Offset" },
-	{ 'C', "set_ctcss_tone",    set_ctcss_tone, ARG_IN, "CTCSS Tone" },
-	{ 'c', "get_ctcss_tone",    get_ctcss_tone, ARG_OUT, "CTCSS Tone" },
-	{ 'D', "set_dcs_code",      set_dcs_code,   ARG_IN, "DCS Code" },
-	{ 'd', "get_dcs_code",      get_dcs_code,   ARG_OUT, "DCS Code" },
-	{ 0x90, "set_ctcss_sql",    set_ctcss_sql,  ARG_IN, "CTCSS Sql" },
-	{ 0x91, "get_ctcss_sql",    get_ctcss_sql,  ARG_OUT, "CTCSS Sql" },
-	{ 0x92, "set_dcs_sql",      set_dcs_sql,    ARG_IN, "DCS Sql" },
-	{ 0x93, "get_dcs_sql",      get_dcs_sql,    ARG_OUT, "DCS Sql" },
-	{ 'V', "set_vfo",           set_vfo,        ARG_IN|ARG_NOVFO, "VFO" },
-	{ 'v', "get_vfo",           get_vfo,        ARG_OUT, "VFO" },
-	{ 'T', "set_ptt",           set_ptt,        ARG_IN, "PTT" },
-	{ 't', "get_ptt",           get_ptt,        ARG_OUT, "PTT" },
-	{ 'E', "set_mem",           set_mem,        ARG_IN, "Memory#" },
-	{ 'e', "get_mem",           get_mem,        ARG_OUT, "Memory#" },
-	{ 'H', "set_channel",       set_channel,    ARG_IN|ARG_NOVFO, "Channel" },
-	{ 'h', "get_channel",       get_channel,    ARG_IN|ARG_NOVFO, "Channel" },
-	{ 'B', "set_bank",          set_bank,       ARG_IN, "Bank" },
-	{ '_', "get_info",          get_info,       ARG_OUT|ARG_NOVFO, "Info" },
-	{ 'J', "set_rit",           set_rit,        ARG_IN, "RIT" },
-	{ 'j', "get_rit",           get_rit,        ARG_OUT, "RIT" },
-	{ 'Z', "set_xit",           set_xit,        ARG_IN, "XIT" },
-	{ 'z', "get_xit",           get_xit,        ARG_OUT, "XIT" },
-	{ 'Y', "set_ant",           set_ant,        ARG_IN, "Antenna" },
-	{ 'y', "get_ant",           get_ant,        ARG_OUT, "Antenna" },
-	{ 0x87, "set_powerstat",    set_powerstat,  ARG_IN|ARG_NOVFO, "Power Status" },
-	{ 0x88, "get_powerstat",    get_powerstat,  ARG_OUT|ARG_NOVFO, "Power Status" },
-	{ 0x89, "send_dtmf",        send_dtmf,      ARG_IN, "Digits" },
-	{ 0x8a, "recv_dtmf",        recv_dtmf,      ARG_OUT, "Digits" },
-	{ '*', "reset",             reset,          ARG_IN, "Reset" },
-	{ 'w', "send_cmd",          send_cmd,       ARG_IN1|ARG_IN_LINE|ARG_OUT2|ARG_NOVFO, "Cmd", "Reply" },
-	{ 'b', "send_morse",        send_morse,     ARG_IN|ARG_IN_LINE, "Morse" },
-	{ 0x8b, "get_dcd",          get_dcd,        ARG_OUT, "DCD" },
-	{ '2', "power2mW",          power2mW,       ARG_IN1|ARG_IN2|ARG_IN3|ARG_OUT1|ARG_NOVFO, "Power [0.0..1.0]", "Frequency", "Mode", "Power mW" },
-	{ '4', "mW2power",          mW2power,       ARG_IN1|ARG_IN2|ARG_IN3|ARG_OUT1|ARG_NOVFO, "Power mW", "Frequency", "Mode", "Power [0.0..1.0]" },
-	{ '1', "dump_caps",         dump_caps,      ARG_NOVFO },
-	{ '3', "dump_conf",         dump_conf,      ARG_NOVFO },
-	{ 0x8f,"dump_state",        dump_state,     ARG_OUT|ARG_NOVFO },
-	{ 0xf0,"chk_vfo",           chk_vfo,        ARG_NOVFO },	/* rigctld only--check for VFO mode */
-	{ 0xf1,"halt",              halt,           ARG_NOVFO },	/* rigctld only--halt the daemon */
+	{ 'F', "set_freq",          ACTION(set_freq),       ARG_IN, "Frequency" },
+	{ 'f', "get_freq",          ACTION(get_freq),       ARG_OUT, "Frequency" },
+	{ 'M', "set_mode",          ACTION(set_mode),       ARG_IN, "Mode", "Passband" },
+	{ 'm', "get_mode",          ACTION(get_mode),       ARG_OUT, "Mode", "Passband" },
+	{ 'I', "set_split_freq",    ACTION(set_split_freq), ARG_IN, "TX Frequency" },
+	{ 'i', "get_split_freq",    ACTION(get_split_freq), ARG_OUT, "TX Frequency" },
+	{ 'X', "set_split_mode",    ACTION(set_split_mode), ARG_IN, "TX Mode", "TX Passband" },
+	{ 'x', "get_split_mode",    ACTION(get_split_mode), ARG_OUT, "TX Mode", "TX Passband" },
+	{ 'S', "set_split_vfo",     ACTION(set_split_vfo),  ARG_IN, "Split", "TX VFO" },
+	{ 's', "get_split_vfo",     ACTION(get_split_vfo),  ARG_OUT, "Split", "TX VFO" },
+	{ 'N', "set_ts",            ACTION(set_ts),         ARG_IN, "Tuning Step" },
+	{ 'n', "get_ts",            ACTION(get_ts),         ARG_OUT, "Tuning Step" },
+	{ 'L', "set_level",         ACTION(set_level),      ARG_IN, "Level", "Level Value" },
+	{ 'l', "get_level",         ACTION(get_level),      ARG_IN1|ARG_OUT2, "Level", "Level Value" },
+	{ 'U', "set_func",          ACTION(set_func),       ARG_IN, "Func", "Func Status" },
+	{ 'u', "get_func",          ACTION(get_func),       ARG_IN1|ARG_OUT2, "Func", "Func Status" },
+	{ 'P', "set_parm",          ACTION(set_parm),       ARG_IN|ARG_NOVFO, "Parm", "Parm Value" },
+	{ 'p', "get_parm",          ACTION(get_parm),       ARG_IN1|ARG_OUT2|ARG_NOVFO, "Parm", "Parm Value" },
+	{ 'G', "vfo_op",            ACTION(vfo_op),         ARG_IN, "Mem/VFO Op" },
+	{ 'g', "scan",              ACTION(scan),           ARG_IN, "Scan Fct", "Scan Channel" },
+	{ 'A', "set_trn",           ACTION(set_trn),        ARG_IN|ARG_NOVFO, "Transceive" },
+	{ 'a', "get_trn",           ACTION(get_trn),        ARG_OUT|ARG_NOVFO, "Transceive" },
+	{ 'R', "set_rptr_shift",    ACTION(set_rptr_shift), ARG_IN, "Rptr Shift" },
+	{ 'r', "get_rptr_shift",    ACTION(get_rptr_shift), ARG_OUT, "Rptr Shift" },
+	{ 'O', "set_rptr_offs",     ACTION(set_rptr_offs),  ARG_IN, "Rptr Offset" },
+	{ 'o', "get_rptr_offs",     ACTION(get_rptr_offs),  ARG_OUT, "Rptr Offset" },
+	{ 'C', "set_ctcss_tone",    ACTION(set_ctcss_tone), ARG_IN, "CTCSS Tone" },
+	{ 'c', "get_ctcss_tone",    ACTION(get_ctcss_tone), ARG_OUT, "CTCSS Tone" },
+	{ 'D', "set_dcs_code",      ACTION(set_dcs_code),   ARG_IN, "DCS Code" },
+	{ 'd', "get_dcs_code",      ACTION(get_dcs_code),   ARG_OUT, "DCS Code" },
+	{ 0x90, "set_ctcss_sql",    ACTION(set_ctcss_sql),  ARG_IN, "CTCSS Sql" },
+	{ 0x91, "get_ctcss_sql",    ACTION(get_ctcss_sql),  ARG_OUT, "CTCSS Sql" },
+	{ 0x92, "set_dcs_sql",      ACTION(set_dcs_sql),    ARG_IN, "DCS Sql" },
+	{ 0x93, "get_dcs_sql",      ACTION(get_dcs_sql),    ARG_OUT, "DCS Sql" },
+	{ 'V', "set_vfo",           ACTION(set_vfo),        ARG_IN|ARG_NOVFO, "VFO" },
+	{ 'v', "get_vfo",           ACTION(get_vfo),        ARG_OUT, "VFO" },
+	{ 'T', "set_ptt",           ACTION(set_ptt),        ARG_IN, "PTT" },
+	{ 't', "get_ptt",           ACTION(get_ptt),        ARG_OUT, "PTT" },
+	{ 'E', "set_mem",           ACTION(set_mem),        ARG_IN, "Memory#" },
+	{ 'e', "get_mem",           ACTION(get_mem),        ARG_OUT, "Memory#" },
+	{ 'H', "set_channel",       ACTION(set_channel),    ARG_IN|ARG_NOVFO, "Channel" },
+	{ 'h', "get_channel",       ACTION(get_channel),    ARG_IN|ARG_NOVFO, "Channel" },
+	{ 'B', "set_bank",          ACTION(set_bank),       ARG_IN, "Bank" },
+	{ '_', "get_info",          ACTION(get_info),       ARG_OUT|ARG_NOVFO, "Info" },
+	{ 'J', "set_rit",           ACTION(set_rit),        ARG_IN, "RIT" },
+	{ 'j', "get_rit",           ACTION(get_rit),        ARG_OUT, "RIT" },
+	{ 'Z', "set_xit",           ACTION(set_xit),        ARG_IN, "XIT" },
+	{ 'z', "get_xit",           ACTION(get_xit),        ARG_OUT, "XIT" },
+	{ 'Y', "set_ant",           ACTION(set_ant),        ARG_IN, "Antenna" },
+	{ 'y', "get_ant",           ACTION(get_ant),        ARG_OUT, "Antenna" },
+	{ 0x87, "set_powerstat",    ACTION(set_powerstat),  ARG_IN|ARG_NOVFO, "Power Status" },
+	{ 0x88, "get_powerstat",    ACTION(get_powerstat),  ARG_OUT|ARG_NOVFO, "Power Status" },
+	{ 0x89, "send_dtmf",        ACTION(send_dtmf),      ARG_IN, "Digits" },
+	{ 0x8a, "recv_dtmf",        ACTION(recv_dtmf),      ARG_OUT, "Digits" },
+	{ '*', "reset",             ACTION(reset),          ARG_IN, "Reset" },
+	{ 'w', "send_cmd",          ACTION(send_cmd),       ARG_IN1|ARG_IN_LINE|ARG_OUT2|ARG_NOVFO, "Cmd", "Reply" },
+	{ 'b', "send_morse",        ACTION(send_morse),     ARG_IN|ARG_IN_LINE, "Morse" },
+	{ 0x8b, "get_dcd",          ACTION(get_dcd),        ARG_OUT, "DCD" },
+	{ '2', "power2mW",          ACTION(power2mW),       ARG_IN1|ARG_IN2|ARG_IN3|ARG_OUT1|ARG_NOVFO, "Power [0.0..1.0]", "Frequency", "Mode", "Power mW" },
+	{ '4', "mW2power",          ACTION(mW2power),       ARG_IN1|ARG_IN2|ARG_IN3|ARG_OUT1|ARG_NOVFO, "Power mW", "Frequency", "Mode", "Power [0.0..1.0]" },
+	{ '1', "dump_caps",         ACTION(dump_caps),      ARG_NOVFO },
+	{ '3', "dump_conf",         ACTION(dump_conf),      ARG_NOVFO },
+	{ 0x8f,"dump_state",        ACTION(dump_state),     ARG_OUT|ARG_NOVFO },
+	{ 0xf0,"chk_vfo",           ACTION(chk_vfo),        ARG_NOVFO },	/* rigctld only--check for VFO mode */
+	{ 0xf1,"halt",              ACTION(halt),           ARG_NOVFO },	/* rigctld only--halt the daemon */
+	{ 0x8c, "pause",            ACTION(pause),          ARG_IN, "Seconds" },
 	{ 0x00, "", NULL },
 };
 
@@ -307,6 +370,35 @@ void hash_delete_all() {
 }
 
 
+#ifdef HAVE_LIBREADLINE
+/* Frees allocated memory and sets pointers to NULL before calling readline
+ * and then parses the input into space separated tokens.
+ */
+static void rp_getline(const char *s)
+{
+	int i;
+
+	/* free allocated memory and set pointers to NULL */
+	if (input_line) {
+		free(input_line);
+		input_line = (char *)NULL;
+	}
+
+	if (result) {
+		result = (char *)NULL;
+	}
+
+	for (i = 0; i < 5; i++)
+		parsed_input[i] = NULL;
+
+	/* Action!  Returns typed line with newline stripped. */
+	input_line = readline(s);
+}
+
+
+#endif
+
+
 /*
  * TODO: use Lex?
  */
@@ -338,6 +430,73 @@ static int scanfc(FILE *fin, const char *format, void *p)
 	} while(1);
 }
 
+/*
+ * function to get the next word from the command line or from stdin
+ * until stdin exhausted. stdin is read if the special token '-' is
+ * found on the command line.
+ *
+ * returns EOF when words exhausted
+ * returns <0 is error number
+ * returns >=0 when successful
+ */
+static int next_word (char *buffer, int argc, char *argv[], int newline)
+{
+  int ret;
+  char c;
+  static int reading_stdin;
+
+  if (!reading_stdin)
+    {
+      if (optind >= argc) return EOF;
+      else if ('-' == argv[optind][0])
+        {
+          ++optind;
+          reading_stdin = 1;
+        }
+    }
+
+  if (reading_stdin)
+    {
+      do
+        {
+          do ret = scanf (" %c%" STR(MAXARGSZ) "[^ \t\n#]", &c, &buffer[1]); while (EINTR == ret);
+          if (ret > 0 && '#' == c)
+            {
+              do ret = scanf ("%*[^\n]"); while (EINTR == ret); /* consume comments */
+              ret = 0;
+            }
+        }
+      while (!ret);
+      if (EOF == ret) reading_stdin = 0;
+      else if (ret < 0)
+        {
+          rig_debug (RIG_DEBUG_ERR, "scanf: %s\n", strerror (errno));
+          reading_stdin = 0;
+        }
+      else
+        {
+          buffer[0] = c;
+          buffer[1 == ret ? 1 : MAXARGSZ] = '\0';
+          if (newline) putchar ('\n');
+          fputs (buffer, stdout);
+          putchar (' ');
+        }
+    }
+
+  if (!reading_stdin)
+    {
+			if (optind < argc)
+        {
+          strncpy (buffer, argv[optind++], MAXARGSZ);
+          buffer[MAXARGSZ] = '\0';
+          ret = 1;
+        }
+      else ret = EOF;
+    }
+
+  return ret;
+}
+
 #define fprintf_flush(f, a...) \
             ({ int __ret; \
                __ret = fprintf((f), a); \
@@ -345,11 +504,8 @@ static int scanfc(FILE *fin, const char *format, void *p)
                __ret; \
             })
 
-#define MAXARGSZ 127
-
 extern int interactive;
 extern int prompt;
-extern int opt_end;
 extern int vfo_mode;
 extern char send_cmd_term;
 int ext_resp = 0;
@@ -359,194 +515,614 @@ int rigctl_parse(RIG *my_rig, FILE *fin, FILE *fout, char *argv[], int argc)
 {
 	int retcode;		/* generic return code from functions */
 	unsigned char cmd;
-	struct test_table *cmd_entry;
+	struct test_table *cmd_entry = NULL;
 
-	char arg1[MAXARGSZ+1], *p1;
-	char arg2[MAXARGSZ+1], *p2;
-	char arg3[MAXARGSZ+1], *p3;
+	char command[MAXARGSZ+1];
+	char arg1[MAXARGSZ+1], *p1 = NULL;
+	char arg2[MAXARGSZ+1], *p2 = NULL;
+	char arg3[MAXARGSZ+1], *p3 = NULL;
 	static int last_was_ret = 1;
 	vfo_t vfo = RIG_VFO_CURR;
 
-	if (interactive) {
-		if (prompt)
-			fprintf_flush(fout, "\nRig command: ");
+	/* cmd, internal, rigctld */
+	if (!(interactive && prompt && have_rl)) {
+		if (interactive) {
+			if (prompt)
+				fprintf_flush(fout, "\nRig command: ");
 
-		do {
-			if (scanfc(fin, "%c", &cmd) < 1)
-				return -1;
-
-			/* Extended response protocol requested with leading '+' on command
-			 * string--rigctld only!
-			 */
-			if (cmd == '+' && !prompt) {
-				ext_resp = 1;
+			do {
 				if (scanfc(fin, "%c", &cmd) < 1)
 					return -1;
-			} else if (cmd == '+' && prompt) {
-				return 0;
-			}
 
-			if (cmd != '\\' && cmd != '_'  && cmd != '#' && ispunct(cmd) && !prompt) {
-				ext_resp = 1;
-				resp_sep = cmd;
-				if (scanfc(fin, "%c", &cmd) < 1)
-					return -1;
-			} else if (cmd != '\\' && cmd != '?' && cmd != '_' && cmd != '#' && ispunct(cmd) && prompt) {
-				return 0;
-			}
-
-			/* command by name */
-			if (cmd == '\\') {
-				unsigned char cmd_name[MAXNAMSIZ], *pcmd = cmd_name;
-				int c_len = MAXNAMSIZ;
-
-				if (scanfc(fin, "%c", pcmd) < 1)
-					return -1;
-
-				while(c_len-- && (isalnum(*pcmd) || *pcmd == '_' ))
-					if (scanfc(fin, "%c", ++pcmd) < 1)
+				/* Extended response protocol requested with leading '+' on command
+				 * string--rigctld only!
+				 */
+				if (cmd == '+' && !prompt) {
+					ext_resp = 1;
+					if (scanfc(fin, "%c", &cmd) < 1)
 						return -1;
-
-				*pcmd = '\0';
-				cmd = parse_arg((char *)cmd_name);
-				break;
-			}
-
-			if (cmd == 0x0a || cmd == 0x0d) {
-				if (last_was_ret) {
-					if (prompt) {
-						fprintf(fout, "? for help, q to quit.\n");
-						fprintf_flush(fout, "\nRig command: ");
-					}
+				} else if (cmd == '+' && prompt) {
 					return 0;
 				}
-				last_was_ret = 1;
+
+				if (cmd != '\\' && cmd != '_'  && cmd != '#' && ispunct(cmd) && !prompt) {
+					ext_resp = 1;
+					resp_sep = cmd;
+					if (scanfc(fin, "%c", &cmd) < 1)
+						return -1;
+				} else if (cmd != '\\' && cmd != '?' && cmd != '_' && cmd != '#' && ispunct(cmd) && prompt) {
+					return 0;
+				}
+
+				/* command by name */
+				if (cmd == '\\') {
+					unsigned char cmd_name[MAXNAMSIZ], *pcmd = cmd_name;
+					int c_len = MAXNAMSIZ;
+
+					if (scanfc(fin, "%c", pcmd) < 1)
+						return -1;
+
+					while(c_len-- && (isalnum(*pcmd) || *pcmd == '_' ))
+						if (scanfc(fin, "%c", ++pcmd) < 1)
+							return -1;
+
+					*pcmd = '\0';
+					cmd = parse_arg((char *)cmd_name);
+					break;
+				}
+
+				if (cmd == 0x0a || cmd == 0x0d) {
+					if (last_was_ret) {
+						if (prompt) {
+							fprintf(fout, "? for help, q to quit.\n");
+							fprintf_flush(fout, "\nRig command: ");
+						}
+						return 0;
+					}
+					last_was_ret = 1;
+				}
+			} while (cmd == 0x0a || cmd == 0x0d);
+
+			last_was_ret = 0;
+
+			/* comment line */
+			if (cmd == '#') {
+				while( cmd != '\n' && cmd != '\r')
+					if (scanfc(fin, "%c", &cmd) < 1)
+						return -1;
+				return 0;
 			}
-		} while (cmd == 0x0a || cmd == 0x0d);
+			if (cmd == 'Q' || cmd == 'q')
+					return 1;
+			if (cmd == '?') {
+				usage_rig(fout);
+				fflush(fout);
+				return 0;
+			}
+		} else {
+			/* parse rest of command line */
+      retcode = next_word (command, argc, argv, 1);
+      if (EOF == retcode) return 1;
+      else if (retcode < 0) return retcode;
+      else if ('\0' == command[1]) {
+        cmd = command[0];
+      } else {
+        cmd = parse_arg (command);
+      }
+		}
 
-		last_was_ret = 0;
-
-		/* comment line */
-		if (cmd == '#') {
-			while( cmd != '\n' && cmd != '\r')
-				if (scanfc(fin, "%c", &cmd) < 1)
-					return -1;
+		cmd_entry = find_cmd_entry(cmd);
+		if (!cmd_entry) {
+			fprintf(stderr, "Command '%c' not found!\n", cmd);
 			return 0;
 		}
-		if (cmd == 'Q' || cmd == 'q')
-				return 1;
-		if (cmd == '?') {
+
+		if (!(cmd_entry->flags & ARG_NOVFO) && vfo_mode) {
+			if (interactive) {
+				if (prompt)
+					fprintf_flush(fout, "VFO: ");
+				if (scanfc(fin, "%s", arg1) < 1)
+					return -1;
+				vfo = rig_parse_vfo(arg1);
+			} else {
+        retcode = next_word (arg1, argc, argv, 0);
+        if (EOF == retcode) {
+					fprintf(stderr, "Invalid arg for command '%s'\n",
+                  cmd_entry->name);
+        }
+        else if (retcode < 0) return retcode;
+				vfo = rig_parse_vfo(arg1);
+			}
+		}
+
+		if ((cmd_entry->flags & ARG_IN_LINE) &&
+				(cmd_entry->flags & ARG_IN1) && cmd_entry->arg1) {
+			if (interactive) {
+				char *nl;
+				if (prompt)
+					fprintf_flush(fout, "%s: ", cmd_entry->arg1);
+				if (fgets(arg1, MAXARGSZ, fin) == NULL)
+					return -1;
+				if (arg1[0] == 0xa)
+					if (fgets(arg1, MAXARGSZ, fin) == NULL)
+						return -1;
+				nl = strchr(arg1, 0xa);
+				if (nl) *nl = '\0';	/* chomp */
+				p1 = arg1[0] == ' ' ? arg1 + 1 : arg1;
+			} else {
+        retcode = next_word (arg1, argc, argv, 0);
+        if (EOF == retcode) {
+					fprintf(stderr, "Invalid arg for command '%s'\n",
+                  cmd_entry->name);
+          return 1;
+        }
+        else if (retcode < 0) return retcode;
+				p1 = arg1;
+			}
+		} else
+		  if ((cmd_entry->flags & ARG_IN1) && cmd_entry->arg1) {
+			if (interactive) {
+				if (prompt)
+					fprintf_flush(fout, "%s: ", cmd_entry->arg1);
+				if (scanfc(fin, "%s", arg1) < 1)
+					return -1;
+				p1 = arg1;
+			} else {
+        retcode = next_word (arg1, argc, argv, 0);
+        if (EOF == retcode) {
+					fprintf(stderr, "Invalid arg for command '%s'\n",
+                  cmd_entry->name);
+          return 1;
+        }
+        else if (retcode < 0) return retcode;
+				p1 = arg1;
+			}
+		}
+		if (p1 && p1[0] != '?' && (cmd_entry->flags & ARG_IN2) && cmd_entry->arg2) {
+			if (interactive) {
+				if (prompt)
+					fprintf_flush(fout, "%s: ", cmd_entry->arg2);
+				if (scanfc(fin, "%s", arg2) < 1)
+					return -1;
+				p2 = arg2;
+			} else {
+        retcode = next_word (arg2, argc, argv, 0);
+        if (EOF == retcode) {
+					fprintf(stderr, "Invalid arg for command '%s'\n",
+                  cmd_entry->name);
+          return 1;
+        }
+        else if (retcode < 0) return retcode;
+				p2 = arg2;
+			}
+		}
+		if (p1 && p1[0] != '?' && (cmd_entry->flags & ARG_IN3) && cmd_entry->arg3) {
+			if (interactive) {
+				if (prompt)
+					fprintf_flush(fout, "%s: ", cmd_entry->arg3);
+				if (scanfc(fin, "%s", arg3) < 1)
+					return -1;
+				p3 = arg3;
+			} else {
+        retcode = next_word (arg3, argc, argv, 0);
+        if (EOF == retcode) {
+					fprintf(stderr, "Invalid arg for command '%s'\n",
+                  cmd_entry->name);
+          return 1;
+        }
+        else if (retcode < 0) return retcode;
+				p3 = arg3;
+			}
+		}
+	}
+
+#ifdef HAVE_LIBREADLINE
+
+	if (interactive && prompt && have_rl) {
+		int j, x;
+
+#ifdef HAVE_READLINE_HISTORY
+		/* Minimum space for 32+1+32+1+128+1+128+1+128+1 = 453 chars, so
+		 * allocate 512 chars cleared to zero for safety.
+		 */
+		rp_hist_buf = (char *)calloc(512, sizeof(char));
+#endif
+
+		rl_instream = fin;
+		rl_outstream = fout;
+
+		rp_getline("\nRig command: ");
+
+		/* EOF (Ctl-D) received on empty input line, bail out gracefully. */
+		if (!input_line) {
+			fprintf_flush(fout, "\n");
+			return 1;
+		}
+
+		/* Q or q to quit */
+		if (!(strncasecmp(input_line, "q", 1)))
+			return 1;
+
+		/* '?' for help */
+		if (!(strncmp(input_line, "?", 1))) {
 			usage_rig(fout);
 			fflush(fout);
 			return 0;
 		}
-	} else {
-		/* parse rest of command line */
-		if (optind >= argc)
+
+		/* '#' for comment */
+		if (!(strncmp(input_line, "#", 1)))
+			return 0;
+
+		/* Blank line entered */
+		if (!(strcmp(input_line, ""))) {
+			fprintf(fout, "? for help, q to quit.\n");
+			fflush(fout);
+			return 0;
+		}
+
+		rig_debug(RIG_DEBUG_BUG, "%s: input_line: %s\n", __func__, input_line);
+
+		/* Split input_line on any number of spaces to get the command token
+		 * Tabs are intercepted by readline for completion and a newline
+		 * causes readline to return the typed text.  If more than one
+		 * argument is given, it will be parsed out later.
+		 */
+		result = strtok(input_line, " ");
+
+		/* parsed_input stores pointers into input_line where the token strings
+		 * start.
+		 */
+		if (result) {
+			parsed_input[0] = result;
+		} else {
+			/* Oops!  Invoke GDB!! */
+			fprintf_flush(fout, "\n");
 			return 1;
-		if (argv[optind][1] == '\0')
-			cmd = argv[optind][0];
-		else
-			cmd = parse_arg(argv[optind]);
-		optind++;
-	}
-
-	cmd_entry = find_cmd_entry(cmd);
-	if (!cmd_entry) {
-		fprintf(stderr, "Command '%c' not found!\n", cmd);
-		return 0;
-	}
-
-	p1 = p2 = p3 = NULL;
-	if (!(cmd_entry->flags & ARG_NOVFO) && vfo_mode) {
-		if (interactive) {
-			if (prompt)
-				fprintf_flush(fout, "VFO: ");
-			if (scanfc(fin, "%s", arg1) < 1)
-				return -1;
-			vfo = rig_parse_vfo(arg1);
-		} else {
-			if (!argv[optind]) {
-				fprintf(stderr, "Invalid arg for command '%s'\n",
-							cmd_entry->name);
-				exit(1);
-			}
-			vfo = rig_parse_vfo(argv[optind++]);
 		}
-	}
 
-	if ((cmd_entry->flags & ARG_IN_LINE) &&
-			(cmd_entry->flags & ARG_IN1) && cmd_entry->arg1) {
-		if (interactive) {
-			char *nl;
-			if (prompt)
-				fprintf_flush(fout, "%s: ", cmd_entry->arg1);
-			if (fgets(arg1, MAXARGSZ, fin) == NULL)
-                return -1;
-			if (arg1[0] == 0xa)
-				if (fgets(arg1, MAXARGSZ, fin) == NULL)
-                    return -1;
-			nl = strchr(arg1, 0xa);
-			if (nl) *nl = '\0';	/* chomp */
-			p1 = arg1[0] == ' ' ? arg1 + 1 : arg1;
-		} else {
-			if (!argv[optind]) {
-				fprintf(stderr, "Invalid arg for command '%s'\n",
-							cmd_entry->name);
-				exit(1);
-			}
-			p1 = argv[optind++];
+		/* At this point parsed_input contains the typed text of the command
+		 * with surrounding space characters removed.  If Readline History is
+		 * available, copy the command string into a history buffer.
+		 */
+
+		/* Single character command */
+		if ((strlen(parsed_input[0]) == 1) && (*parsed_input[0] != '\\')) {
+			cmd = *parsed_input[0];
+
+#ifdef HAVE_READLINE_HISTORY
+			/* Store what is typed, not validated, for history. */
+			if (rp_hist_buf)
+				strncpy(rp_hist_buf, parsed_input[0], 1);
+#endif
 		}
-	} else
-	  if ((cmd_entry->flags & ARG_IN1) && cmd_entry->arg1) {
-		if (interactive) {
-			if (prompt)
-				fprintf_flush(fout, "%s: ", cmd_entry->arg1);
-			if (scanfc(fin, "%s", arg1) < 1)
-				return -1;
+		/* Test the command token, parsed_input[0] */
+		else if ((*parsed_input[0] == '\\') && (strlen(parsed_input[0]) > 1)) {
+			char cmd_name[MAXNAMSIZ];
+
+			/* if there is no terminating '\0' character in the source string,
+			 * srncpy() doesn't add one even if the supplied length is less
+			 * than the destination array.  Truncate the source string here.
+			 */
+			 if (strlen(parsed_input[0] + 1) >= MAXNAMSIZ)
+				*(parsed_input[0] + MAXNAMSIZ) = '\0';
+
+#ifdef HAVE_READLINE_HISTORY
+			if (rp_hist_buf)
+				strncpy(rp_hist_buf, parsed_input[0], MAXNAMSIZ);
+#endif
+			/* The starting position of the source string is the first
+			 * character past the initial '\'.  Using MAXNAMSIZ for the
+			 * length leaves enough space for the '\0' string terminator in the
+			 * cmd_name array.
+			 */
+			strncpy(cmd_name, parsed_input[0] + 1, MAXNAMSIZ);
+
+			/* Sanity check as valid multiple character commands consist of
+			 * alpha-numeric characters and the underscore ('_') character.
+			 */
+			for (j = 0; cmd_name[j] != '\0'; j++) {
+				if (!(isalnum((int)cmd_name[j]) || cmd_name[j] == '_')) {
+					fprintf(stderr, "Valid multiple character command names contain alpha-numeric characters plus '_'\n");
+					return 0;
+				}
+			}
+
+			cmd = parse_arg(cmd_name);
+		}
+		/* Single '\' entered, prompt again */
+		 else if ((*parsed_input[0] == '\\') && (strlen(parsed_input[0]) == 1)) {
+			return 0;
+		}
+		/* Multiple characters but no leading '\' */
+		else {
+			fprintf(stderr, "Precede multiple character command names with '\\'\n");
+			return 0;
+		}
+
+		cmd_entry = find_cmd_entry(cmd);
+		if (!cmd_entry) {
+			if (cmd == '\0')
+				fprintf(stderr, "Command '%s' not found!\n", parsed_input[0]);
+			else
+				fprintf(stderr, "Command '%c' not found!\n", cmd);
+
+			return 0;
+		}
+
+		/* If vfo_mode is enabled (-o|--vfo) check if already given
+		 * or prompt for it.
+		 */
+		if (!(cmd_entry->flags & ARG_NOVFO) && vfo_mode) {
+			/* Check if VFO was given with command. */
+			result = strtok(NULL, " ");
+
+			if (result) {
+				x = 1;
+				parsed_input[x] = result;
+			}
+			/* Need to prompt if a VFO string was not given. */
+			else {
+				x = 0;
+				rp_getline("VFO: ");
+
+				if (!input_line) {
+					fprintf_flush(fout, "\n");
+					return 1;
+				}
+
+				/* Blank line entered */
+				if (!(strcmp(input_line, ""))) {
+					fprintf(fout, "? for help, q to quit.\n");
+					fflush(fout);
+					return 0;
+				}
+
+				/* Get the first token of input, the rest, if any, will be
+				 * used later.
+				 */
+				result = strtok(input_line, " ");
+
+				if (result) {
+					parsed_input[x] = result;
+				} else {
+					fprintf_flush(fout, "\n");
+					return 1;
+				}
+			}
+
+			/* VFO name tokens are presently quite short.  Truncate excessively
+			 * long strings.
+			 */
+			if (strlen(parsed_input[x]) >= MAXNAMSIZ)
+				*(parsed_input[x] + (MAXNAMSIZ - 1)) = '\0';
+
+#ifdef HAVE_READLINE_HISTORY
+			if (rp_hist_buf) {
+				strncat(rp_hist_buf, " ", 1);
+				strncat(rp_hist_buf, parsed_input[x], MAXNAMSIZ);
+			}
+#endif
+			/* Sanity check, VFO names are alpha only. */
+			for (j = 0; j < MAXNAMSIZ && parsed_input[x][j] != '\0'; j++) {
+				if (!(isalpha((int)parsed_input[x][j]))) {
+					parsed_input[x][j] = '\0';
+
+					break;
+				}
+			}
+			vfo = rig_parse_vfo(parsed_input[x]);
+
+			if (vfo == RIG_VFO_NONE) {
+				fprintf(stderr, "Warning:  VFO '%s' unrecognized, using 'currVFO' instead.\n",
+					parsed_input[x]);
+				vfo = RIG_VFO_CURR;
+			}
+		}
+
+		/* \send_cmd, \send_morse */
+		if ((cmd_entry->flags & ARG_IN_LINE) &&
+				(cmd_entry->flags & ARG_IN1) && cmd_entry->arg1) {
+			/* Check for a non-existent delimiter so as to not break up
+			 * remaining line into separate tokens (spaces OK).
+			 */
+			result = strtok(NULL, "\0");
+
+			if (vfo_mode && result) {
+				x = 2;
+				parsed_input[x] = result;
+			} else if (result) {
+				x = 1;
+				parsed_input[x] = result;
+			} else {
+				x = 0;
+				char pmptstr[(strlen(cmd_entry->arg1) + 3)];
+
+				strcpy(pmptstr, cmd_entry->arg1);
+				strcat(pmptstr, ": ");
+
+				rp_getline(pmptstr);
+
+				/* Blank line entered */
+				if (!(strcmp(input_line, ""))) {
+					fprintf(fout, "? for help, q to quit.\n");
+					fflush(fout);
+					return 0;
+				}
+
+				if (input_line)
+					parsed_input[x] = input_line;
+				else {
+					fprintf_flush(fout, "\n");
+					return 1;
+				}
+			}
+
+			/* The arg1 array size is MAXARGSZ + 1 so truncate it to fit if larger. */
+			if (strlen(parsed_input[x]) > MAXARGSZ)
+				parsed_input[x][MAXARGSZ] = '\0';
+
+#ifdef HAVE_READLINE_HISTORY
+			if (rp_hist_buf) {
+				strncat(rp_hist_buf, " ", 1);
+				strncat(rp_hist_buf, parsed_input[x], MAXARGSZ);
+			}
+#endif
+			strcpy(arg1, parsed_input[x]);
 			p1 = arg1;
-		} else {
-			if (!argv[optind]) {
-				fprintf(stderr, "Invalid arg for command '%s'\n",
-							cmd_entry->name);
-				exit(1);
-			}
-			p1 = argv[optind++];
 		}
-	}
-	if (p1 && p1[0] != '?' && (cmd_entry->flags & ARG_IN2) && cmd_entry->arg2) {
-		if (interactive) {
-			if (prompt)
-				fprintf_flush(fout, "%s: ", cmd_entry->arg2);
-			if (scanfc(fin, "%s", arg2) < 1)
-				return -1;
+
+		/* Normal argument parsing. */
+		else if ((cmd_entry->flags & ARG_IN1) && cmd_entry->arg1) {
+			result = strtok(NULL, " ");
+
+			if (vfo_mode && result) {
+				x = 2;
+				parsed_input[x] = result;
+			} else if (result) {
+				x = 1;
+				parsed_input[x] = result;
+			} else {
+				x = 0;
+				char pmptstr[(strlen(cmd_entry->arg1) + 3)];
+
+				strcpy(pmptstr, cmd_entry->arg1);
+				strcat(pmptstr, ": ");
+
+				rp_getline(pmptstr);
+
+				if (!(strcmp(input_line, ""))) {
+					fprintf(fout, "? for help, q to quit.\n");
+					fflush(fout);
+					return 0;
+				}
+
+				result = strtok(input_line, " ");
+
+				if (result) {
+					parsed_input[x] = result;
+				} else {
+					fprintf_flush(fout, "\n");
+					return 1;
+				}
+			}
+
+			if (strlen(parsed_input[x]) > MAXARGSZ)
+				parsed_input[x][MAXARGSZ] = '\0';
+
+#ifdef HAVE_READLINE_HISTORY
+			if (rp_hist_buf) {
+				strncat(rp_hist_buf, " ", 1);
+				strncat(rp_hist_buf, parsed_input[x], MAXARGSZ);
+			}
+#endif
+			strcpy(arg1, parsed_input[x]);
+			p1 = arg1;
+		}
+		if (p1 && p1[0] != '?' && (cmd_entry->flags & ARG_IN2) && cmd_entry->arg2) {
+			result = strtok(NULL, " ");
+
+			if (vfo_mode && result) {
+				x = 3;
+				parsed_input[x] = result;
+			} else if (result) {
+				x = 2;
+				parsed_input[x] = result;
+			} else {
+				x = 0;
+				char pmptstr[(strlen(cmd_entry->arg2) + 3)];
+
+				strcpy(pmptstr, cmd_entry->arg2);
+				strcat(pmptstr, ": ");
+
+				rp_getline(pmptstr);
+
+				if (!(strcmp(input_line, ""))) {
+					fprintf(fout, "? for help, q to quit.\n");
+					fflush(fout);
+					return 0;
+				}
+
+				result = strtok(input_line, " ");
+
+				if (result) {
+					parsed_input[x] = result;
+				} else {
+					fprintf_flush(fout, "\n");
+					return 1;
+				}
+			}
+
+			if (strlen(parsed_input[x]) > MAXARGSZ)
+				parsed_input[x][MAXARGSZ] = '\0';
+
+#ifdef HAVE_READLINE_HISTORY
+			if (rp_hist_buf) {
+				strncat(rp_hist_buf, " ", 1);
+				strncat(rp_hist_buf, parsed_input[x], MAXARGSZ);
+			}
+#endif
+			strcpy(arg2, parsed_input[x]);
 			p2 = arg2;
-		} else {
-			if (!argv[optind]) {
-				fprintf(stderr, "Invalid arg for command '%s'\n",
-							cmd_entry->name);
-				exit(1);
-			}
-			p2 = argv[optind++];
 		}
-	}
-	if (p1 && p1[0] != '?' && (cmd_entry->flags & ARG_IN3) && cmd_entry->arg3) {
-		if (interactive) {
-			if (prompt)
-				fprintf_flush(fout, "%s: ", cmd_entry->arg3);
-			if (scanfc(fin, "%s", arg3) < 1)
-				return -1;
+		if (p1 && p1[0] != '?' && (cmd_entry->flags & ARG_IN3) && cmd_entry->arg3) {
+			result = strtok(NULL, " ");
+
+			if (vfo_mode && result) {
+				x = 4;
+				parsed_input[x] = result;
+			} else if (result) {
+				x = 3;
+				parsed_input[x] = result;
+			} else {
+				x = 0;
+				char pmptstr[(strlen(cmd_entry->arg3) + 3)];
+
+				strcpy(pmptstr, cmd_entry->arg3);
+				strcat(pmptstr, ": ");
+
+				rp_getline(pmptstr);
+
+				if (!(strcmp(input_line, ""))) {
+					fprintf(fout, "? for help, q to quit.\n");
+					fflush(fout);
+					return 0;
+				}
+
+				result = strtok(input_line, " ");
+
+				if (result) {
+					parsed_input[x] = result;
+				} else {
+					fprintf_flush(fout, "\n");
+					return 1;
+				}
+			}
+
+			if (strlen(parsed_input[x]) > MAXARGSZ)
+				parsed_input[x][MAXARGSZ] = '\0';
+
+#ifdef HAVE_READLINE_HISTORY
+			if (rp_hist_buf) {
+				strncat(rp_hist_buf, " ", 1);
+				strncat(rp_hist_buf, parsed_input[x], MAXARGSZ);
+			}
+#endif
+			strcpy(arg3, parsed_input[x]);
 			p3 = arg3;
-		} else {
-			if (!argv[optind]) {
-				fprintf(stderr, "Invalid arg for command '%s'\n",
-							cmd_entry->name);
-				exit(1);
-			}
-			p3 = argv[optind++];
 		}
+#ifdef HAVE_READLINE_HISTORY
+		if (rp_hist_buf) {
+			add_history(rp_hist_buf);
+			free(rp_hist_buf);
+			rp_hist_buf = (char *)NULL;
+		}
+#endif
 	}
 
+#endif	/* HAVE_LIBREADLINE */
+
+
 	/*
 	 * mutex locking needed because rigctld is multithreaded
 	 * and hamlib is not MT-safe
@@ -599,7 +1175,7 @@ int rigctl_parse(RIG *my_rig, FILE *fin, FILE *fout, char *argv[], int argc)
 		if (interactive && !prompt) {
 			/* netrigctl RIG_OK */
 			if (!(cmd_entry->flags & ARG_OUT)
-				&& !opt_end && !ext_resp && cmd != 0xf0)
+				&& !ext_resp && cmd != 0xf0)
 				fprintf(fout, NETRIGCTL_RET "0\n");
 
 			/* Extended Response protocol */
@@ -608,10 +1184,6 @@ int rigctl_parse(RIG *my_rig, FILE *fin, FILE *fout, char *argv[], int argc)
 				ext_resp = 0;
 				resp_sep = '\n';
 			}
-
-			/* Nate's protocol (obsolete) */
-			else if ((cmd_entry->flags & ARG_OUT) && opt_end)
-				fprintf(fout, "END\n");
 		}
 	}
 
@@ -649,6 +1221,12 @@ void usage_rig(FILE *fout)
 		else
 			fprintf(fout, ")%*s", nbspaces, " ");
 	}
+
+	fprintf(fout, "\n\nIn interactive mode prefix long command names with '\\', e.g. '\\dump_state'\n\n"
+          "The special command '-' is used to read further commands from standard input\n"
+          "Commands and arguments read from standard input must be white space separated,\n"
+          "comments are allowed, comments start with the # character and continue to the\n"
+          "end of the line.\n");
 }
 
 
@@ -876,6 +1454,13 @@ declare_proto_rig(get_mode)
 /* 'V' */
 declare_proto_rig(set_vfo)
 {
+	if (!strcmp(arg1, "?")) {
+		char s[SPRINTF_MAX_SIZE];
+		sprintf_vfo(s, rig->state.vfo_list);
+		fprintf(fout, "%s\n", s);
+		return RIG_OK;
+	}
+
 	return rig_set_vfo(rig, rig_parse_vfo(arg1));
 }
 
@@ -1161,6 +1746,14 @@ declare_proto_rig(set_split_vfo)
 	vfo_t tx_vfo;
 
 	CHKSCN1ARG(sscanf(arg1, "%d", &split));
+
+	if (!strcmp(arg2, "?")) {
+		char s[SPRINTF_MAX_SIZE];
+		sprintf_vfo(s, rig->state.vfo_list);
+		fprintf(fout, "%s\n", s);
+		return RIG_OK;
+	}
+
 	tx_vfo = rig_parse_vfo(arg2);
 	if (tx_vfo == RIG_VFO_NONE)
 		return -RIG_EINVAL;
@@ -1630,7 +2223,7 @@ declare_proto_rig(set_channel)
 
 	memset(&chan, 0, sizeof(channel_t));
 
-	if (isdigit(arg1[0])) {
+	if (isdigit((int)arg1[0])) {
 		chan.vfo = RIG_VFO_MEM;
 		CHKSCN1ARG(sscanf(arg1, "%d", &chan.channel_num));
 		/*
@@ -1804,7 +2397,7 @@ declare_proto_rig(get_channel)
 
 	memset(&chan, 0, sizeof(channel_t));
 
-	if (isdigit(arg1[0])) {
+	if (isdigit((int)arg1[0])) {
 		chan.vfo = RIG_VFO_MEM;
 		CHKSCN1ARG(sscanf(arg1, "%d", &chan.channel_num));
 	} else {
@@ -2302,3 +2895,11 @@ declare_proto_rig(halt)
 	return RIG_OK;
 }
 
+/* '0x8c'--pause processing */
+declare_proto_rig(pause)
+{
+  unsigned seconds;
+	CHKSCN1ARG(sscanf(arg1, "%u", &seconds));
+  sleep (seconds);
+	return RIG_OK;
+}
