@@ -1,5 +1,7 @@
 /*
  * rigctld.c - (C) Stephane Fillod 2000-2011
+ *             (C) Nate Bargmann 2008,2010,2011,2012,2013
+ *             (C) The Hamlib Group 2012
  *
  * This program test/control a radio using Hamlib.
  * It takes commands from network connection.
@@ -47,6 +49,9 @@
 #elif HAVE_WS2TCPIP_H
 #include <ws2tcpip.h>
 #include <fcntl.h>
+#	if defined(HAVE_WSPIAPI_H)
+#		include <wspiapi.h>
+#	endif
 #endif
 #ifdef HAVE_NETDB_H
 #include <netdb.h>
@@ -70,7 +75,7 @@
  * NB: do NOT use -W since it's reserved by POSIX.
  * TODO: add an option to read from a file
  */
-#define SHORT_OPTIONS "m:r:p:d:P:D:s:c:T:t:C:lLuoevhV"
+#define SHORT_OPTIONS "m:r:p:d:P:D:s:c:T:t:C:lLuovhV"
 static struct option long_options[] =
 {
 	{"model",       1, 0, 'm'},
@@ -88,7 +93,6 @@ static struct option long_options[] =
 	{"show-conf",   0, 0, 'L'},
 	{"dump-caps",   0, 0, 'u'},
 	{"vfo",         0, 0, 'o'},
-	{"end-marker",  0, 0, 'e'},
 	{"verbose",     0, 0, 'v'},
 	{"help",        0, 0, 'h'},
 	{"version",     0, 0, 'V'},
@@ -107,7 +111,6 @@ void usage(void);
 
 int interactive = 1;    /* no cmd because of daemon */
 int prompt = 0;         /* Daemon mode for rigparse return string */
-int opt_end = 0;        /* END marker for rigctld */
 int vfo_mode = 0;       /* vfo_mode=0 means target VFO is current VFO */
 
 char send_cmd_term = '\r';  /* send_cmd termination char */
@@ -274,10 +277,6 @@ int main (int argc, char *argv[])
 			case 'u':
 				dump_caps_opt++;
 				break;
-			case 'e':
-				opt_end++;
-				fprintf(stderr, "-e|--end-marker option is deprecated!\nPlease consider using the Extended Response protocol instead.\n");
-				break;
 			default:
 				usage();	/* unknown option? */
 				exit(1);
@@ -557,7 +556,6 @@ void usage(void)
 	"  -l, --list                 list all model numbers and exit\n"
 	"  -u, --dump-caps            dump capabilities and exit\n"
 	"  -o, --vfo                  do not default to VFO_CURR, require extra vfo arg\n"
-	"  -e, --end-marker           use END marker in rigctld protocol (obsolete)\n"
 	"  -v, --verbose              set verbose mode, cumulative\n"
 	"  -h, --help                 display this help and exit\n"
 	"  -V, --version              output version information and exit\n\n",
@@ -568,5 +566,3 @@ void usage(void)
 	printf("\nReport bugs to <hamlib-developer@lists.sourceforge.net>.\n");
 
 }
-
-
