@@ -1,5 +1,7 @@
 /*
  * rotctl_parse.c - (C) Stephane Fillod 2000-2010
+ *                  (C) Nate Bargmann 2003,2007,2010,2011,2012,2013
+ *                  (C) The Hamlib Group 2002,2006,2011
  *
  * This program test/control a rotator using Hamlib.
  * It takes commands in interactive mode as well as
@@ -31,13 +33,55 @@
 #include <string.h>
 #include <unistd.h>
 #include <ctype.h>
-
 #include <errno.h>
 
+#ifdef HAVE_LIBREADLINE
+#  if defined(HAVE_READLINE_READLINE_H)
+#    include <readline/readline.h>
+#  elif defined(HAVE_READLINE_H)    /* !defined(HAVE_READLINE_READLINE_H) */
+#    include <readline.h>
+#  else                             /* !defined(HAVE_READLINE_H) */
+extern char *readline ();
+#  endif                            /* HAVE_READLINE_H */
+#else
+/* no readline */
+#endif                              /* HAVE_LIBREADLINE */
+
+#ifdef HAVE_READLINE_HISTORY
+#  if defined(HAVE_READLINE_HISTORY_H)
+#    include <readline/history.h>
+#  elif defined(HAVE_HISTORY_H)
+#    include <history.h>
+#  else                             /* !defined(HAVE_HISTORY_H) */
+extern void add_history ();
+extern int write_history ();
+extern int read_history ();
+#  endif                            /* defined(HAVE_READLINE_HISTORY_H) */
+                                    /* no history */
+#endif                              /* HAVE_READLINE_HISTORY */
+
+
 #include <hamlib/rotator.h>
 #include "serial.h"
 #include "misc.h"
 
+/* HAVE_SSLEEP is defined when Windows Sleep is found
+ * HAVE_SLEEP is defined when POSIX sleep is found
+ * _WIN32 is defined when compiling with MinGW
+ *
+ * When cross-compiling from POSIX to Windows using MinGW, HAVE_SLEEP
+ * will often be defined by configure although it is not supported by
+ * MinGW.  So substitute the sleep definition below in such a case and
+ * when compiling on Windows using MinGW where HAVE_SLEEP will be
+ * undefined.
+ *
+ * FIXME:  Needs better handling for all versions of MinGW.
+ *
+ */
+#if (defined(HAVE_SSLEEP) || defined(_WIN32)) && (!defined(HAVE_SLEEP))
+#include "hl_sleep.h"
+#endif
+
 #include "rotctl_parse.h"
 
 /* Hash table implementation See:  http://uthash.sourceforge.net/ */
@@ -49,10 +93,12 @@
 static pthread_mutex_t rot_mutex = PTHREAD_MUTEX_INITIALIZER;
 #endif
 
+#define STR1(S) #S
+#define STR(S) STR1(S)
 
 #define MAXNAMSIZ 32
 #define MAXNBOPT 100	/* max number of different options */
-
+#define MAXARGSZ 127
 
 #define ARG_IN1  0x01
 #define ARG_OUT1 0x02
@@ -68,6 +114,21 @@ static pthread_mutex_t rot_mutex = PTHREAD_MUTEX_INITIALIZER;
 #define ARG_IN  (ARG_IN1|ARG_IN2|ARG_IN3|ARG_IN4)
 #define ARG_OUT  (ARG_OUT1|ARG_OUT2|ARG_OUT3|ARG_OUT4)
 
+/* variables for readline support */
+#ifdef HAVE_LIBREADLINE
+static char *input_line = (char *)NULL;
+static char *result = (char *)NULL;
+static char *parsed_input[sizeof(char) * 7];
+static const int have_rl = 1;
+
+#ifdef HAVE_READLINE_HISTORY
+static char *rp_hist_buf = (char *)NULL;
+#endif
+
+#else                               /* no readline */
+static const int have_rl = 0;
+#endif
+
 struct test_table {
 	unsigned char cmd;
 	const char *name;
@@ -84,7 +145,8 @@ struct test_table {
 
 #define CHKSCN1ARG(a) if ((a) != 1) return -RIG_EINVAL; else do {} while(0)
 
-#define declare_proto_rot(f) static int (f)(ROT *rot, FILE *fout, int interactive, \
+#define ACTION(f) rigctl_##f
+#define declare_proto_rot(f) static int (ACTION(f))(ROT *rot, FILE *fout, int interactive, \
 			const struct test_table *cmd, const char *arg1, const char *arg2, \
 			const char *arg3, const char *arg4, const char *arg5, const char *arg6)
 
@@ -109,6 +171,7 @@ declare_proto_rot(dec2d_mm);
 declare_proto_rot(coord2qrb);
 declare_proto_rot(az_sp2az_lp);
 declare_proto_rot(dist_sp2dist_lp);
+declare_proto_rot(pause);
 
 /*
  * convention: upper case cmd is set, lowercase is get
@@ -116,26 +179,27 @@ declare_proto_rot(dist_sp2dist_lp);
  * NB: 'q' 'Q' '?' are reserved by interactive mode interface
  */
 struct test_table test_list[] = {
-	{ 'P', "set_pos",   set_position,   ARG_IN, "Azimuth", "Elevation" },
-	{ 'p', "get_pos",   get_position,   ARG_OUT, "Azimuth", "Elevation" },
-	{ 'K', "park",      park,           ARG_NONE, },
-	{ 'S', "stop",      stop,           ARG_NONE, },
-	{ 'R', "reset",     reset,          ARG_IN, "Reset" },
-	{ 'M', "move",      move,           ARG_IN, "Direction", "Speed" },
-	{ 'C', "set_conf",  inter_set_conf, ARG_IN, "Token", "Value" },
-	{ '_', "get_info",  get_info,       ARG_OUT, "Info" },
-	{ 'w', "send_cmd",  send_cmd,       ARG_IN1|ARG_IN_LINE|ARG_OUT2, "Cmd", "Reply" },
-	{ '1', "dump_caps", dump_caps, },
-	{ 0x8f,"dump_state",dump_state,     ARG_OUT },
-	{ 'L', "lonlat2loc",lonlat2loc,     ARG_IN1|ARG_IN2|ARG_IN3|ARG_OUT1, "Longitude", "Latitude", "Loc Len [2-12]", "Locator" },
-	{ 'l', "loc2lonlat",loc2lonlat,     ARG_IN1|ARG_OUT1|ARG_OUT2, "Locator", "Longitude", "Latitude" },
-	{ 'D', "dms2dec",   d_m_s2dec,      ARG_IN1|ARG_IN2|ARG_IN3|ARG_IN4|ARG_OUT1, "Degrees", "Minutes", "Seconds", "S/W", "Dec Degrees" },
-	{ 'd', "dec2dms",   dec2d_m_s,      ARG_IN1|ARG_OUT1|ARG_OUT2|ARG_OUT3|ARG_OUT4, "Dec Degrees", "Degrees", "Minutes", "Seconds", "S/W" },
-	{ 'E', "dmmm2dec",  d_mm2dec,       ARG_IN1|ARG_IN2|ARG_IN3|ARG_OUT1, "Degrees", "Dec Minutes", "S/W", "Dec Deg" },
-	{ 'e', "dec2dmmm",  dec2d_mm,       ARG_IN1|ARG_OUT1|ARG_OUT2|ARG_OUT3, "Dec Deg", "Degrees", "Dec Minutes", "S/W" },
-	{ 'B', "qrb",       coord2qrb,      ARG_IN1|ARG_IN2|ARG_IN3|ARG_IN4|ARG_OUT1|ARG_OUT2, "Lon 1", "Lat 1", "Lon 2", "Lat 2", "QRB Distance", "QRB Azimuth" },
-	{ 'A', "a_sp2a_lp", az_sp2az_lp,    ARG_IN1|ARG_OUT1, "Short Path Deg", "Long Path Deg" },
-	{ 'a', "d_sp2d_lp", dist_sp2dist_lp,ARG_IN1|ARG_OUT1, "Short Path km", "Long Path km" },
+	{ 'P', "set_pos",   ACTION(set_position),   ARG_IN, "Azimuth", "Elevation" },
+	{ 'p', "get_pos",   ACTION(get_position),   ARG_OUT, "Azimuth", "Elevation" },
+	{ 'K', "park",      ACTION(park),           ARG_NONE, },
+	{ 'S', "stop",      ACTION(stop),           ARG_NONE, },
+	{ 'R', "reset",     ACTION(reset),          ARG_IN, "Reset" },
+	{ 'M', "move",      ACTION(move),           ARG_IN, "Direction", "Speed" },
+	{ 'C', "set_conf",  ACTION(inter_set_conf), ARG_IN, "Token", "Value" },
+	{ '_', "get_info",  ACTION(get_info),       ARG_OUT, "Info" },
+	{ 'w', "send_cmd",  ACTION(send_cmd),       ARG_IN1|ARG_IN_LINE|ARG_OUT2, "Cmd", "Reply" },
+	{ '1', "dump_caps", ACTION(dump_caps), },
+	{ 0x8f,"dump_state",ACTION(dump_state),     ARG_OUT },
+	{ 'L', "lonlat2loc",ACTION(lonlat2loc),     ARG_IN1|ARG_IN2|ARG_IN3|ARG_OUT1, "Longitude", "Latitude", "Loc Len [2-12]", "Locator" },
+	{ 'l', "loc2lonlat",ACTION(loc2lonlat),     ARG_IN1|ARG_OUT1|ARG_OUT2, "Locator", "Longitude", "Latitude" },
+	{ 'D', "dms2dec",   ACTION(d_m_s2dec),      ARG_IN1|ARG_IN2|ARG_IN3|ARG_IN4|ARG_OUT1, "Degrees", "Minutes", "Seconds", "S/W", "Dec Degrees" },
+	{ 'd', "dec2dms",   ACTION(dec2d_m_s),      ARG_IN1|ARG_OUT1|ARG_OUT2|ARG_OUT3|ARG_OUT4, "Dec Degrees", "Degrees", "Minutes", "Seconds", "S/W" },
+	{ 'E', "dmmm2dec",  ACTION(d_mm2dec),       ARG_IN1|ARG_IN2|ARG_IN3|ARG_OUT1, "Degrees", "Dec Minutes", "S/W", "Dec Deg" },
+	{ 'e', "dec2dmmm",  ACTION(dec2d_mm),       ARG_IN1|ARG_OUT1|ARG_OUT2|ARG_OUT3, "Dec Deg", "Degrees", "Dec Minutes", "S/W" },
+	{ 'B', "qrb",       ACTION(coord2qrb),      ARG_IN1|ARG_IN2|ARG_IN3|ARG_IN4|ARG_OUT1|ARG_OUT2, "Lon 1", "Lat 1", "Lon 2", "Lat 2", "QRB Distance", "QRB Azimuth" },
+	{ 'A', "a_sp2a_lp", ACTION(az_sp2az_lp),    ARG_IN1|ARG_OUT1, "Short Path Deg", "Long Path Deg" },
+	{ 'a', "d_sp2d_lp", ACTION(dist_sp2dist_lp),ARG_IN1|ARG_OUT1, "Short Path km", "Long Path km" },
+	{ 0x8c, "pause",    ACTION(pause),          ARG_IN, "Seconds" },
 	{ 0x00, "", NULL },
 
 };
@@ -210,6 +274,38 @@ void hash_delete_all() {
 }
 
 
+#ifdef HAVE_LIBREADLINE
+/* Frees allocated memory and sets pointers to NULL before calling readline
+ * and then parses the input into space separated tokens.
+ */
+static void rp_getline(const char *s)
+{
+	int i;
+
+	/* free allocated memory and set pointers to NULL */
+	if (input_line) {
+		free(input_line);
+		input_line = (char *)NULL;
+	}
+
+	if (result) {
+		result = (char *)NULL;
+	}
+
+	/* cmd, arg1, arg2, arg3, arg4, arg5, arg6
+	 * arg5 and arg 6 are currently unused.
+	 */
+	for (i = 0; i < 7; i++)
+		parsed_input[i] = NULL;
+
+	/* Action!  Returns typed line with newline stripped. */
+	input_line = readline(s);
+}
+
+
+#endif
+
+
 /*
  * TODO: use Lex?
  */
@@ -240,6 +336,73 @@ static int scanfc(FILE *fin, const char *format, void *p)
 	} while(1);
 }
 
+/*
+ * function to get the next word from the command line or from stdin
+ * until stdin exhausted. stdin is read if the special token '-' is
+ * found on the command line.
+ *
+ * returns EOF when words exhausted
+ * returns <0 is error number
+ * returns >=0 when successful
+ */
+static int next_word (char *buffer, int argc, char *argv[], int newline)
+{
+  int ret;
+  char c;
+  static int reading_stdin;
+
+  if (!reading_stdin)
+    {
+      if (optind >= argc) return EOF;
+      else if ('-' == argv[optind][0])
+        {
+          ++optind;
+          reading_stdin = 1;
+        }
+    }
+
+  if (reading_stdin)
+    {
+      do
+        {
+          do ret = scanf (" %c%" STR(MAXARGSZ) "[^ \t\n#]", &c, &buffer[1]); while (EINTR == ret);
+          if (ret > 0 && '#' == c)
+            {
+              do ret = scanf ("%*[^\n]"); while (EINTR == ret); /* consume comments */
+              ret = 0;
+            }
+        }
+      while (!ret);
+      if (EOF == ret) reading_stdin = 0;
+      else if (ret < 0)
+        {
+          rig_debug (RIG_DEBUG_ERR, "scanf: %s\n", strerror (errno));
+          reading_stdin = 0;
+        }
+      else
+        {
+          buffer[0] = c;
+          buffer[1 == ret ? 1 : MAXARGSZ] = '\0';
+          if (newline) putchar ('\n');
+          fputs (buffer, stdout);
+          putchar (' ');
+        }
+    }
+
+  if (!reading_stdin)
+    {
+			if (optind < argc)
+        {
+          strncpy (buffer, argv[optind++], MAXARGSZ);
+          buffer[MAXARGSZ] = '\0';
+          ret = 1;
+        }
+      else ret = EOF;
+    }
+
+  return ret;
+}
+
 #define fprintf_flush(f, a...) \
             ({ int __ret; \
                __ret = fprintf((f), a); \
@@ -248,11 +411,8 @@ static int scanfc(FILE *fin, const char *format, void *p)
             })
 
 
-#define MAXARGSZ 127
-
 extern int interactive;
 extern int prompt;
-extern int opt_end;
 extern char send_cmd_term;
 int ext_resp = 0;
 unsigned char resp_sep = '\n';      /* Default response separator */
@@ -263,194 +423,576 @@ int rotctl_parse(ROT *my_rot, FILE *fin, FILE *fout, char *argv[], int argc)
 	unsigned char cmd;
 	struct test_table *cmd_entry;
 
-	char arg1[MAXARGSZ + 1], *p1;
-	char arg2[MAXARGSZ + 1], *p2;
-	char arg3[MAXARGSZ + 1], *p3;
-	char arg4[MAXARGSZ + 1], *p4;
-	char *p5, *p6;
+	char command[MAXARGSZ+1];
+	char arg1[MAXARGSZ + 1], *p1 = NULL;
+	char arg2[MAXARGSZ + 1], *p2 = NULL;
+	char arg3[MAXARGSZ + 1], *p3 = NULL;
+	char arg4[MAXARGSZ + 1], *p4 = NULL;
+	char *p5 = NULL;
+	char *p6 = NULL;
 	static int last_was_ret = 1;
 
-	if (interactive) {
-		if (prompt)
-			fprintf_flush(fout, "\nRotator command: ");
+	/* cmd, internal, rotctld */
+	if (!(interactive && prompt && have_rl)) {
+		if (interactive) {
+			if (prompt)
+				fprintf_flush(fout, "\nRotator command: ");
 
-		do {
-			if (scanfc(fin, "%c", &cmd) < 1)
-				return -1;
-
-			/* Extended response protocol requested with leading '+' on command
-			 * string--rotctld only!
-			 */
-			if (cmd == '+' && !prompt) {
-				ext_resp = 1;
+			do {
 				if (scanfc(fin, "%c", &cmd) < 1)
 					return -1;
-			} else if (cmd == '+' && prompt) {
-				return 0;
-			}
 
-			if (cmd != '\\' && cmd != '_' && cmd != '#' && ispunct(cmd) && !prompt) {
-				ext_resp = 1;
-				resp_sep = cmd;
-				if (scanfc(fin, "%c", &cmd) < 1)
-					return -1;
-			} else if (cmd != '\\' && cmd != '?' && cmd != '_' && cmd != '#' && ispunct(cmd) && prompt) {
-				return 0;
-			}
-
-			/* command by name */
-			if (cmd == '\\') {
-				unsigned char cmd_name[MAXNAMSIZ], *pcmd = cmd_name;
-				int c_len = MAXNAMSIZ;
-
-				if (scanfc(fin, "%c", pcmd) < 1)
-					return -1;
-
-				while(c_len-- && (isalnum(*pcmd) || *pcmd == '_' ))
-					if (scanfc(fin, "%c", ++pcmd) < 1)
+				/* Extended response protocol requested with leading '+' on command
+				 * string--rotctld only!
+				 */
+				if (cmd == '+' && !prompt) {
+					ext_resp = 1;
+					if (scanfc(fin, "%c", &cmd) < 1)
 						return -1;
-
-				*pcmd = '\0';
-				cmd = parse_arg((char *) cmd_name);
-				break;
-			}
-
-			if (cmd == 0x0a || cmd == 0x0d) {
-				if (last_was_ret) {
-					if (prompt) {
-						fprintf_flush(fout, "? for help, q to quit.\n");
-					}
+				} else if (cmd == '+' && prompt) {
 					return 0;
 				}
-				last_was_ret = 1;
+
+				if (cmd != '\\' && cmd != '_' && cmd != '#' && ispunct(cmd) && !prompt) {
+					ext_resp = 1;
+					resp_sep = cmd;
+					if (scanfc(fin, "%c", &cmd) < 1)
+						return -1;
+				} else if (cmd != '\\' && cmd != '?' && cmd != '_' && cmd != '#' && ispunct(cmd) && prompt) {
+					return 0;
+				}
+
+				/* command by name */
+				if (cmd == '\\') {
+					unsigned char cmd_name[MAXNAMSIZ], *pcmd = cmd_name;
+					int c_len = MAXNAMSIZ;
+
+					if (scanfc(fin, "%c", pcmd) < 1)
+						return -1;
+
+					while(c_len-- && (isalnum(*pcmd) || *pcmd == '_' ))
+						if (scanfc(fin, "%c", ++pcmd) < 1)
+							return -1;
+
+					*pcmd = '\0';
+					cmd = parse_arg((char *) cmd_name);
+					break;
+				}
+
+				if (cmd == 0x0a || cmd == 0x0d) {
+					if (last_was_ret) {
+						if (prompt) {
+							fprintf_flush(fout, "? for help, q to quit.\n");
+						}
+						return 0;
+					}
+					last_was_ret = 1;
+				}
+			} while (cmd == 0x0a || cmd == 0x0d);
+
+			last_was_ret = 0;
+
+			/* comment line */
+			if (cmd == '#') {
+				while( cmd != '\n' && cmd != '\r')
+					if (scanfc(fin, "%c", &cmd) < 1)
+						return -1;
+				return 0;
 			}
-		} while (cmd == 0x0a || cmd == 0x0d);
+			if (cmd == 'Q' || cmd == 'q')
+				return 1;
+			if (cmd == '?') {
+				usage_rot(fout);
+				fflush(fout);
+				return 0;
+			}
+		} else {
+			/* parse rest of command line */
+      retcode = next_word (command, argc, argv, 1);
+      if (EOF == retcode) return 1;
+      else if (retcode < 0) return retcode;
+      else if ('\0' == command[1]) {
+        cmd = command[0];
+      } else {
+        cmd = parse_arg (command);
+      }
+		}
 
-		last_was_ret = 0;
-
-		/* comment line */
-		if (cmd == '#') {
-			while( cmd != '\n' && cmd != '\r')
-				if (scanfc(fin, "%c", &cmd) < 1)
-					return -1;
+		cmd_entry = find_cmd_entry(cmd);
+		if (!cmd_entry) {
+			fprintf_flush(stderr, "Command '%c' not found!\n", cmd);
 			return 0;
 		}
-		if (cmd == 'Q' || cmd == 'q')
+
+		if ((cmd_entry->flags & ARG_IN_LINE) &&
+				(cmd_entry->flags & ARG_IN1) && cmd_entry->arg1) {
+			if (interactive) {
+				char *nl;
+				if (prompt)
+					fprintf_flush(fout, "%s: ", cmd_entry->arg1);
+				if (fgets(arg1, MAXARGSZ, fin) == NULL)
+					return -1;
+				if (arg1[0] == 0xa)
+					if (fgets(arg1, MAXARGSZ, fin) == NULL)
+						return -1;
+				nl = strchr(arg1, 0xa);
+				if (nl) *nl = '\0';	/* chomp */
+				p1 = arg1[0]==' '?arg1+1:arg1;
+			} else {
+        retcode = next_word (arg1, argc, argv, 0);
+        if (EOF == retcode) {
+					fprintf(stderr, "Invalid arg for command '%s'\n",
+                  cmd_entry->name);
+          return 1;
+        }
+        else if (retcode < 0) return retcode;
+				p1 = arg1;
+			}
+		} else 	if ((cmd_entry->flags & ARG_IN1) && cmd_entry->arg1) {
+			if (interactive) {
+				if (prompt)
+					fprintf_flush(fout, "%s: ", cmd_entry->arg1);
+				if (scanfc(fin, "%s", arg1) < 1)
+					return -1;
+				p1 = arg1;
+			} else {
+        retcode = next_word (arg1, argc, argv, 0);
+        if (EOF == retcode) {
+					fprintf(stderr, "Invalid arg for command '%s'\n",
+                  cmd_entry->name);
+          return 1;
+        }
+        else if (retcode < 0) return retcode;
+				p1 = arg1;
+			}
+		}
+		if (p1 && p1[0]!='?' && (cmd_entry->flags & ARG_IN2) && cmd_entry->arg2) {
+			if (interactive) {
+				if (prompt)
+					fprintf_flush(fout, "%s: ", cmd_entry->arg2);
+				if (scanfc(fin, "%s", arg2) < 1)
+					return -1;
+				p2 = arg2;
+			} else {
+        retcode = next_word (arg2, argc, argv, 0);
+        if (EOF == retcode) {
+					fprintf(stderr, "Invalid arg for command '%s'\n",
+                  cmd_entry->name);
+          return 1;
+        }
+        else if (retcode < 0) return retcode;
+				p2 = arg2;
+			}
+		}
+		if (p1 && p1[0]!='?' && (cmd_entry->flags & ARG_IN3) && cmd_entry->arg3) {
+			if (interactive) {
+				if (prompt)
+					fprintf_flush(fout, "%s: ", cmd_entry->arg3);
+				if (scanfc(fin, "%s", arg3) < 1)
+					return -1;
+				p3 = arg3;
+			} else {
+        retcode = next_word (arg3, argc, argv, 0);
+        if (EOF == retcode) {
+					fprintf(stderr, "Invalid arg for command '%s'\n",
+                  cmd_entry->name);
+          return 1;
+        }
+        else if (retcode < 0) return retcode;
+				p3 = arg3;
+			}
+		}
+
+		if (p1 && p1[0]!='?' && (cmd_entry->flags & ARG_IN4) && cmd_entry->arg4) {
+			if (interactive) {
+				if (prompt)
+					fprintf_flush(fout, "%s: ", cmd_entry->arg4);
+				if (scanfc(fin, "%s", arg4) < 1)
+					return -1;
+				p4 = arg4;
+			} else {
+        retcode = next_word (arg4, argc, argv, 0);
+        if (EOF == retcode) {
+					fprintf(stderr, "Invalid arg for command '%s'\n",
+                  cmd_entry->name);
+          return 1;
+        }
+        else if (retcode < 0) return retcode;
+				p4 = arg4;
+			}
+		}
+	}
+
+#ifdef HAVE_LIBREADLINE
+
+	if (interactive && prompt && have_rl) {
+		int j, x;
+
+#ifdef HAVE_READLINE_HISTORY
+		/* Minimum space for 32+1+128+1+128+1+128+1+128+1+128+1+128+1 = 807
+		 * chars, so allocate 896 chars cleared to zero for safety.
+		 */
+		rp_hist_buf = (char *)calloc(896, sizeof(char));
+#endif
+
+		rl_instream = fin;
+		rl_outstream = fout;
+
+		rp_getline("\nRotator command: ");
+
+		/* EOF (Ctl-D) received on empty input line, bail out gracefully. */
+		if (!input_line) {
+			fprintf_flush(fout, "\n");
 			return 1;
-		if (cmd == '?') {
+		}
+
+		/* Q or q to quit */
+		if (!(strncasecmp(input_line, "q", 1)))
+			return 1;
+
+		/* '?' for help */
+		if (!(strncmp(input_line, "?", 1))) {
 			usage_rot(fout);
 			fflush(fout);
 			return 0;
 		}
-	} else {
-		/* parse rest of command line */
-		if (optind >= argc)
+
+		/* '#' for comment */
+		if (!(strncmp(input_line, "#", 1)))
+			return 0;
+
+		/* Blank line entered */
+		if (!(strcmp(input_line, ""))) {
+			fprintf(fout, "? for help, q to quit.\n");
+			fflush(fout);
+			return 0;
+		}
+
+		rig_debug(RIG_DEBUG_BUG, "%s: input_line: %s\n", __func__, input_line);
+
+		/* Split input_line on any number of spaces to get the command token
+		 * Tabs are intercepted by readline for completion and a newline
+		 * causes readline to return the typed text.  If more than one
+		 * argument is given, it will be parsed out later.
+		 */
+		result = strtok(input_line, " ");
+
+		/* parsed_input stores pointers into input_line where the token strings
+		 * start.
+		 */
+		if (result) {
+			parsed_input[0] = result;
+		} else {
+			/* Oops!  Invoke GDB!! */
+			fprintf_flush(fout, "\n");
 			return 1;
-		if (argv[optind][1] == '\0')
-				cmd = argv[optind][0];
-		else
-				cmd = parse_arg(argv[optind]);
-		optind++;
-	}
-
-	cmd_entry = find_cmd_entry(cmd);
-	if (!cmd_entry) {
-		fprintf_flush(stderr, "Command '%c' not found!\n", cmd);
-		return 0;
-	}
-
-	p1 = p2 = p3 = p4 = p5 = p6 = NULL;
-
-	if ((cmd_entry->flags & ARG_IN_LINE) &&
-			(cmd_entry->flags & ARG_IN1) && cmd_entry->arg1) {
-		if (interactive) {
-			char *nl;
-			if (prompt)
-				fprintf_flush(fout, "%s: ", cmd_entry->arg1);
-			if (fgets(arg1, MAXARGSZ, fin) == NULL)
-                return -1;
-			if (arg1[0] == 0xa)
-				if (fgets(arg1, MAXARGSZ, fin) == NULL)
-                    return -1;
-			nl = strchr(arg1, 0xa);
-			if (nl) *nl = '\0';	/* chomp */
-			p1 = arg1[0]==' '?arg1+1:arg1;
-		} else {
-			if (!argv[optind]) {
-				fprintf(stderr, "Invalid arg for command '%s'\n",
-							cmd_entry->name);
-				exit(1);
-			}
-			p1 = argv[optind++];
 		}
-	} else 	if ((cmd_entry->flags & ARG_IN1) && cmd_entry->arg1) {
-		if (interactive) {
-			if (prompt)
-				fprintf_flush(fout, "%s: ", cmd_entry->arg1);
-			if (scanfc(fin, "%s", arg1) < 1)
-				return -1;
+
+		/* At this point parsed_input contains the typed text of the command
+		 * with surrounding space characters removed.  If Readline History is
+		 * available, copy the command string into a history buffer.
+		 */
+
+		/* Single character command */
+		if ((strlen(parsed_input[0]) == 1) && (*parsed_input[0] != '\\')) {
+			cmd = *parsed_input[0];
+
+#ifdef HAVE_READLINE_HISTORY
+			/* Store what is typed, not validated, for history. */
+			if (rp_hist_buf)
+				strncpy(rp_hist_buf, parsed_input[0], 1);
+#endif
+		}
+		/* Test the command token, parsed_input[0] */
+		else if ((*parsed_input[0] == '\\') && (strlen(parsed_input[0]) > 1)) {
+			char cmd_name[MAXNAMSIZ];
+
+			/* if there is no terminating '\0' character in the source string,
+			 * srncpy() doesn't add one even if the supplied length is less
+			 * than the destination array.  Truncate the source string here.
+			 */
+			 if (strlen(parsed_input[0] + 1) >= MAXNAMSIZ)
+				*(parsed_input[0] + MAXNAMSIZ) = '\0';
+
+#ifdef HAVE_READLINE_HISTORY
+			if (rp_hist_buf)
+				strncpy(rp_hist_buf, parsed_input[0], MAXNAMSIZ);
+#endif
+			/* The starting position of the source string is the first
+			 * character past the initial '\'.  Using MAXNAMSIZ for the
+			 * length leaves enough space for the '\0' string terminator in the
+			 * cmd_name array.
+			 */
+			strncpy(cmd_name, parsed_input[0] + 1, MAXNAMSIZ);
+
+			/* Sanity check as valid multiple character commands consist of
+			 * alpha-numeric characters and the underscore ('_') character.
+			 */
+			for (j = 0; cmd_name[j] != '\0'; j++) {
+				if (!(isalnum((int)cmd_name[j]) || cmd_name[j] == '_')) {
+					fprintf(stderr, "Valid multiple character command names contain alpha-numeric characters plus '_'\n");
+					return 0;
+				}
+			}
+
+			cmd = parse_arg(cmd_name);
+		}
+		/* Single '\' entered, prompt again */
+		 else if ((*parsed_input[0] == '\\') && (strlen(parsed_input[0]) == 1)) {
+			return 0;
+		}
+		/* Multiple characters but no leading '\' */
+		else {
+			fprintf(stderr, "Precede multiple character command names with '\\'\n");
+			return 0;
+		}
+
+		cmd_entry = find_cmd_entry(cmd);
+		if (!cmd_entry) {
+			if (cmd == '\0')
+				fprintf(stderr, "Command '%s' not found!\n", parsed_input[0]);
+			else
+				fprintf(stderr, "Command '%c' not found!\n", cmd);
+
+			return 0;
+		}
+
+		/* \send_cmd */
+		if ((cmd_entry->flags & ARG_IN_LINE) &&
+				(cmd_entry->flags & ARG_IN1) && cmd_entry->arg1) {
+			/* Check for a non-existent delimiter so as to not break up
+			 * remaining line into separate tokens (spaces OK).
+			 */
+			result = strtok(NULL, "\0");
+
+			if (result) {
+				x = 1;
+				parsed_input[x] = result;
+			} else {
+				x = 0;
+				char pmptstr[(strlen(cmd_entry->arg1) + 3)];
+
+				strcpy(pmptstr, cmd_entry->arg1);
+				strcat(pmptstr, ": ");
+
+				rp_getline(pmptstr);
+
+				/* Blank line entered */
+				if (!(strcmp(input_line, ""))) {
+					fprintf(fout, "? for help, q to quit.\n");
+					fflush(fout);
+					return 0;
+				}
+
+				if (input_line)
+					parsed_input[x] = input_line;
+				else {
+					fprintf_flush(fout, "\n");
+					return 1;
+				}
+			}
+
+			/* The arg1 array size is MAXARGSZ + 1 so truncate it to fit if larger. */
+			if (strlen(parsed_input[x]) > MAXARGSZ)
+				parsed_input[x][MAXARGSZ] = '\0';
+
+#ifdef HAVE_READLINE_HISTORY
+			if (rp_hist_buf) {
+				strncat(rp_hist_buf, " ", 1);
+				strncat(rp_hist_buf, parsed_input[x], MAXARGSZ);
+			}
+#endif
+			strcpy(arg1, parsed_input[x]);
 			p1 = arg1;
-		} else {
-			if (!argv[optind]) {
-				fprintf(stderr, "Invalid arg for command '%s'\n",
-							cmd_entry->name);
-				exit(1);
-			}
-			p1 = argv[optind++];
 		}
-	}
-	if (p1 && p1[0]!='?' && (cmd_entry->flags & ARG_IN2) && cmd_entry->arg2) {
-		if (interactive) {
-			if (prompt)
-				fprintf_flush(fout, "%s: ", cmd_entry->arg2);
-			if (scanfc(fin, "%s", arg2) < 1)
-				return -1;
+
+		/* Normal argument parsing. */
+		else if ((cmd_entry->flags & ARG_IN1) && cmd_entry->arg1) {
+			result = strtok(NULL, " ");
+
+			if (result) {
+				x = 1;
+				parsed_input[x] = result;
+			} else {
+				x = 0;
+				char pmptstr[(strlen(cmd_entry->arg1) + 3)];
+
+				strcpy(pmptstr, cmd_entry->arg1);
+				strcat(pmptstr, ": ");
+
+				rp_getline(pmptstr);
+
+				if (!(strcmp(input_line, ""))) {
+					fprintf(fout, "? for help, q to quit.\n");
+					fflush(fout);
+					return 0;
+				}
+
+				result = strtok(input_line, " ");
+
+				if (result) {
+					parsed_input[x] = result;
+				} else {
+					fprintf_flush(fout, "\n");
+					return 1;
+				}
+			}
+
+			if (strlen(parsed_input[x]) > MAXARGSZ)
+				parsed_input[x][MAXARGSZ] = '\0';
+
+#ifdef HAVE_READLINE_HISTORY
+			if (rp_hist_buf) {
+				strncat(rp_hist_buf, " ", 1);
+				strncat(rp_hist_buf, parsed_input[x], MAXARGSZ);
+			}
+#endif
+			strcpy(arg1, parsed_input[x]);
+			p1 = arg1;
+		}
+		if (p1 && p1[0] != '?' && (cmd_entry->flags & ARG_IN2) && cmd_entry->arg2) {
+			result = strtok(NULL, " ");
+
+			if (result) {
+				x = 2;
+				parsed_input[x] = result;
+			} else {
+				x = 0;
+				char pmptstr[(strlen(cmd_entry->arg2) + 3)];
+
+				strcpy(pmptstr, cmd_entry->arg2);
+				strcat(pmptstr, ": ");
+
+				rp_getline(pmptstr);
+
+				if (!(strcmp(input_line, ""))) {
+					fprintf(fout, "? for help, q to quit.\n");
+					fflush(fout);
+					return 0;
+				}
+
+				result = strtok(input_line, " ");
+
+				if (result) {
+					parsed_input[x] = result;
+				} else {
+					fprintf_flush(fout, "\n");
+					return 1;
+				}
+			}
+
+			if (strlen(parsed_input[x]) > MAXARGSZ)
+				parsed_input[x][MAXARGSZ] = '\0';
+
+#ifdef HAVE_READLINE_HISTORY
+			if (rp_hist_buf) {
+				strncat(rp_hist_buf, " ", 1);
+				strncat(rp_hist_buf, parsed_input[x], MAXARGSZ);
+			}
+#endif
+			strcpy(arg2, parsed_input[x]);
 			p2 = arg2;
-		} else {
-			if (!argv[optind]) {
-				fprintf(stderr, "Invalid arg for command '%s'\n",
-							cmd_entry->name);
-				exit(1);
-			}
-			p2 = argv[optind++];
 		}
-	}
-	if (p1 && p1[0]!='?' && (cmd_entry->flags & ARG_IN3) && cmd_entry->arg3) {
-		if (interactive) {
-			if (prompt)
-				fprintf_flush(fout, "%s: ", cmd_entry->arg3);
-			if (scanfc(fin, "%s", arg3) < 1)
-				return -1;
+		if (p1 && p1[0] != '?' && (cmd_entry->flags & ARG_IN3) && cmd_entry->arg3) {
+			result = strtok(NULL, " ");
+
+			if (result) {
+				x = 3;
+				parsed_input[x] = result;
+			} else {
+				x = 0;
+				char pmptstr[(strlen(cmd_entry->arg3) + 3)];
+
+				strcpy(pmptstr, cmd_entry->arg3);
+				strcat(pmptstr, ": ");
+
+				rp_getline(pmptstr);
+
+				if (!(strcmp(input_line, ""))) {
+					fprintf(fout, "? for help, q to quit.\n");
+					fflush(fout);
+					return 0;
+				}
+
+				result = strtok(input_line, " ");
+
+				if (result) {
+					parsed_input[x] = result;
+				} else {
+					fprintf_flush(fout, "\n");
+					return 1;
+				}
+			}
+
+			if (strlen(parsed_input[x]) > MAXARGSZ)
+				parsed_input[x][MAXARGSZ] = '\0';
+
+#ifdef HAVE_READLINE_HISTORY
+			if (rp_hist_buf) {
+				strncat(rp_hist_buf, " ", 1);
+				strncat(rp_hist_buf, parsed_input[x], MAXARGSZ);
+			}
+#endif
+			strcpy(arg3, parsed_input[x]);
 			p3 = arg3;
-		} else {
-			if (!argv[optind]) {
-				fprintf(stderr, "Invalid arg for command '%s'\n",
-							cmd_entry->name);
-				exit(1);
-			}
-			p3 = argv[optind++];
 		}
+		if (p1 && p1[0] != '?' && (cmd_entry->flags & ARG_IN4) && cmd_entry->arg4) {
+			result = strtok(NULL, " ");
+
+			if (result) {
+				x = 4;
+				parsed_input[x] = result;
+			} else {
+				x = 0;
+				char pmptstr[(strlen(cmd_entry->arg4) + 3)];
+
+				strcpy(pmptstr, cmd_entry->arg4);
+				strcat(pmptstr, ": ");
+
+				rp_getline(pmptstr);
+
+				if (!(strcmp(input_line, ""))) {
+					fprintf(fout, "? for help, q to quit.\n");
+					fflush(fout);
+					return 0;
+				}
+
+				result = strtok(input_line, " ");
+
+				if (result) {
+					parsed_input[x] = result;
+				} else {
+					fprintf_flush(fout, "\n");
+					return 1;
+				}
+			}
+
+			if (strlen(parsed_input[x]) > MAXARGSZ)
+				parsed_input[x][MAXARGSZ] = '\0';
+
+#ifdef HAVE_READLINE_HISTORY
+			if (rp_hist_buf) {
+				strncat(rp_hist_buf, " ", 1);
+				strncat(rp_hist_buf, parsed_input[x], MAXARGSZ);
+			}
+#endif
+			strcpy(arg4, parsed_input[x]);
+			p4 = arg4;
+		}
+#ifdef HAVE_READLINE_HISTORY
+		if (rp_hist_buf) {
+			add_history(rp_hist_buf);
+			free(rp_hist_buf);
+			rp_hist_buf = (char *)NULL;
+		}
+#endif
 	}
 
-	if (p1 && p1[0]!='?' && (cmd_entry->flags & ARG_IN4) && cmd_entry->arg4) {
-		if (interactive) {
-			if (prompt)
-				fprintf_flush(fout, "%s: ", cmd_entry->arg4);
-			if (scanfc(fin, "%s", arg4) < 1)
-				return -1;
-			p4 = arg4;
-		} else {
-			if (!argv[optind]) {
-				fprintf(stderr, "Invalid arg for command '%s'\n",
-							cmd_entry->name);
-				exit(1);
-			}
-			p4 = argv[optind++];
-		}
-	}
+#endif	/* HAVE_LIBREADLINE */
+
 
 	/*
-	 * mutex locking needed because rigctld is multithreaded
+	 * mutex locking needed because rotctld is multithreaded
 	 * and hamlib is not MT-safe
 	 */
 #ifdef HAVE_PTHREAD
@@ -500,7 +1042,7 @@ int rotctl_parse(ROT *my_rot, FILE *fin, FILE *fout, char *argv[], int argc)
 		/* only for rotctld */
 		if (interactive && !prompt) {
 			/* netrotctl RIG_OK */
-			if (!(cmd_entry->flags & ARG_OUT) && !opt_end && !ext_resp)
+			if (!(cmd_entry->flags & ARG_OUT) && !ext_resp)
 				fprintf(fout, NETROTCTL_RET "0\n");
 
 			/* Extended Response protocol */
@@ -509,10 +1051,6 @@ int rotctl_parse(ROT *my_rot, FILE *fin, FILE *fout, char *argv[], int argc)
 				ext_resp = 0;
 				resp_sep = '\n';
 			}
-
-			/* Nate's protocol (obsolete) */
-			else if ((cmd_entry->flags & ARG_OUT) && opt_end)
-				fprintf(fout, "END\n");
 		}
 	}
 
@@ -534,7 +1072,7 @@ void usage_rot(FILE *fout)
 {
 	int i, nbspaces;
 
-	fprintf(fout, "Commands (some may not be available for this rig):\n");
+	fprintf(fout, "Commands (some may not be available for this rotator):\n");
 	for (i = 0; test_list[i].cmd != 0; i++) {
 		fprintf(fout, "%c: %-12s(", isprint(test_list[i].cmd) ?
 				test_list[i].cmd : '?', test_list[i].name);
@@ -551,6 +1089,12 @@ void usage_rot(FILE *fout)
 
 		fprintf(fout, ")\n");
 	}
+
+	fprintf(fout, "\n\nIn interactive mode prefix long command names with '\\', e.g. '\\dump_state'\n\n"
+          "The special command '-' is used to read further commands from standard input\n"
+          "Commands and arguments read from standard input must be white space separated,\n"
+          "comments are allowed, comments start with the # character and continue to the\n"
+          "end of the line.\n");
 }
 
 
@@ -1086,3 +1630,11 @@ declare_proto_rot(dist_sp2dist_lp)
 	return RIG_OK;
 }
 
+/* '0x8c'--pause processing */
+declare_proto_rot(pause)
+{
+  unsigned seconds;
+	CHKSCN1ARG(sscanf(arg1, "%u", &seconds));
+  sleep (seconds);
+	return RIG_OK;
+}
