@@ -1,5 +1,6 @@
 /*
  * rotctld.c - (C) Stephane Fillod 2000-2011
+ *             (C) Nate Bargmann 2010,2011,2012,2013
  *
  * This program test/control a rotator using Hamlib.
  * It takes commands from network connection.
@@ -47,6 +48,9 @@
 #elif HAVE_WS2TCPIP_H
 #include <ws2tcpip.h>
 #include <fcntl.h>
+#	if defined(HAVE_WSPIAPI_H)
+#		include <wspiapi.h>
+#	endif
 #endif
 #ifdef HAVE_NETDB_H
 #include <netdb.h>
@@ -78,7 +82,7 @@ void usage();
  * NB: do NOT use -W since it's reserved by POSIX.
  * TODO: add an option to read from a file
  */
-#define SHORT_OPTIONS "m:r:s:C:t:T:LuevhVl"
+#define SHORT_OPTIONS "m:r:s:C:t:T:LuvhVl"
 static struct option long_options[] =
 {
 	{"model",       1, 0, 'm'},
@@ -90,7 +94,6 @@ static struct option long_options[] =
 	{"set-conf",    1, 0, 'C'},
 	{"show-conf",   0, 0, 'L'},
 	{"dump-caps",   0, 0, 'u'},
-	{"end-marker",  0, 0, 'e'},
 	{"verbose",     0, 0, 'v'},
 	{"help",        0, 0, 'h'},
 	{"version",     0, 0, 'V'},
@@ -99,7 +102,6 @@ static struct option long_options[] =
 
 int interactive = 1;    /* no cmd because of daemon */
 int prompt= 0 ;         /* Daemon mode for rigparse return string */
-int opt_end= 0 ;        /* END marker for rotctld */
 
 const char *portno = "4533";
 const char *src_addr = NULL; /* INADDR_ANY */
@@ -199,10 +201,6 @@ int main (int argc, char *argv[])
 			case 'u':
 				dump_caps_opt++;
 				break;
-			case 'e':
-				opt_end = 1;
-				fprintf(stderr, "-e|--end-marker option is deprecated!\nPlease consider using the Extended Response protocol instead.\n");
-				break;
 			default:
 				usage();	/* unknown option? */
 				exit(1);
@@ -461,7 +459,6 @@ void usage()
 	"  -L, --show-conf            list all config parameters\n"
 	"  -l, --list                 list all model numbers and exit\n"
 	"  -u, --dump-caps            dump capabilities and exit\n"
-	"  -e, --end-marker           use END marker in rotctld protocol (obsolete)\n"
 	"  -v, --verbose              set verbose mode, cumulative\n"
 	"  -h, --help                 display this help and exit\n"
 	"  -V, --version              output version information and exit\n\n",
@@ -471,4 +468,3 @@ void usage()
 
 	printf("\nReport bugs to <hamlib-developer@lists.sourceforge.net>.\n");
 }
-
