@@ -1,5 +1,5 @@
 
-/* 
+/*
  * Very simple test program to check BCD convertion against some other --SF
  * This is mainly to test freq2bcd and bcd2freq functions.
  */
@@ -33,14 +33,14 @@ int main (int argc, char *argv[])
 		if (digits > MAXDIGITS)
 			exit(1);
 	}
-	
+
 	printf("Little Endian mode\n");
 	printf("Frequency: %"PRIfreq"\n",f);
 	to_bcd(b, f, digits);
 	printf("BCD: %2.2x",b[0]);
 	for (i = 1; i < (digits+1)/2; i++)
 		printf(",%2.2x",b[i]);
-	printf("\nResult after recoding: %"SCNll"\n", from_bcd(b, digits));
+	printf("\nResult after recoding: %"PRIll"\n", from_bcd(b, digits));
 
 	printf("\nBig Endian mode\n");
 	printf("Frequency: %"PRIfreq"\n",f);
@@ -48,7 +48,7 @@ int main (int argc, char *argv[])
 	printf("BCD: %2.2x",b[0]);
 	for (i = 1; i < (digits+1)/2; i++)
 		printf(",%2.2x",b[i]);
-	printf("\nResult after recoding: %"SCNll"\n", from_bcd_be(b, digits));
+	printf("\nResult after recoding: %"PRIll"\n", from_bcd_be(b, digits));
 
 	return 0;
 }
