@@ -3,7 +3,6 @@
 # testctld.pl - (C) 2008,2010 Nate Bargmann, n0nb@arrl.net
 # A Perl test script for the rigctld program.
 #
-#  $Id$
 #
 # It connects to the rigctld TCP port (default 4532) and queries the daemon
 # for some common rig information and sets some values.  It also aims to
@@ -13,19 +12,19 @@
 # response mode.  See the rigctld(8) man page for details.
 
 #############################################################################
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; either version 2
-# of the License, or (at your option) any later version.
+#    This program is free software; you can redistribute it and/or modify
+#    it under the terms of the GNU General Public License as published by
+#    the Free Software Foundation; either version 2 of the License, or
+#    (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+#    You should have received a copy of the GNU General Public License along
+#    with this program; if not, write to the Free Software Foundation, Inc.,
+#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 # See the file 'COPYING' in the main Hamlib distribution directory for the
 # complete text of the GNU Public License version 2.
@@ -56,7 +55,7 @@ my $ret_val;
 
 # Error values returned from rigctld by Hamlib name
 my %errstr = (
-    RIG_OK          => "0",     # No error, operation completed sucessfully
+    RIG_OK          => "0",     # No error, operation completed successfully
     RIG_EINVAL      => "-1",    # invalid parameter
     RIG_ECONF       => "-2",    # invalid configuration (serial,..)
     RIG_ENOMEM      => "-3",    # memory shortage
