@@ -43,12 +43,29 @@
 #include <string.h>     /* String function definitions */
 #include <unistd.h>     /* UNIX standard function definitions */
 
-#include "hamlib/rig.h"
+#include <hamlib/rig.h>
 #include "serial.h"
 #include "misc.h"
 #include "yaesu.h"
 #include "ft757gx.h"
 
+/* HAVE_SSLEEP is defined when Windows Sleep is found
+ * HAVE_SLEEP is defined when POSIX sleep is found
+ * _WIN32 is defined when compiling with MinGW
+ *
+ * When cross-compiling from POSIX to Windows using MinGW, HAVE_SLEEP
+ * will often be defined by configure although it is not supported by
+ * MinGW.  So substitute the sleep definition below in such a case and
+ * when compiling on Windows using MinGW where HAVE_SLEEP will be
+ * undefined.
+ *
+ * FIXME:  Needs better handling for all versions of MinGW.
+ *
+ */
+#if (defined(HAVE_SSLEEP) || defined(_WIN32)) && (!defined(HAVE_SLEEP))
+#include "hl_sleep.h"
+#endif
+
 
 /* Private helper function prototypes */
 static int ft757_get_update_data(RIG *rig);
@@ -299,7 +316,7 @@ int ft757_init(RIG *rig)
     if (!rig)
         return -RIG_EINVAL;
 
-    p = (struct ft757_priv_data *)malloc(sizeof(struct ft757_priv_data));
+    p = (struct ft757_priv_data * ) calloc(1, sizeof(struct ft757_priv_data));
     if (!p)         /* whoops! memory shortage! */
         return -RIG_ENOMEM;
 
@@ -720,4 +737,3 @@ int rig2mode(RIG *rig, int md, rmode_t *mode, pbwidth_t *width)
 
     return RIG_OK;
 }
-
