@@ -346,7 +346,7 @@ static int ft840_init(RIG *rig) {
   if (!rig)
     return -RIG_EINVAL;
 
-  priv = (struct ft840_priv_data *)malloc(sizeof(struct ft840_priv_data));
+  priv = (struct ft840_priv_data *) calloc(1, sizeof(struct ft840_priv_data));
   if (!priv)                       /* whoops! memory shortage! */
     return -RIG_ENOMEM;
 
