@@ -367,7 +367,7 @@ int ft847_init(RIG *rig) {
   if (!rig)
     return -RIG_EINVAL;
 
-  p = (struct ft847_priv_data*)malloc(sizeof(struct ft847_priv_data));
+  p = (struct ft847_priv_data *) calloc(1, sizeof(struct ft847_priv_data));
   if (!p) {
 				/* whoops! memory shortage! */
     return -RIG_ENOMEM;
