@@ -57,6 +57,7 @@
 #include "config.h"
 #endif
 
+#include <math.h>
 #include <stdlib.h>
 #include <string.h>  	/* String function definitions */
 #include <unistd.h>  	/* UNIX standard function definitions */
@@ -149,7 +150,7 @@ const struct rig_caps ft857_caps = {
   .rig_model = 		RIG_MODEL_FT857,
   .model_name = 	"FT-857",
   .mfg_name = 		"Yaesu",
-  .version = 		"0.4",
+  .version = 		"0.5",
   .copyright = 		"LGPL",
   .status = 		RIG_STATUS_BETA,
   .rig_type = 		RIG_TYPE_TRANSCEIVER,
@@ -474,7 +475,7 @@ int ft857_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 
   *freq = from_bcd_be(p->fm_status, 8) * 10;
 
-  return -RIG_OK;
+  return RIG_OK;
 }
 
 int ft857_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
@@ -582,10 +583,14 @@ static int ft857_get_pometer_level(RIG *rig, value_t *val)
       return n;
 
   /* Valid only if PTT is on */
-  if ((p->tx_status & 0x80) == 0)
-    val->f = ((p->tx_status & 0x0F) / 15.0);
+  if ((p->tx_status & 0x80) == 0) {
+    // convert watts to dBm
+    val->i = 10 * log10(p->tx_status & 0x0F) + 30;
+    // now convert to db over S9
+    val->i += 73;
+  }
   else
-    val->f = 0.0;
+    val->i = -911; // invalid value return
 
   return RIG_OK;
 }
@@ -599,9 +604,7 @@ static int ft857_get_smeter_level(RIG *rig, value_t *val)
     if ((n = ft857_get_status(rig, FT857_NATIVE_CAT_GET_RX_STATUS)) < 0)
       return n;
 
-  n = (p->rx_status & 0x0F) - 9;
-
-  val->i = n * ((n > 0) ? 10 : 6);
+  val->i = n = (p->rx_status & 0x0F)  * 6 - 20;  // Convert S level to dB
 
   return RIG_OK;
 }
@@ -719,6 +722,8 @@ int ft857_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
   /* fill in the frequency */
   to_bcd_be(data, (freq + 5) / 10, 8);
 
+  rig_force_cache_timeout(&((struct ft857_priv_data *) rig->state.priv)->fm_status_tv);
+
   return ft857_send_icmd(rig, FT857_NATIVE_CAT_SET_FREQ, data);
 }
 
@@ -766,6 +771,8 @@ int ft857_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
   if (width != RIG_PASSBAND_NORMAL)
     return -RIG_EINVAL;
 
+  rig_force_cache_timeout(&((struct ft857_priv_data *) rig->state.priv)->fm_status_tv);
+
   return ft857_send_cmd(rig, index);
 }
 
@@ -791,6 +798,8 @@ int ft857_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t tx_vfo)
 
   n = ft857_send_cmd(rig, index);
 
+  rig_force_cache_timeout(&((struct ft857_priv_data *) rig->state.priv)->tx_status_tv);
+
   if (n < 0 && n != -RIG_ERJCTED)
     return n;
 
@@ -819,6 +828,8 @@ int ft857_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
 
   n = ft857_send_cmd(rig, index);
 
+  rig_force_cache_timeout(&((struct ft857_priv_data *) rig->state.priv)->tx_status_tv);
+
   if (n < 0 && n != -RIG_ERJCTED)
     return n;
 
