@@ -65,7 +65,7 @@ const struct rig_caps ft950_caps = {
     .write_delay =        FT950_WRITE_DELAY,
     .post_write_delay =   FT950_POST_WRITE_DELAY,
     .timeout =            2000,
-    .retry =              0,
+    .retry =              3,
     .has_get_func =       FT950_FUNCS,
     .has_set_func =       FT950_FUNCS,
     .has_get_level =      FT950_LEVELS,
@@ -85,7 +85,7 @@ const struct rig_caps ft950_caps = {
     .max_ifshift =        Hz(1000),
     .vfo_ops =            FT950_VFO_OPS,
     .targetable_vfo =     RIG_TARGETABLE_FREQ,
-    .transceive =         RIG_TRN_RIG,        /* May enable later as the 950 has an Auto Info command */
+    .transceive =         RIG_TRN_OFF,        /* May enable later as the 950 has an Auto Info command */
     .bank_qty =           0,
     .chan_desc_sz =       0,
     .str_cal =            FT950_STR_CAL,
