@@ -4,21 +4,20 @@
  * This shared library provides an API for communicating
  * via serial interface to an VR-5000 using the "CAT" interface
  *
- *  $Id: vr5000.c,v 1.3 2006-10-07 15:51:38 csete Exp $
  *
- *   This library is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2 of
- *   the License, or (at your option) any later version.
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License as published by the Free Software Foundation; either
+ *   version 2.1 of the License, or (at your option) any later version.
  *
- *   This program is distributed in the hope that it will be useful,
+ *   This library is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU Library General Public License for more details.
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *   Lesser General Public License for more details.
  *
- *   You should have received a copy of the GNU Library General Public
+ *   You should have received a copy of the GNU Lesser General Public
  *   License along with this library; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -243,7 +242,7 @@ int vr5000_init(RIG *rig)
 {
   struct vr5000_priv_data *priv;
 
-  priv = (struct vr5000_priv_data*)malloc(sizeof(struct vr5000_priv_data));
+  priv = (struct vr5000_priv_data *) calloc(1, sizeof(struct vr5000_priv_data));
   if (!priv)  return -RIG_ENOMEM;
 
   rig->state.priv = (void*)priv;
